/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

/** Represents one module dependency
 *
 * @author  rnovak
 * @version 1.0
 */
public class Dependency extends Object {

    public String type;
    public String module;
    public String version = null;
    public String condition = null;
    public String language = "en";
    public static String EQUAL = "Equals";
    public static String GREATER = "GreaterThan";
    public static String GREATER_EQUAL = "GreaterThanAndEqualTo";
    public static String LOWER = "LowerThan";
    public static String LOWER_EQUAL = "LowerThanAndEqualTo";

    /**
     * Creates Dependency object from dependency string and type
     */
    public Dependency(String depString, String type, String language) {
        this.type = type;
        this.language = language;

        int i;
        int flag = 0;
        int n = depString.length();
        char[] dep = depString.toCharArray();

        StringBuffer version_tmp = new StringBuffer();
        StringBuffer cond_tmp = new StringBuffer();


        for (i = n - 1; i >= 0; i--) {
            if ((Character.isDigit(dep[i]) || dep[i] == '.') && flag == 0) {
                version_tmp.append(dep[i]);
            } else if (dep[i] == '>' || dep[i] == '<' || dep[i] == '=') {
                cond_tmp.append(dep[i]);
                flag = 1;
            } else {
                break;
            }
        }

        version_tmp.reverse();
        cond_tmp.reverse();

        if (cond_tmp.length() > 0) {
            depString = depString.substring(0, i + 1);
            version = version_tmp.toString();

            if (cond_tmp.toString().compareTo("=") == 0) {
                condition = EQUAL;
            }
            if (cond_tmp.toString().compareTo(">") == 0) {
                condition = GREATER;
            }
            if (cond_tmp.toString().compareTo(">=") == 0) {
                condition = GREATER_EQUAL;
            }
            if (cond_tmp.toString().compareTo("<") == 0) {
                condition = LOWER;
            }
            if (cond_tmp.toString().compareTo("<=") == 0) {
                condition = LOWER_EQUAL;
            }
        }

        module = depString.toString();
        if (type.compareTo("module_dependency") == 0) {
            applyModuleSetup();
        }
    }

    /**
     * Sets module params according to External Parameters setup file
     */
    private void applyModuleSetup() {
        String osName;
        String ver = this.version;

        if (ver == null) {
            ver = "*";
        }

        if (language.compareTo("en") != 0) {
            module += "_" + language;
        }

        osName = ModulesSetup.getParam(module, ver, Module.OS_NAME);
        if (osName != null) {
            if (language.compareTo("en") == 0) {
                osName = language + "_" + osName;
            }
            module += "_" + osName;
        }
    }
}
    