/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

/** 
 * Utilities for XML documents handling
 *
 */
public class XMLutils {

    /**
     * Creates new XMLutils object
     */
    private XMLutils() {
    }

    /**
     * Opens XML document from file xml_file
     */
    public static Document openXML(String xmlFile) {

        Document document = null;
        DocumentBuilder builder;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setValidating(false);
        factory.setNamespaceAware(false);

        try {
            builder = factory.newDocumentBuilder();

            builder.setEntityResolver(AuEntityResolver.getInstance());
            document = builder.parse(xmlFile);
        } catch (SAXException se) {
            System.out.println(se);
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException ioe) {
            System.out.println(ioe);
        } catch (ParserConfigurationException pce) {
            System.out.println(pce);
        }

        return (document);
    }

    /**
     * Opens XML document from file xml_file
     */
    public static Document openXML(File xmlFile) {

        Document document = null;
        DocumentBuilder builder;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setValidating(false);
        factory.setNamespaceAware(false);

        try {
            builder = factory.newDocumentBuilder();

            builder.setEntityResolver(AuEntityResolver.getInstance());
            document = builder.parse(xmlFile);
        } catch (SAXException se) {
            System.out.println(se);
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException ioe) {
            System.out.println(ioe);
        } catch (ParserConfigurationException pce) {
            System.out.println(pce);
        }

        return (document);
    }

    /**
     * Opens XML document from input stream xml_file_stream
     */
    public static Document openXML(InputStream xmlFileStream) {

        Document document = null;
        DocumentBuilder builder;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setValidating(false);
        factory.setNamespaceAware(false);

        try {
            builder = factory.newDocumentBuilder();

            builder.setEntityResolver(AuEntityResolver.getInstance());
            document = builder.parse(xmlFileStream);
        } catch (SAXException se) {
            System.out.println(se);
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException ioe) {
            System.out.println(ioe);
        } catch (ParserConfigurationException pce) {
            System.out.println(pce);
        }

        return (document);
    }

    /**
     * Opens XML document from input stream xml_file_stream with DTD file dtd_file
     */
    public static Document openXML(InputStream xmlFileStream, String dtdFile) {

        Document document = null;
        DocumentBuilder builder;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setValidating(false);
        factory.setNamespaceAware(false);

        try {
            builder = factory.newDocumentBuilder();

            builder.setEntityResolver(AuEntityResolver.getInstance());
            document = builder.parse(xmlFileStream);
        } catch (SAXException se) {
            System.out.println(se);
        } catch (FileNotFoundException e) {
            System.out.println(e);
        } catch (IOException ioe) {
            System.out.println(ioe);
        } catch (ParserConfigurationException pce) {
            System.out.println(pce);
        }

        return (document);
    }

    /** 
     *  Creates new empty XML document
     */
    public static Document newXML() {
        Document document = null;
        DocumentBuilder builder;
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        factory.setValidating(false);
        factory.setNamespaceAware(true);

        try {
            builder = factory.newDocumentBuilder();
            document = builder.newDocument();
        } catch (ParserConfigurationException pce) {
            System.out.println(pce);
        }

        return (document);
    }

    /**
     * Transforms XML document doc 
     */
    public static void XMLtransform(Document doc) {
        TransformerFactory tfactory = TransformerFactory.newInstance();
        Properties oprops = new Properties();
        DOMSource dom_source = new DOMSource(doc);
        File testFile = new File("test_out.xml");
        try {
            Transformer serializer = tfactory.newTransformer();
            serializer.setOutputProperties(oprops);
            serializer.transform(dom_source, new StreamResult(testFile));
        } catch (Exception e) {
            System.out.println(e);
        }

    }

    /**
     * Transforms XML document doc to output file out_file
     */
    public static void XMLtransform(Document doc, String outFile) {
        TransformerFactory tfactory = TransformerFactory.newInstance();
        Properties oprops = new Properties();
        DOMSource dom_source = new DOMSource(doc);
        File resultFile = new File(outFile);

        try {
            Transformer serializer = tfactory.newTransformer();
            serializer.setOutputProperties(oprops);
            serializer.transform(dom_source, new StreamResult(resultFile));
        } catch (Exception e) {
            System.out.println("Transformation error: " + e);
        }

    }

    /**
     * Transforms XML document doc to output file out_file using template 
     * from file template_file 
     */
    public static void XMLtransform(Document doc, String outFile, String templateFile) {
        TransformerFactory tfactory = TransformerFactory.newInstance();

        if (tfactory.getFeature(DOMSource.FEATURE) && tfactory.getFeature(DOMResult.FEATURE)) {
            DocumentBuilderFactory dfactory = DocumentBuilderFactory.newInstance();
            dfactory.setNamespaceAware(true);
            Properties oprops = new Properties();
            try {
                DocumentBuilder docBuilder = dfactory.newDocumentBuilder();
                Node xslDOM = docBuilder.parse(templateFile);
                DOMSource dsource = new DOMSource(xslDOM);
                Templates templates = tfactory.newTemplates(dsource);
                File testFile = new File(outFile);
                Transformer transformer = templates.newTransformer();
                transformer.setOutputProperties(oprops);
                transformer.transform(new DOMSource(doc), new StreamResult(testFile));
            } catch (Exception e) {
                throw new IllegalStateException("Trannsformation failed.", e);
            }
        }
    }
}
    