/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml.web;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.netbeans.auxml.UpdateCenter;
import org.netbeans.auxml.Version;
import org.netbeans.auxml.service.StageModuleForm;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.view.RedirectView;
import static org.netbeans.auxml.web.Config.getManager;

/**
 *
 * @author mschovanek@netbeans.org
 */
public class StageModuleFormController extends SimpleFormController {

    private String sel_version = null;
    private String reloadForm = null;
    
    /** Logger for this class and subclasses */
    protected final Logger LOG = Logger.getLogger(StageModuleFormController.class.getName());
    
    public String getSel_version() {
        return(sel_version);
    }

    public void setSel_version(String sel_version) {
        this.sel_version = sel_version;
    }
    
    public String getReloadForm() {
        return(reloadForm);
    }

    public void setReloadForm(String reloadForm) {
        this.reloadForm = reloadForm;
    }    

    @Override
    public ModelAndView onSubmit(Object command) throws ServletException, IOException {
        StageModuleForm form = (StageModuleForm) command;
        ModelAndView model;
        
        MultipartFile mf = form.getFile();              
        sel_version = form.getVersion();
        String type = form.getType();
        if ( type.equals("type") || mf.isEmpty() ) {
            model = new ModelAndView(new RedirectView(getReloadForm()));
            return(model);
        }
        
        File workDir = null;
        workDir = File.createTempFile("auxml_ ", null).getParentFile();
        if (workDir == null || !workDir.canWrite()) {
            throw new IllegalStateException("Cannot get a writeble workdir.");
        }
        String origName = new File(mf.getOriginalFilename()).getName();
        File tmpFile = new File(workDir, origName);
        mf.transferTo(tmpFile);
        getManager().stageModule(form.getVersion(), form.getType(), form.getDir(), tmpFile);
        return new ModelAndView(new RedirectView(getSuccessView()));
    }

    @Override
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, Exception {
        ModelAndView model = super.handleRequest(request, response);
        Map map = model.getModel();
        
        List<Version> versions = getManager().getVersions();
        List<String> verNames = new ArrayList<String>(versions.size());
        List<String> typeSet = new ArrayList<String>();
        
        Version sel_one_version = versions.get(0);
        String one_verName;
        for (Version one_version : versions) {
            one_verName = one_version.getName();
            verNames.add(one_verName);
            if (one_verName.equals(sel_version))
                sel_one_version = one_version;
        }

        for (UpdateCenter uc : sel_one_version.getUpdateCenters()) {
                typeSet.add(uc.getType());
        }
        
        map.put("versions", verNames);
        map.put("types", typeSet);
        return model;
    }

    @Override
    protected Object formBackingObject(HttpServletRequest request) throws ServletException {
        StageModuleForm form = new StageModuleForm(sel_version, "type", null);
        return form;
    }
}