/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.config.Config;
import org.netbeans.modules.coverage.utils.ModuleActivityModes;
import org.netbeans.modules.coverage.utils.ProjectStructureUtils;
import org.openide.awt.Actions;
import org.openide.awt.Mnemonics;
import org.openide.nodes.Node;
import org.openide.util.ContextAwareAction;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import org.openide.util.actions.Presenter;
import org.openide.util.actions.SystemAction;

/**
 * The action that holds the Coverage Plugin On/Off actions
 * The action is attached to the project node
 */
public final class ProjectMenuAction extends CookieAction implements ContextAwareAction {

    private static final String name = NbBundle.getMessage(ProjectMenuAction.class, "CTL_Coverage_Menu");

    
    
    protected void performAction(Node[] activatedNodes) {
    // do nothing, this is a popup menu
    }

    public Action createContextAwareInstance(Lookup actionContext) {
        return new ProjectContextAwareMenuAction(actionContext);
    }

    protected int mode() {
        return CookieAction.MODE_EXACTLY_ONE;
    }
    
    @Override
    protected boolean enable(Node[] activatedNodes){
        return false;
    }

    public String getName() {
        return name;
    }

    public String iconResource() {
        return null;
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    protected Class[] cookieClasses() {
        
        return new Class[]{
            Project.class
        };
    }

    protected boolean asynchronous() {
        return false;
    }

    private static class ProjectContextAwareMenuAction extends AbstractAction implements Presenter.Popup {

        private Lookup context;

        public ProjectContextAwareMenuAction(Lookup actionContext) {
            super(name);
            this.context = actionContext;

        }

       
        public JMenuItem getPopupPresenter() {
            return new ProjectMenuItems();
        }

        public void actionPerformed(ActionEvent e) {
        // do nothing, this is a popup menu
        }

        private class ProjectMenuItems extends JMenu {

            private boolean popupContructed;
            static final long serialVersionUID = 1L;

            public ProjectMenuItems() {
                Mnemonics.setLocalizedText(this, name);
                Node[] nodes = (Node[]) context.lookup(new Lookup.Template(Node.class)).allInstances().toArray(new Node[0]);
                if (nodes.length != 1){
                    this.setEnabled(false);
                }

                Project project = ProjectStructureUtils.selectProjectByName(nodes[0].getDisplayName());
                if (project == null || !Config.isSupportedProject(project)){
                    setEnabled(false);
                }
                

            }

            /**
             * Initializes the sub actions and turns one of them off according to the coverage module
             * state.
             */
            public void setSelected(boolean selected) {

                if (selected && popupContructed == false) {
                    // lazy submenu construction
                    Action[] actions = getActions();

                    for (int i = 0; i < actions.length; i++) {
                        Action action = actions[i];
                        if (action == null) {
                            add(new JSeparator());    // workaround openide bug
                        } else {
                            JMenuItem item = new JMenuItem();
                            Actions.connect(item, actions[i], true);
                            Mnemonics.setLocalizedText(item, item.getText());
                            add(item);
                        }
                    }
                    popupContructed = true;
                }
                super.setSelected(selected);

            }

            /**
             * Initializes the sub actions (the actions that enable/disable coverage collection) according 
             *to the project state.
             * @return Returns the array of properly initialised subactions. The array is never null.
             */
            private Action[] getActions() {
                Node[] nodes = (Node[]) context.lookup(new Lookup.Template(Node.class)).allInstances().toArray(new Node[0]);

                Project project = ProjectStructureUtils.selectProjectByName(nodes[0].getDisplayName());

                Action _ProjectCoverageActivityOnAction = SystemAction.get(ProjectCoverageActivityOnAction.class);
                Action _ProjectCoverageActivityOffAction = SystemAction.get(ProjectCoverageActivityOffAction.class);
                Action _ProjectShowStatisticsAction = SystemAction.get(ProjectShowStatisticsAction.class);

                if (ModuleActivityModes.isModuleActive(project)) {
                    _ProjectCoverageActivityOnAction.setEnabled(false);
                    _ProjectCoverageActivityOffAction.setEnabled(true);
                    _ProjectShowStatisticsAction.setEnabled(true);
                } else {
                    _ProjectCoverageActivityOnAction.setEnabled(true);
                    _ProjectCoverageActivityOffAction.setEnabled(false);
                    _ProjectShowStatisticsAction.setEnabled(false);
                }
                Action[] actions = new Action[]{
                    _ProjectCoverageActivityOnAction,
                    _ProjectCoverageActivityOffAction,
                    null,
                    _ProjectShowStatisticsAction
                };
                return actions;
            }
        }
    }
}

