/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.annotations;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import javax.swing.text.StyledDocument;
import org.netbeans.modules.coverage.utils.LineCoverageStates;
import org.openide.cookies.EditorCookie;
import org.openide.text.Line;
import org.openide.text.Line.Set;
import org.openide.text.NbDocument;

/**
 *The class is used to color sources according to the coverage data
 **/
public class DocumentColorer {
    
    /** Creates a new instance of DocumentColorer */
    public DocumentColorer() {
    }
    
    /**
     * colors all the document without writing per-line testcase info
     */
    public static ArrayList<String> colorDoc(String projectName, String className, EditorCookie editorCookie, Hashtable<Integer, LineCoverageStates> coverages){
        return colorDoc(projectName, className, editorCookie, coverages, null);
    }
    
    /**
     * colors all the document and adds per-line testcase info
     */
    public static ArrayList<String> colorDoc(String projectName, String className, EditorCookie editorCookie, Hashtable<Integer, LineCoverageStates> coverages, Hashtable<Integer, String> lineInfo){
        //editorCookie.open();
        StyledDocument doc = editorCookie.getDocument();
        if (doc==null){
            return new ArrayList<String>();
        }
        int startLine = 0;
        int endLine = NbDocument.findLineNumber(doc, doc.getLength());
        return colorDoc(projectName, className, editorCookie, startLine, endLine, coverages);
    }
    
    /**
     * colors specified lines of the document without writing per-line testcase info
     */
    public static ArrayList<String> colorDoc(String projectName, String className, EditorCookie editorCookie, int startLine, int endLine, Hashtable<Integer, LineCoverageStates> coverages){
        return colorDoc(projectName, className, editorCookie, startLine, endLine, coverages, null);
    }
    
    /**
     * colors specified lines of the document and adds per-line testcase info
     */
    public static ArrayList<String> colorDoc(String projectName, String className, EditorCookie editorCookie, int startLine, int endLine, Hashtable<Integer, LineCoverageStates> coverages, Hashtable<Integer, String> lineInfo){
        //editorCookie.open();
        Set lineset = editorCookie.getLineSet();
        ArrayList<String> coveredButAbsentLines = new ArrayList();
        
        Integer currLine;
        for (Iterator<Integer> it = coverages.keySet().iterator(); it.hasNext();){
            currLine = it.next();
            if (currLine.intValue() >= startLine && currLine.intValue() <= endLine){
                Line line = lineset.getOriginal( currLine.intValue() );
                // let's check if line has 0 annotations here and then
                // use attach() instead of attachLonely();
                if (line.getAnnotationCount() == 0){
                    AbstractCoverageAnnotation annotation = new CoverageAnnotation(coverages.get(currLine), projectName, className, currLine);
                    if (lineInfo != null && lineInfo.containsKey(currLine))
                        annotation.setMessage( lineInfo.get(currLine) );
                    // while we have already checked if line has 0 annotations,
                    // use attach() instead of attachLonely();
                    //annotation.attachLonely( line );
                    annotation.attach( line );
                    
                    line.addPropertyChangeListener(annotation);
                }
            } else {
                coveredButAbsentLines.add(String.valueOf(currLine));
            }
            
        }// for
        //editorCookie.open();
        return coveredButAbsentLines;
    }
    
}
