/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.instrumentation;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;

/**
 *
 * @author apepin
 */
public class TextFileBuilder {
    private FileObject file = null;
    private List<String> text = new LinkedList<String>();
    /** Creates a new instance of TextFileBuilder */
    public TextFileBuilder(FileObject file) {
        this.file = file;
    }
    
    public void load(){
        if(file != null){
            try {
                InputStream stream = file.getInputStream();
                //read file
                read(stream);
                stream.close();
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public void save(){
        if(file != null){
            FileLock lock = null;
            try {
                //System.out.println("start saving to "+file.getName()+"."+file.getExt());
                lock = file.lock();
                OutputStream stream = file.getOutputStream(lock);
                //write file
                write(stream);
                stream.flush();
                stream.close();
                //System.out.println("finish saving to "+file.getName()+"."+file.getExt());
            } catch (IOException ex) {
                ex.printStackTrace();
            } finally{
                if(lock != null && lock.isValid()){
                    lock.releaseLock();
                }
            }
        }
    }
    
    public void reload(){
        text = new LinkedList<String>();
        load();
    }
    
    private void read(InputStream stream){
        InputStreamReader reader = new InputStreamReader(stream);
        read(reader, text);
    }
    
    private void read(Reader reader, List txt){
        int simbol;
        StringBuilder line = new StringBuilder();
        try {
            while((simbol=reader.read()) != -1){
                line.append((char)simbol);
                //pick out a string
                if(simbol == '\n'){
                    txt.add(line.toString());
                    line = new StringBuilder();
                }
            }
            if(line.length()>0)
                txt.add(line.toString());
            
            reader.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        
    }
    
    private void write(OutputStream stream){
        OutputStreamWriter writer = new OutputStreamWriter(stream);
        try {
            for(String line: text){
                writer.write(line);
            }
            writer.flush();
            writer.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    public boolean appendLine(String txt, int line){
        boolean result = false;
        try{
            text.add(line, txt+"\n");
            result = true;
        } catch(Exception e){}
        return result;
    }
    
    public boolean appendText(String txt, int line){
        List<String> lines = new ArrayList<String>();
        read(new StringReader(txt), lines);
        //System.out.println("insert text at line "+line);
        return text.addAll(line, lines);
    }
    
    public boolean appendText(TextFileBuilder builder, int line){
        return text.addAll(line, builder.text);
    }
    
    
    public boolean appendLine(String txt){
        return text.add(txt+"\n");
    }
    
    public void removeLine(int line){
        text.remove(line);
    }
    
    public boolean replaceLine(int line, String newText){
        text.remove(line);
        return appendLine(newText, line);
    }
    
    public int findStringStartsWith(String prefix){
        int index = 0;
        boolean res = false;
        for(String line: text){
//            System.out.println("line "+index+" :"+line);
            if(res = line.trim().startsWith(prefix))
                break;
            index++;
        }
        return res ? index: -1;
    }
    
    public int findLineStartsWith(String prefix, int fromLine){
        int index = 0;
        boolean res = false;
        for(String line: text){
//            System.out.println("line "+index+" :"+line);
            if(index>=fromLine){
                if(res = line.trim().startsWith(prefix))
                    break;
            }
            index++;
        }
        return res ? index: -1;
    }
    
    public String getLine(int line){
        return text.get(line).replace("\n", "").replace("\r", "");
    }
}
