/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

/*
* ProjectUtils.java
  */

package org.netbeans.modules.coverage.utils;
import org.netbeans.api.java.classpath.ClassPath;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectInformation;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.modules.coverage.ui.CommonDialogsCollection;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.netbeans.api.java.source.JavaSource;
import org.netbeans.api.java.source.ClasspathInfo;
/**
* Temporary class to store helpfull methods
* @author  akorostelev
*/
public class ProjectStructureUtils {


    /**
     * Returns full name of the class thais associated witha node
     * @param node Node to find class name for
     * @return Full class name or null if could not determine
     */
    public static String getClassFullNameByNode(Node node){
        String result = null;
        if(node != null) {
            DataObject dob = node.getCookie(DataObject.class);
            FileObject fo = dob.getPrimaryFile();
            JavaSource js = JavaSource.forFileObject(fo);
            if(js != null) {
                ClasspathInfo cpi = js.getClasspathInfo();
                ClassPath cp = cpi.getClassPath (ClasspathInfo.PathKind.SOURCE);
                if(cp != null) {
                    return cp.getResourceName(fo,'.',false);
                }
            }
        }
        return(result);
    }




    /**
     * Returns opened project associated with the given name
     * @param pName Project name
     * @return Project object
     */
    public static Project selectProjectByName(String pName){
        Project project = null;
        Project[] projects = OpenProjects.getDefault ().getOpenProjects();
        ProjectInformation pInfo = null;
        for (int i = 0; i < projects.length; i++){
            pInfo = ProjectUtils.getInformation(projects[i]);
            // TEMP action.
            // getDisplayName used for modules in nb5.0
            // getName will work for java project in nb5.0
            // and java project/nb module in nb5.5
            if ( pInfo.getDisplayName().equalsIgnoreCase(pName)
            || pName.equalsIgnoreCase(pInfo.getDisplayName()+" [Main]")){
                project = projects[i];
                break;
            }
        }
        return project;
    }

    /**
     * returns Project object tha contains specified node
     * @param node Node to get project for
     * @return Enclosing project
     */
    public static Project getProjectForNode(Node node){
        DataObject cc = (DataObject)node.getCookie(DataObject.class);
        Project project = FileOwnerQuery.getOwner(cc.getPrimaryFile());
        if (project == null){
            CommonDialogsCollection.userError("Unsapported type of  project:"+ ProjectUtils.getInformation(project).getDisplayName()+".  This module works only with java projects.");
        }
        return project;
    }



}
