/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;

public class AccessFlag {

	int flag;
	int type;
	String str;
	
	AccessFlag (int t) {
		type = t;
		flag = 0;
	}

	public void fromClass (DataInputStream dis) throws Exception {
		flag = dis.readUnsignedShort ();
		createStr ();
	}
	
	public void fromJRAW (Lexan lex) throws Exception {
		lex.cmpLexel ("AccessFlag");
		lex.cmpLexel ("{");
		flag = 0;
		for (;;) {
			String s = lex.getLexel ();
			lex.cmpLexel ();
			if ("}".equals (s))
				break;
			switch (type) {
				case 0:
					if ("ACC_PUBLIC".equals (s))
						flag |= 0x0001;
					else if ("ACC_FINAL".equals (s))
						flag |= 0x0010;
					else if ("ACC_SUPER".equals (s))
						flag |= 0x0020;
					else if ("ACC_INTERFACE".equals (s))
						flag |= 0x0200;
					else if ("ACC_ABSTRACT".equals (s))
						flag |= 0x0400;
					else
						throw new ParserException ("Unknown flag: " + s);
					break;
				case 1:
					if ("ACC_PUBLIC".equals (s))
						flag |= 0x0001;
					else if ("ACC_PRIVATE".equals (s))
						flag |= 0x0002;
					else if ("ACC_PROTECTED".equals (s))
						flag |= 0x0004;
					else if ("ACC_STATIC".equals (s))
						flag |= 0x0008;
					else if ("ACC_FINAL".equals (s))
						flag |= 0x0010;
					else if ("ACC_VOLATIVE".equals (s))
						flag |= 0x0040;
					else if ("ACC_TRANSIENT".equals (s))
						flag |= 0x0080;
					else
						throw new ParserException ("Unknown flag: " + s);
					break;
				case 2:
					if ("ACC_PUBLIC".equals (s))
						flag |= 0x0001;
					else if ("ACC_PRIVATE".equals (s))
						flag |= 0x0002;
					else if ("ACC_PROTECTED".equals (s))
						flag |= 0x0004;
					else if ("ACC_STATIC".equals (s))
						flag |= 0x0008;
					else if ("ACC_FINAL".equals (s))
						flag |= 0x0010;
					else if ("ACC_SYNCHRONIZED".equals (s))
						flag |= 0x0020;
					else if ("ACC_NATIVE".equals (s))
						flag |= 0x0100;
					else if ("ACC_ABSTRACT".equals (s))
						flag |= 0x0400;
					else if ("ACC_STRICT".equals (s))
						flag |= 0x0800;
					else
						throw new ParserException ("Unknown flag: " + s);
					break;
				case 3:
					if ("ACC_PUBLIC".equals (s))
						flag |= 0x0001;
					else if ("ACC_PRIVATE".equals (s))
						flag |= 0x0002;
					else if ("ACC_PROTECTED".equals (s))
						flag |= 0x0004;
					else if ("ACC_STATIC".equals (s))
						flag |= 0x0008;
					else if ("ACC_FINAL".equals (s))
						flag |= 0x0010;
					else if ("ACC_INTERFACE".equals (s))
						flag |= 0x0200;
					else if ("ACC_ABSTRACT".equals (s))
						flag |= 0x0400;
					else
						throw new ParserException ("Unknown flag: " + s);
					break;
			}
		}
	}
	
	public void createStr () {
		str = "AccessFlag {";
		switch (type) {
			case 0:
				if ((flag & 0x0001) != 0)
					str += " ACC_PUBLIC";
				if ((flag & 0x0010) != 0)
					str += " ACC_FINAL";
				if ((flag & 0x0020) != 0)
					str += " ACC_SUPER";
				if ((flag & 0x0200) != 0)
					str += " ACC_INTERFACE";
				if ((flag & 0x0400) != 0)
					str += " ACC_ABSTRACT";
				break;
			case 1:
				if ((flag & 0x0001) != 0)
					str += " ACC_PUBLIC";
				if ((flag & 0x0002) != 0)
					str += " ACC_PRIVATE";
				if ((flag & 0x0004) != 0)
					str += " ACC_PROTECTED";
				if ((flag & 0x0008) != 0)
					str += " ACC_STATIC";
				if ((flag & 0x0010) != 0)
					str += " ACC_FINAL";
				if ((flag & 0x0040) != 0)
					str += " ACC_VOLATIVE";
				if ((flag & 0x0080) != 0)
					str += " ACC_TRANSIENT";
				break;
			case 2:
				if ((flag & 0x0001) != 0)
					str += " ACC_PUBLIC";
				if ((flag & 0x0002) != 0)
					str += " ACC_PRIVATE";
				if ((flag & 0x0004) != 0)
					str += " ACC_PROTECTED";
				if ((flag & 0x0008) != 0)
					str += " ACC_STATIC";
				if ((flag & 0x0010) != 0)
					str += " ACC_FINAL";
				if ((flag & 0x0020) != 0)
					str += " ACC_SYNCHRONIZED";
				if ((flag & 0x0100) != 0)
					str += " ACC_NATIVE";
				if ((flag & 0x0400) != 0)
					str += " ACC_ABSTRACT";
				if ((flag & 0x0800) != 0)
					str += " ACC_STRICT";
				break;
			case 3:
				if ((flag & 0x0001) != 0)
					str += " ACC_PUBLIC";
				if ((flag & 0x0002) != 0)
					str += " ACC_PRIVATE";
				if ((flag & 0x0004) != 0)
					str += " ACC_PROTECTED";
				if ((flag & 0x0008) != 0)
					str += " ACC_STATIC";
				if ((flag & 0x0010) != 0)
					str += " ACC_FINAL";
				if ((flag & 0x0200) != 0)
					str += " ACC_INTERFACE";
				if ((flag & 0x0400) != 0)
					str += " ACC_ABSTRACT";
				break;
		}
		str += " }";
	}
	
	public int getFlag () {
		return flag;
	}
	
	public String toString () {
		return str;
	}

}
