/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;
import java.util.ArrayList;

public class Parser implements Serializable {

	int majorVersion, minorVersion;
	ConstantPool constPool;
	AccessFlag accFlag;
	int iThis;
	int iSuper;
	int countInterfaces;
	int[] interfaces;
	Fields fields;
	Methods methods;
	Attributes attrs;

	void fromClass (DataInputStream dis) throws Exception {
		if (dis.readUnsignedShort () != 0xCAFE  ||  dis.readUnsignedShort () != 0xBABE)
			throw new ParserException ();
		majorVersion = dis.readUnsignedShort ();
		minorVersion = dis.readUnsignedShort ();
		constPool = new ConstantPool (this);
		constPool.fromClass (dis);
		accFlag = new AccessFlag (0);
		accFlag.fromClass (dis);
		iThis = dis.readUnsignedShort ();
		iSuper = dis.readUnsignedShort ();
		countInterfaces = dis.readUnsignedShort ();
		interfaces = new int[countInterfaces];
		for (int a = 0; a < countInterfaces; a ++)
			interfaces[a] = dis.readUnsignedShort ();
		fields = new Fields (this, constPool);
		fields.fromClass (dis);
		methods = new Methods (this, constPool);
		methods.fromClass (dis);
		attrs = new Attributes (this, constPool);
		attrs.fromClass (dis);
	}
	
	void toClass (DataOutputStream dos) throws Exception {
		dos.writeInt (0xCAFEBABE);
		dos.writeShort (majorVersion);
		dos.writeShort (minorVersion);
		constPool.toClass (dos);
		dos.writeShort (accFlag.getFlag ());
		dos.writeShort (iThis);
		dos.writeShort (iSuper);
		dos.writeShort (countInterfaces);
		for (int a = 0; a < countInterfaces; a ++)
			dos.writeShort (interfaces[a]);
		fields.toClass (dos);
		methods.toClass (dos);
		attrs.toClass (dos);
	}
	
	void fromJRAW (Lexan lex) throws Exception {
		lex.cmpLexel ("MajorVersion");
		majorVersion = (int) lex.getLong ();
		lex.cmpLexel ();
		lex.cmpLexel ("MinorVersion");
		minorVersion = (int) lex.getLong ();
		lex.cmpLexel ();
		constPool = new ConstantPool (this);
		constPool.fromJRAW (lex);
		accFlag = new AccessFlag (0);
		accFlag.fromJRAW (lex);
		lex.cmpLexel ("This");
		try {
			iThis = (int) lex.getLong ();
		} catch (Exception e) {
			iThis = constPool.getLabelPos (lex.getLexel ());
		}
		lex.cmpLexel ();
		lex.cmpLexel ("Super");
		try {
			iSuper = (int) lex.getLong ();
		} catch (Exception e) {
			iSuper = constPool.getLabelPos (lex.getLexel ());
		}
		lex.cmpLexel ();
		lex.cmpLexel ("Interfaces");
		lex.cmpLexel ("{");
		ArrayList al = new ArrayList ();
		for (;;) {
			String s = lex.getLexel ();
			if ("}".equals (s))
				break;
			int i;
			try {
				i = (int) lex.getLong ();
			} catch (Exception e) {
				i = constPool.getLabelPos (lex.getLexel ());
			}
			al.add (new Integer (i));
			lex.cmpLexel ();
		}
		countInterfaces = al.size ();
		interfaces = new int[countInterfaces];
		for (int a = 0; a < countInterfaces; a ++) {
			interfaces[a] = ((Integer) al.get (a)).intValue ();
		}
		lex.cmpLexel ();
		fields = new Fields (this, constPool);
		fields.fromJRAW (lex);
		methods = new Methods (this, constPool);
		methods.fromJRAW (lex);
		attrs = new Attributes (this, constPool);
		attrs.fromJRAW (lex);
	}
	
	void toJRAW (ExtStringBuffer esb) {
		esb.println ("MajorVersion " + majorVersion);
		esb.println ("MinorVersion " + minorVersion);
		constPool.toJRAW (esb);
		esb.println (accFlag.toString ());
		esb.println ("This #cp" + iThis);
		esb.println ("Super " + StringUtil.cpName (iSuper));
		esb.println ("Interfaces {");
		esb.incIndent ();
		for (int a = 0; a < countInterfaces; a ++)
			esb.println ("#cp" + interfaces[a]);
		esb.decIndent ();
		esb.println ("}");
		fields.toJRAW (esb);
		methods.toJRAW (esb);
		attrs.toJRAW (esb);
	}
	
	void fromClassFile (String fileName) throws Exception {
		DataInputStream dis = new DataInputStream (new FileInputStream (fileName));
		fromClass (dis);
		dis.close ();
	}

	void toClassFile (String fileName) throws Exception {
		DataOutputStream dos = new DataOutputStream (new FileOutputStream (fileName));
		toClass (dos);
		dos.close ();
	}
	
	void fromJRAWFile (String fileName) throws Exception {
		Lexan lex = new Lexan (new BufferedReader (new FileReader (fileName)));
		fromJRAW (lex);
		lex.close ();
	}

	void toJRAWFile (String fileName) throws Exception {
		FileWriter fw = new FileWriter (fileName);
		ExtStringBuffer esb = new ExtStringBuffer ();
		try {
			toJRAW (esb);
		} catch (Exception e) {
			e.printStackTrace ();
		}
		fw.write (esb.toString ());
		fw.close ();
	}
	
	public static void class2jraw (String inputClass, String outputJRAW) throws Exception {
            Parser parser = new Parser ();
            parser.fromClassFile (inputClass);
            parser.toJRAWFile (outputJRAW);
	}
	
	public static void jraw2class (String inputJRAW, String outputClass) throws Exception {
            Parser parser = new Parser ();
            parser.fromJRAWFile (inputJRAW);
            parser.toClassFile (outputClass);
	}

	public static void class2jrawIO (InputStream is, OutputStream os) throws Exception {
            DataInputStream dis = new DataInputStream (is);
            ExtStringBuffer esb = new ExtStringBuffer ();
            BufferedWriter bw = new BufferedWriter (new OutputStreamWriter (os));
            Parser parser = new Parser ();
            parser.fromClass (dis);
            parser.toJRAW (esb);
            bw.write (esb.toString ());
            try { dis.close (); } catch (Exception e) {}
            try { bw.close (); } catch (Exception e) {}
	}
	
	public static void class2jrawIO (InputStream is, OutputStream os, InputStream source) throws Exception {
            DataInputStream dis = new DataInputStream (is);
            ExtStringBuffer esb = new ExtStringBuffer ();
            BufferedWriter bw = new BufferedWriter (new OutputStreamWriter (os));
            ExtStringArray esa = new ExtStringArray (source);
            Parser parser = new Parser ();
            parser.fromClass (dis);
            parser.constPool.esa = esa;
            parser.toJRAW (esb);
            parser.constPool.esa = null;
            bw.write (esb.toString ());
            try { dis.close (); } catch (Exception e) {}
            try { bw.close (); } catch (Exception e) {}
	}
	
	public static void jraw2classIO (InputStream is, OutputStream os) throws Exception {
            Lexan lex = new Lexan (new BufferedReader (new InputStreamReader (is)));
            DataOutputStream dos = new DataOutputStream (os);
            Parser parser = new Parser ();
            parser.fromJRAW (lex);
            parser.toClass (dos);
            try { lex.close (); } catch (Exception e) {}
            try { dos.close (); } catch (Exception e) {}
	}
	
	public static void main (String[] args) throws Exception {
		switch (args.length) {
		case 1:
			if ("1".equals (args[0]))
				Parser.class2jraw ("Parser.class", "Parser.jraw");
			else if ("2".equals (args[0]))
				Parser.jraw2class ("Parser.jraw", "Parser.class_");
			break;
		case 2:
			if ("1".equals (args[0]))
				Parser.class2jraw (args[1] + ".class", args[1] + ".jraw");
			else if ("2".equals (args[0]))
				Parser.jraw2class (args[1] + ".jraw", args[1] + ".class_");
			break;
		case 3:
			if ("1".equals (args[0]))
				Parser.class2jraw (args[1], args[2]);
			else if ("2".equals (args[0]))
				Parser.jraw2class (args[1], args[2]);
			break;
		default:
			Parser.class2jraw ("Parser.class", "Parser.jraw");
			Parser.jraw2class ("Parser.jraw", "Parser.class_");
		}
	}

}
