/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.settings;

import java.io.IOException;
import org.openide.awt.StatusDisplayer;
import org.openide.execution.ExecInfo;
import org.openide.execution.NbProcessDescriptor;
import org.openide.execution.ProcessExecutor;
import org.openide.execution.ProcessExecutor.Format;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.NbBundle;

public class JadDisassembler extends Disassembler {

    public JadDisassembler() {
    }

    public String getName () {
        return NbBundle.getMessage(JadDisassembler.class, "NAME_JAD"); // NOI18N
    }
    
    public void decompile (FileObject file) {
        ProcessExecutor exec = new ProcessExecutor ();
        exec.setExternalExecutor(new NbProcessDescriptor (
            JASMSettings.getDefault().getJadBinary(),
            "{" + Format.TAG_ARGUMENTS + "}", // NOI18N
            NbBundle.getMessage(JadDisassembler.class, "NAME_JAD_Decompiler") // NOI18N
        ));
        try {
            exec.execute(new ExecInfo (file.getPackageName('.'), new String[] {
                "-d", // NOI18N
                FileUtil.toFile(file.getParent()).getAbsolutePath(),
                FileUtil.toFile(file).getAbsolutePath(),
            })).waitFinished();
            file.getParent().refresh(true);
            renameFile (file, "jad", "java"); // NOI18N
            StatusDisplayer.getDefault().setStatusText(NbBundle.getMessage (JadDisassembler.class, "LBL_Decompile_Finished") + file.getPackageName('.')); // NOI18N
        } catch (IOException e) {
            StatusDisplayer.getDefault().setStatusText(NbBundle.getMessage (JadDisassembler.class, "LBL_Decompile_Error") + file.getPackageName('.')); // NOI18N
        }
    }
    
}
