
package org.netbeans.jsstubsgenerator;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class Zipper {

    private List<String> fileList;
    private final String outputFilePath;
    private final String sourceFolderPath;

    Zipper(String sourceFolderPath, String outputFilePath) {
        this.outputFilePath = outputFilePath;
        this.sourceFolderPath = sourceFolderPath;
        fileList = new ArrayList<String>();
    }

    public void process() {
        generateFileList(new File(sourceFolderPath));
        zipIt(outputFilePath);
    }

    public void zipIt(String zipFile) {
        byte[] buffer = new byte[1024];
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(zipFile);
            ZipOutputStream zos = new ZipOutputStream(fos);
            System.out.println("Output to Zip: " + zipFile);
            for (String file : this.fileList) {
                System.out.println("File Added: " + file);
                ZipEntry ze = new ZipEntry(file);
                zos.putNextEntry(ze);
                FileInputStream in = new FileInputStream(sourceFolderPath + File.separator + file);
                int len;
                while ((len = in.read(buffer)) > 0) {
                    zos.write(buffer, 0, len);
                }
                in.close();
            }
            zos.closeEntry();
            //remember close it
            zos.close();
            System.out.println("Done");
        } catch (IOException ex) {
            Logger.getLogger(Zipper.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                fos.close();
            } catch (IOException ex) {
                Logger.getLogger(Zipper.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void generateFileList(File node) {
        //add file only
        if (node.isFile()) {
            fileList.add(generateZipEntry(node.getAbsoluteFile().toString()));
        }

        if (node.isDirectory()) {
            String[] subNote = node.list();
            for (String filename : subNote) {
                generateFileList(new File(node, filename));
            }
        }
    }

    private String generateZipEntry(String file) {
        return file.substring(sourceFolderPath.length() + 1, file.length());
    }
}