
package org.netbeans.jsstubsgenerator.core.mdn;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.StubObject;

/**
 * TODO - whole TypedArraysGenerator
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class TypedArraysGenerator {

    private static final Logger LOGGER = Logger.getLogger(TypedArraysGenerator.class.getName());

    private static final String JS_TYPED_ARRAYS_DIR = "JavaScript_typed_arrays";

    public static void parseCoreDocs(Map<String, StubObject> allStubs, File inputDir) throws IOException {
        File dirWithArraysStubs = new File(inputDir, JS_TYPED_ARRAYS_DIR);
        if (!dirWithArraysStubs.exists()) {
            LOGGER.log(Level.WARNING, "Dir with typed arrays docs {0} doesn''t exist!", dirWithArraysStubs.getPath());
        }

        for (File docFile : dirWithArraysStubs.listFiles(new CoreJsFileFilter())) {
            // get stub name from the file name
            String className = docFile.getName();
            StubObject stubObject = new StubObject(Origin.CORE, className, StubObject.GLOBAL);

            if (allStubs.get(className) != null) {
                LOGGER.log(Level.WARNING, "Stub entry already exists: ", className);
            }

            // objects are defined in directories
            if (docFile.isDirectory()) {
                LOGGER.log(Level.WARNING, "No directories should be found for typed arrays like ", docFile.getPath());
            } else {
                attachSummary(stubObject, docFile);
            }

            allStubs.put(className, stubObject);
        }
    }

    private static void attachSummary(StubObject stubObject, File summaryFile) throws IOException {
        Document summaryDoc = Jsoup.parse(summaryFile, "UTF-8");
        Elements divs = summaryDoc.select("div[class=pageText] > div");
        attachSummaryForHeaders(stubObject, divs, "h2");
        if ("".equals(stubObject.getSummary())) {
            attachSummaryForHeaders(stubObject, divs, "h3");
        }
    }

    private static void attachSummaryForHeaders(StubObject stubObject, Elements divs, String headerType) {
        for (Element div : divs) {
            if (div.select(headerType).text().equals("Summary")) {
                stubObject.setSummary(div.select("p").text());
            } else if (div.select(headerType).text().equals("Syntax")) {
                stubObject.setSyntax(div.select("p").html());
            } else if (div.select(headerType).text().equals("Description")) {
                stubObject.setDescription(div.select("p").html());
            }
        }
    }

}
