/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.Return;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class DocumentInstanceGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject documentObject = new StubObject(Origin.DOM, "document", StubObject.GLOBAL);
        documentObject.setPrune(true);
        documentObject.setType(Type.OBJECT);
        documentObject.setSuperClass("HTMLDocument");
        documentObject.setInstance(true);
        documentObject.setSummary("Each web page loaded in the browser has its own document object. This object serves as an entry point to the web page's content (the DOM tree, including elements such as <body> and <table> ) and provides functionality global to the document (such as obtaining the page's URL and creating new elements in the document).");

        // properties definition
        String[][] attributes = new String[][]{
            {"activeElement", "Element", "Returns the currently focused element"},
            {"async", "undefined", "Used with document.load to indicate an asynchronous request."},
            {"characterSet", "String", "Gets/sets the background color of the current document."},
            {"compatMode", "String", "Indicates whether the document is rendered in Quirks or Strict mode."},
            {"contentType", "String", "Returns the Content-Type from the MIME Header of the current document."},
            {"defaultView", "Window", "Returns a reference to the window object."},
            {"designMode", "String", "Gets/sets WYSYWIG editing capability of Midas. It can only be used for HTML documents."},
            {"dir", "String", "Gets/sets directionality (rtl/ltr) of the document"},
            {"embeds", "HTMLCollection", "Returns a list of the embedded OBJECTS within the current document."},
            {"head", "HTMLHeadElement", "Returns the HEAD node of the current document."},
            {"lastModified", "String", "Returns the date on which the document was last modified."},
            {"location", "Location", "Returns the URI of the current document."},
            {"plugins", "HTMLCollection", "Returns a list of the available plugins."},
            {"popupNode", "Node", "Returns the node upon which a popup was invoked (XUL documents only)."},
            {"readyState", "String", "Returns loading status of the document"},
            {"scripts", "HTMLCollection", "Returns all the <script> elements on the document."},
            {"styleSheets", "StyleSheetList", "Returns a list of the stylesheet objects on the current document."},
            {"tooltipNode", "Node", "Returns the node which is the target of the current tooltip."},
        };
        List<StubObject> generatedProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "document", true);
        for (StubObject instanceProperty : generatedProperties) {
            documentObject.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"addEventListener", "undefined", "addEventListener() registers a single event listener on a single target. The event target may be a single element in a document, the document itself, a window, or an XMLHttpRequest. To register more than one event listener for the target, call addEventListener() for the same target but with different event types or capture parameters."},
            {"createEvent", "Event", "Creates an event of the type specified. The returned object should be first initialized and can then be passed to element.dispatchEvent."},
            {"createExpression", "String", "This method compiles an XPathExpression which can then be used for (repeated) evaluations."},
            {"createNSResolver", "XPathNSResolver", "Creates an XPathNSResolver which resolves namespaces with respect to the definitions in scope for a specified node."},
            {"createRange", "Range", "Returns a new Range object. "},
            {"createTreeWalker", "TreeWalker", "Returns a new TreeWalker object."},
            {"elementFromPoint", "Element", "Returns the element from the document whose elementFromPoint method is being called which is the topmost element which lies under the given point.  To get an element, specify the point via coordinates, in CSS pixels, relative to the upper-left-most point in the window or frame containing the document."},
            {"evaluate", "XPathResult", "Returns an XPathResult based on an XPath expression and other given parameters."},
            {"execCommand", "undefined", "When an HTML document has been switched to designMode, the document object exposes the execCommand method which allows one to run commands to manipulate the contents of the editable region. Most commands affect the document's selection (bold, italics, etc), while others insert new elements (adding a link) or affect an entire line (indenting). When using contentEditable, calling execCommand will affect the currently active editable element."},
            {"getElementsByClassName", "NodeList", "Returns a set of elements which have all the given class names. When called on the document object, the complete document is searched, including the root node. You may also call getElementsByClassName on any element; it will return only elements which are descendants of the specified root element with the given class names."},
            {"getSelection", "Selection", "The DOM getSelection() method is available on the Window and Document interfaces. See window.getSelection()  for details."},
            {"hasFocus", "Boolean", "Returns a Boolean value indicating whether the document or any element inside the document has focus. This method can be used to determine whether the active element in a document has focus."},
            {"load", "undefined", "document.load() is a part of an old version of the W3C DOM Level 3 Load & Save module. Can be used with document.async to indicate whether the request is synchronous or asynchronous (the default). As of at least Gecko 1.9, this no longer supports cross-site loading of documents (Use XMLHttpRequest instead)."},
            {"queryCommandEnabled", "Boolean", "Determines whether the given command can be executed on the document in its current state."},
            {"queryCommandIndeterm", "Boolean", "Determines whether the current selection is in an indetermined state."},
            {"queryCommandState", "Boolean", "Determines whether the given command has been executed on the current selection."},
            {"queryCommandSupported", "Boolean", "Reports whether or not the specified editor query command is supported by the browser."},
            {"queryCommandValue", "Boolean", "Determines the current value of the document, range, or current selection for the given command."},
            {"querySelector", "Element", "Returns the first element within the document (using depth-first pre-order traversal of the document's nodes) that matches the specified group of selectors."},
            {"querySelectorAll", "NodeList", "Returns a list of the elements within the document (using depth-first pre-order traversal of the document's nodes) that match the specified group of selectors. The object returned is a NodeList."},
            {"releaseCapture", "undefined", "Releases mouse capture if it's currently enabled on an element within this document. Enabling mouse capture on an element is done by calling element.setCapture() ."},
            {"addEventListener", "undefined", "This"},
            {"addEventListener", "undefined", "This"},
            {"addEventListener", "undefined", "This"},
            {"registerContentHandler", "undefined", "Allows web sites to register themselves as possible handlers for content of a particular MIME type.<br>Note: Web sites may only register content handlers for themselves. For security reasons, it's not possible for an extension or web site to register content handlers targeting other sites."},
            {"registerProtocolHandler", "undefined", "Allows web sites to register themselves as possible handlers for particular protocols.<br>Note: Web sites may only register protocol handlers for themselves. For security reasons, it's not possible for an extension or web site to register protocol handlers targeting other sites."},
        };
        List<StubObject> generatedMethods = generateMethods(methods);
        for (StubObject instanceMethod : generatedMethods) {
            documentObject.putAttribute(instanceMethod);
        }

        allStubs.put("document", documentObject);
    }

    private static List<StubObject> generateMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "document", true);
        for (StubObject method : methodStubs) {
            if ("addEventListener".equals(method.getName())) {
                method.setSyntax("target.addEventListener(type, listener[, useCapture]);<br>target.addEventListener(type, listener[, useCapture, aWantsUntrusted); // Gecko/Mozilla only");
                method.setReturns(new Return("Boolean", "The return value for this method indicates whether the preference that controls Java is on or off - not whether the browser offers Java support in general."));
                method.addParameter(new Parameter("type", "String", "A string representing the event type to listen for."));
                method.addParameter(new Parameter("listener", "Function", "The object that receives a notification when an event of the specified type occurs. This must be an object implementing the EventListener interface, or simply a JavaScript function."));
                method.addParameter(new Parameter("useCapture", "Boolean", "If true, useCapture indicates that the user wishes to initiate capture. After initiating capture, all events of the specified type will be dispatched to the registered listener before being dispatched to any EventTarget beneath it in the DOM tree. Events which are bubbling upward through the tree will not trigger a listener designated to use capture. See DOM Level 3 Events for a detailed explanation. Note that this parameter is not optional in all browser versions. If not specified, useCapture is false."));
                method.addParameter(new Parameter("aWantsUntrusted", "Boolean", "If true, the event can be triggered by untrusted content. See Interaction between privileged and non-privileged pages."));
            } else if ("createEvent".equals(method.getName())) {
                method.setSyntax("var event = document.createEvent(type);");
                method.setReturns(new Return("Event", "event is the created Event object."));
                method.addParameter(new Parameter("type", "String", "type is a string that represents the type of event to be created. Possible event types include \"UIEvents\", \"MouseEvents\", \"MutationEvents\", and \"HTMLEvents\". See Notes section for details."));
            } else if ("createExpression".equals(method.getName())) {
                method.setSyntax("xpathExpr = document.createExpression(xpathText, namespaceURLMapper);");
                method.addParameter(new Parameter("xpathText", "String", "String xpathText (the XPath expression to be compiled)"));
                method.addParameter(new Parameter("namespaceURLMapper", "Function", "Function namespaceURLMapper (maps a namespace prefix to a namespace URL (or null if none needed))"));
            } else if ("createNSResolver".equals(method.getName())) {
                method.setSyntax("nsResolver = document.createNSResolver(node);");
                method.addParameter(new Parameter("node", "Node", "node is the node to be used as a context for namespace resolution."));
            } else if ("createRange".equals(method.getName())) {
                method.setSyntax("range = document.createRange();");
                method.setReturns(new Return("Range", "range is the created range object. "));
            } else if ("createTreeWalker".equals(method.getName())) {
                method.setSyntax("var treeWalker = document.createTreeWalker(root, whatToShow, filter, entityReferenceExpansion);");
                method.addParameter(new Parameter("root", "Node", "root is the root of the TreeWalker's traversal. Typically this will be an element owned by the document."));
                method.addParameter(new Parameter("whatToShow", "Number", "whatToShow is a bitmask created by combining the constant properties of NodeFilter. It is a convenient way of filtering for certain types of node."));
                method.addParameter(new Parameter("filter", "Number", "filter is an optional object with a method acceptNode, which is called by the TreeWalker to determine whether or not to accept a node that has passed the whatToShow check. The function should return NodeFilter.FILTER_ACCEPT, which causes the TreeWalker to return the node, NodeFilter.FILTER_REJECT, which causes the TreeWalker to ignore the entire subtree, or NodeFilter.FILTER_SKIP. Gecko allows filter to be a function instead of an object."));
                method.addParameter(new Parameter("entityReferenceExpansion", "String", "entityReferenceExpansion is not used by Gecko but the value is saved and returned by the nsIDOMNodeIterator property."));
            } else if ("elementFromPoint".equals(method.getName())) {
                method.setSyntax("var element = document.elementFromPoint(x, y);");
                method.setReturns(new Return("Element", "element is an element object."));
                method.addParameter(new Parameter("x", "Number", "x and y specify the coordinates to check, in CSS pixels relative to the upper-left corner of the document's containing window or frame."));
                method.addParameter(new Parameter("y", "Number", "x and y specify the coordinates to check, in CSS pixels relative to the upper-left corner of the document's containing window or frame."));
            } else if ("execCommand".equals(method.getName())) {
                method.setSyntax("document.execCommand(aCommandName, aShowDefaultUI, aValueArgument)");
                method.addParameter(new Parameter("aCommandName", "String", "the name of the command"));
                method.addParameter(new Parameter("aShowDefaultUI", "Boolean", "whether the default user interface should be shown. This is not implemented in Mozilla."));
                method.addParameter(new Parameter("aValueArgument", "String", "some commands (such as insertimage) require an extra value argument (the image's url). Pass an argument of null if no argument is needed."));
            } else if ("getElementsByClassName".equals(method.getName())) {
                method.setSyntax("elements = document.getElementsByClassName(names);<br>elements = rootElement.getElementsByClassName(names); ");
                method.setReturns(new Return("NodeList", "elements is a live NodeList (but see the note below) of found elements in the order they appear in the tree."));
                method.addParameter(new Parameter("names", "String", "names is a string representing the list of class names to match; class names are separated by whitespace"));
            } else if ("hasFocus".equals(method.getName())) {
                method.setSyntax("focused = document.hasFocus();");
                method.setReturns(new Return("Boolean", "false if the active element in the document has no focus; true if the active element in the document has focus."));
            } else if ("queryCommandEnabled".equals(method.getName())) {
                method.setSyntax("var isEnabled = document.queryCommandEnabled(command);");
                method.addParameter(new Parameter("command", "String", "The commandID to check."));
            } else if ("queryCommandIndeterm".equals(method.getName())) {
                method.setSyntax("var isIndeterm = document.queryCommandIndeterm(command);");
                method.addParameter(new Parameter("command", "String", "The commandID to check."));
            } else if ("queryCommandState".equals(method.getName())) {
                method.setSyntax("var isState = document.queryCommandState(command);");
                method.addParameter(new Parameter("command", "String", "The commandID to check."));
            } else if ("queryCommandSupported".equals(method.getName())) {
                method.setSyntax("var isSupported = document.queryCommandSupported(command);");
                method.addParameter(new Parameter("command", "String", "The commandID to check."));
            } else if ("queryCommandValue".equals(method.getName())) {
                method.setSyntax("var isValue = document.queryCommandValue(command);");
                method.addParameter(new Parameter("command", "String", "The commandID to check."));
            } else if ("elementFromPoint".equals(method.getName())) {
                method.setSyntax("var element = document.elementFromPoint(x, y);");
                method.setReturns(new Return("Element", "element is an element object."));
                method.addParameter(new Parameter("x", "Number", "x and y specify the coordinates to check, in CSS pixels relative to the upper-left corner of the document's containing window or frame."));
                method.addParameter(new Parameter("y", "Number", "x and y specify the coordinates to check, in CSS pixels relative to the upper-left corner of the document's containing window or frame."));
            } else if ("evaluate".equals(method.getName())) {
                method.setSyntax("var xpathResult = document.evaluate(xpathExpression, contextNode, namespaceResolver, resultType, result);");
                method.addParameter(new Parameter("xpathExpression", "String", "xpathExpression is a string representing the XPath to be evaluated."));
                method.addParameter(new Parameter("contextNode", "Node", "contextNode specifies the context node for the query (see the [http://www.w3.org/TR/xpath XPath specification). It's common to pass document as the context node."));
                method.addParameter(new Parameter("namespaceResolver", "String", "namespaceResolver is a function that will be passed any namespace prefixes and should return a string representing the namespace URI associated with that prefix. It will be used to resolve prefixes within the XPath itself, so that they can be matched with the document. null is common for HTML documents or when no namespace prefixes are used."));
                method.addParameter(new Parameter("resultType", "Number", "resultType is an integer that corresponds to the type of result XPathResult to return. Use named constant properties, such as XPathResult.ANY_TYPE, of the XPathResult constructor, which correspond to integers from 0 to 9."));
                method.addParameter(new Parameter("result", "XPathResult", "result is an existing XPathResult to use for the results. null is the most common and will create a new XPathResult"));
            }
        }
        return methodStubs;
    }

}
