/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import org.netbeans.jsstubsgenerator.stub.StubObject;

/**
 * Fill up stub's inheritances.
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class Inheritance {

    private static final Map<String, String> INHERITANCE =
    Collections.unmodifiableMap(new HashMap<String, String>() {{
        // DOM - Core
        put("DOMString", "String");
        put("DOMTimeStamp", "Date");
        put("DOMObject", "Object");
        put("DocumentFragment", "Node");
        put("Document", "Node");
        put("CharacterData", "Node");
        put("Attr", "Node");
        put("Element", "Node");
        put("Text", "CharacterData");
        put("Comment", "CharacterData");
        put("CDATASection", "Text");
        put("DocumentType", "Node");
        put("Notation", "Node");
        put("Entity", "Node");
        put("EntityReference", "Node");
        put("ProcessingInstruction", "Node");
        // DOM - Css
        put("CSSStyleSheet", "StyleSheet");
        put("CSSStyleRule", "CSSRule");
        put("CSSMediaRule", "CSSRule");
        put("CSSFontFaceRule", "CSSRule");
        put("CSSPageRule", "CSSRule");
        put("CSSImportRule", "CSSRule");
        put("CSSCharsetRule", "CSSRule");
        put("CSSUnknownRule", "CSSRule");
        put("CSSPrimitiveValue", "CSSValue");
        put("CSSValueList", "CSSValue");
        put("ViewCSS", "AbstractView");
        put("DocumentCSS", "DocumentStyle");
        put("DOMImplementationCSS", "DOMImplementation");
        // DOM - Events
        put("UIEvent", "Event");
        put("MouseEvent", "UIEvent");
        put("MutationEvent", "Event");
        // DOM - Html
        put("HTMLDocument", "Document");
        put("HTMLElement", "Element");
        put("HTMLHtmlElement", "HTMLElement");
        put("HTMLHeadElement", "HTMLElement");
        put("HTMLLinkElement", "HTMLElement");
        put("HTMLTitleElement", "HTMLElement");
        put("HTMLMetaElement", "HTMLElement");
        put("HTMLBaseElement", "HTMLElement");
        put("HTMLIsIndexElement", "HTMLElement");
        put("HTMLStyleElement", "HTMLElement");
        put("HTMLBodyElement", "HTMLElement");
        put("HTMLFormElement", "HTMLElement");
        put("HTMLSelectElement", "HTMLElement");
        put("HTMLOptGroupElement", "HTMLElement");
        put("HTMLOptionElement", "HTMLElement");
        put("HTMLInputElement", "HTMLElement");
        put("HTMLTextAreaElement", "HTMLElement");
        put("HTMLButtonElement", "HTMLElement");
        put("HTMLLabelElement", "HTMLElement");
        put("HTMLFieldSetElement", "HTMLElement");
        put("HTMLLegendElement", "HTMLElement");
        put("HTMLUListElement", "HTMLElement");
        put("HTMLOListElement", "HTMLElement");
        put("HTMLDListElement", "HTMLElement");
        put("HTMLDirectoryElement", "HTMLElement");
        put("HTMLMenuElement", "HTMLElement");
        put("HTMLLIElement", "HTMLElement");
        put("HTMLDivElement", "HTMLElement");
        put("HTMLParagraphElement", "HTMLElement");
        put("HTMLHeadingElement", "HTMLElement");
        put("HTMLQuoteElement", "HTMLElement");
        put("HTMLPreElement", "HTMLElement");
        put("HTMLBRElement", "HTMLElement");
        put("HTMLBaseFontElement", "HTMLElement");
        put("HTMLFontElement", "HTMLElement");
        put("HTMLHRElement", "HTMLElement");
        put("HTMLModElement", "HTMLElement");
        put("HTMLAnchorElement", "HTMLElement");
        put("HTMLImageElement", "HTMLElement");
        put("HTMLObjectElement", "HTMLElement");
        put("HTMLParamElement", "HTMLElement");
        put("HTMLAppletElement", "HTMLElement");
        put("HTMLMapElement", "HTMLElement");
        put("HTMLAreaElement", "HTMLElement");
        put("HTMLScriptElement", "HTMLElement");
        put("HTMLTableElement", "HTMLElement");
        put("HTMLTableCaptionElement", "HTMLElement");
        put("HTMLTableColElement", "HTMLElement");
        put("HTMLTableSectionElement", "HTMLElement");
        put("HTMLTableRowElement", "HTMLElement");
        put("HTMLTableCellElement", "HTMLElement");
        put("HTMLFrameSetElement", "HTMLElement");
        put("HTMLFrameElement", "HTMLElement");
        put("HTMLIFrameElement", "HTMLElement");
    }});

    public static void enhanceForInheritance(Map<String, StubObject> allStubs) {
        Iterator<Entry<String, StubObject>> iterator = allStubs.entrySet().iterator();
        while (iterator.hasNext()) {
            StubObject stub = iterator.next().getValue();
            if (INHERITANCE.containsKey(stub.getName())) {
                stub.setSuperClass(INHERITANCE.get(stub.getName()));
            }
        }
    }

}
