/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class LocationGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject locationInterface = new StubObject(Origin.DOM, "Location", StubObject.GLOBAL);
        locationInterface.setType(Type.OBJECT);
        locationInterface.setSummary("Represents the DocumentWindow's current location. The value MUST be the same as the Location for all views of the document.");

        // attributes definition
        String[][] attributes = new String[][]{
            {"href", "DOMString", "The value of the href attribute MUST be the absolute URI reference that is the Location's current location. When the href attribute is set, the Location's window MUST navigate to the newly set value."},
            {"hash", "DOMString", "If the absolute URI reference[RFC2396] for the Location's current location has a fragment identifier[RFC2396], then the value of the hash attribute the value of this attribute MUST be the string concatenation of the hash mark(#) and the fragment identifier. Otherwise, the value of the hash attribute MUST be the empty string."},
            {"host", "DOMString", "This attribute represents the network host of the Location's URI. If the port attribute is not null then the host attribute's value is the concatenation of the hostname attribute, a colon (:) and the port attribute. If the port attribute is null then the host attribute's value is identical to the hostname attribute."},
            {"hostname", "DOMString", "This attribute represents the name or IP address of the network location without any port number."},
            {"pathname", "DOMString", "This attribute represents the path component of the Location's URI which consists of everything after the host and port up to and excluding the first question mark (?) or hash mark (#)."},
            {"port", "DOMString", "This attribute represents the port number of the network location."},
            {"protocol", "DOMString", "This attribute represents the scheme of the URI including the trailing colon (:)"},
            {"search", "DOMString", "    This attribute represents the query portion of a URI. It consists of everything after the pathname up to and excluding the first hash mark (#)."},
        };
        List<StubObject> generatedInstanceProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "Location", false);
        for (StubObject instanceProperty : generatedInstanceProperties) {
            locationInterface.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"assign", "undefined", "Load the document at the provided URL."},
            {"replace", "undefined", "Replace the current document with the one at the provided URL. The difference from the assign() method is that after using replace() the current page will not be saved in session history, meaning the user won't be able to use the Back button to navigate to it."},
            {"reload", "undefined", "Reload the document from the current URL. forceget is a boolean, which, when it is true, causes the page to always be reloaded from the server. If it is false or not specified, the browser may reload the page from its cache."},
            {"toString", "DOMString", "Returns the string representation of the Location object's URL. See the JavaScript reference for details."},
        };
        List<StubObject> generatedInstanceMethods = generateInstanceMethods(methods);
        for (StubObject instanceMethod : generatedInstanceMethods) {
            locationInterface.putAttribute(instanceMethod);
        }

        allStubs.put("Location", locationInterface);
    }

    private static List<StubObject> generateInstanceMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "Location", false);
        for (StubObject method : methodStubs) {
            if ("assign".equals(method.getName()) || "replace".equals(method.getName())) {
                method.addParameter(new Parameter("url", "DOMString", ""));
            }
        }
        return methodStubs;
    }
}
