/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class StorageGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject StorageInterface = new StubObject(Origin.DOM, "Storage", StubObject.GLOBAL);
        StorageInterface.setType(Type.OBJECT);
        StorageInterface.setSummary("The DOM Storage mechanism is a means through which string key/value pairs can be securely stored and later retrieved for use. The goal of this addition is to provide a comprehensive means through which interactive applications can be built (including advanced abilities, such as being able to work \"offline\" for extended periods of time).<br>DOM Storage is useful because no good browser-only methods exist for persisting reasonable amounts of data for any period of time. Browser cookies have limited capacity and provide no support for organizing persisted data, and other methods (such as Flash Local Storage) require an external plugin.<br>One of the first public applications to make use of the new DOM Storage functionality (in addition to Internet Explorer's userData Behavior) was halfnote (a note-taking application) written by Aaron Boodman. In his application, Aaron simultaneously saved notes back to a server (when Internet connectivity was available) and a local data store. This allowed the user to safely write backed-up notes even with sporadic Internet connectivity.<br>While the concept, and implementation, presented in halfnote was comparatively simple, its creation shows the possibility for a new breed of web applications that are usable both online and offline.");

        // attributes definition
        String[][] attributes = new String[][]{
            {"length", "Number", "length"},
        };
        List<StubObject> generatedInstanceProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "Storage", false);
        for (StubObject instanceProperty : generatedInstanceProperties) {
            StorageInterface.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"key", "DOMString", "The key(n) method must return the name of the nth key in the list. The order of keys is user-agent defined, but must be consistent within an object so long as the number of keys doesn't change. (Thus, adding or removing a key may change the order of the keys, but merely changing the value of an existing key must not.) If n is greater than or equal to the number of key/value pairs in the object, then this method must return null."},
            {"getItem", "DOMString", "The getItem(key) method must return the current value associated with the given key. If the given key does not exist in the list associated with the object then this method must return null."},
            {"setItem", "undefined", "The setItem(key, value) method must first check if a key/value pair with the given key already exists in the list associated with the object. If it does not, then a new key/value pair must be added to the list, with the given key and with its value set to value. If the given key does exist in the list, then it must have its value updated to value. If it couldn't set the new value, the method must throw an QuotaExceededError exception. (Setting could fail if, e.g., the user has disabled storage for the site, or if the quota has been exceeded.)"},
            {"removeItem", "undefined", "The removeItem(key) method must cause the key/value pair with the given key to be removed from the list associated with the object, if it exists. If no item with that key exists, the method must do nothing."},
            {"clear", "undefined", "The clear() method must atomically cause the list associated with the object to be emptied of all key/value pairs, if there are any. If there are none, then the method must do nothing."},
        };
        List<StubObject> generatedInstanceMethods = generateInstanceMethods(methods);
        for (StubObject instanceMethod : generatedInstanceMethods) {
            StorageInterface.putAttribute(instanceMethod);
        }

        allStubs.put("Storage", StorageInterface);
    }

    private static List<StubObject> generateInstanceMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "Storage", false);
        for (StubObject method : methodStubs) {
            if ("key".equals(method.getName())) {
                method.addParameter(new Parameter("n", "Number", null));
            } else if ("getItem".equals(method.getName())) {
                method.addParameter(new Parameter("key", "DOMString", null));
            } else if ("setItem".equals(method.getName())) {
                method.addParameter(new Parameter("key", "DOMString", null));
                method.addParameter(new Parameter("value", "DOMString", null));
            } else if ("removeItem".equals(method.getName())) {
                method.addParameter(new Parameter("key", "DOMString", null));
            }
        }
        return methodStubs;
    }
}
