
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class WorkerGenerator {

    private static final String OBJECT_NAME = "Worker";

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject worker = new StubObject(Origin.DOM, OBJECT_NAME, StubObject.GLOBAL);
        worker.setPrune(true);
        worker.setType(Type.OBJECT);
        worker.setSuperClass("AbstractWorker");
        worker.setSummary("The Worker interface represents a background task that can be easily created and can send messages back to their creators. Creating a worker is as simple as calling the Worker() constructor, specifying a script to be run in the worker thread. Of note is the fact that workers may in turn spawn new workers as long as those workers are hosted within the same origin as the parent page.  In addition, workers may use XMLHttpRequest for network I/O, with the exception that the responseXML and channel attributes on XMLHttpRequest always return null.");

        // attributes definition
        String[][] attributes = new String[][]{
            {"onmessage", "undefined", "Is an EventListener that is called whenever a MessageEvent of type message bubbles through the worker. The message is stored in the event's data property."},
        };
        List<StubObject> generatedInstanceProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, OBJECT_NAME, false);
        for (StubObject instanceProperty : generatedInstanceProperties) {
            worker.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"postMessage", "undefined", "Sends a message, that is any JavaScript object to the worker's inner scope."},
            {"terminate", "undefined", "Immediately terminates the worker. This does not offer the worker an opportunity to finish its operations; it is simply stopped at once."},
        };
        List<StubObject> generatedInstanceMethods = generateInstanceMethods(methods);
        for (StubObject instanceMethod : generatedInstanceMethods) {
            worker.putAttribute(instanceMethod);
        }

        allStubs.put(OBJECT_NAME, worker);
    }

    private static List<StubObject> generateInstanceMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, OBJECT_NAME, false);
        for (StubObject method : methodStubs) {
            if ("postMessage".equals(method.getName()) || "setInterval".equals(method.getName())) {
                method.setSyntax("worker.postMessage(message, transferList);");
                method.addParameter(new Parameter("message", "Object", "The object to deliver to the worker; this will be in the data field in the event delivered to the Worker.onmessage handler. This may be any value or JavaScript object handled by the structured clone algorithm, which includes cyclical references."));
                method.addParameter(new Parameter("transferList", "array|Transferable", "n optional array of Transferable objects to transfer ownership of. If the ownership of an object is transferred, it becomes unusable (neutered) in the context it was sent from and it becomes available only to the worker it was sent to."));
            }
        }
        return methodStubs;
    }

}
