/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.server.services;

import java.util.Collection;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import org.netbeans.server.services.BugzillaDBConnector.DBTask;
import org.netbeans.server.services.beans.CommunicationHelper;
import org.netbeans.server.services.beans.Components;
import org.netbeans.server.services.beans.Mappings;
import org.netbeans.server.uihandler.api.Issue;

/**
 *
 * @author Jindrich Sedek
 */
@Path("/")
public class BugzillaService {

    @GET
    @Path("filter")
    @Produces(MediaType.TEXT_XML)
    public String filterOpen(@QueryParam("ids") final String idsParam){
        if ((idsParam == null) || idsParam.isEmpty()){
            return idsParam;
        }
        DBTask<Collection<Integer>> task = new FilterTask(idsParam);
        Collection<Integer> filtered = BugzillaDBConnector.getInstance().process(task);
        return CommunicationHelper.pack(filtered);
    }

    @GET
    @Path("bug/{id}")
    @Produces(MediaType.TEXT_XML)
    public Issue getBug(@PathParam("id") final Integer id) {
        DBTask<Issue> task = new IssueDBTask(id);
        Issue result = BugzillaDBConnector.getInstance().process(task);
        return result;
    }

    @GET
    @Path("components")
    @Produces(MediaType.TEXT_XML)
    public Components getComponents() {
        DBTask<Components> task = new ComponentsTask();
        Components result = BugzillaDBConnector.getInstance().process(task);
        return result;
    }

    @GET
    @Path("mappings")
    @Produces(MediaType.TEXT_XML)
    public Mappings getMappings() {
        DBTask<Mappings> task = new MappingsTask();
        Mappings result = BugzillaDBConnector.getInstance().process(task);
        return result;
    }

}
