/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is Forte for Java, Community Edition. The Initial
 * Developer of the Original Software is Sun Microsystems, Inc. Portions
 * Copyright 1997-2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.mdrxml.looks.actions;

import java.util.Collection;
import java.util.Iterator;
import javax.jmi.reflect.*;
import org.openide.nodes.*;
import org.openide.util.actions.NodeAction;
import org.openide.util.NbBundle;
import org.openide.util.HelpCtx;
import org.netbeans.api.looks.*;
import org.netbeans.api.mdr.MDRObject;
import org.netbeans.api.mdr.MDRepository;
/**
 *
 * @author  Tomas Zezula
 */
public class DeleteAction extends NodeAction {

    /** Creates a new instance of DeleteAction */
    public DeleteAction() {
    }
    
    public void performAction(Node[] nodes) {
        for (int i=0; i< nodes.length; i++) {
            LookNode ln = (LookNode) nodes[i];
            xmlmodel.Node node = (xmlmodel.Node) ln.getRepresentedObject();
            MDRepository repo = ((MDRObject)node).repository();
            repo.beginTrans(true);
            try {
                if (node instanceof xmlmodel.ElementNode)
                    this.deleteHierarchy ((xmlmodel.ElementNode)node);
                ((RefObject)node).refDelete();
            }catch (Exception e) {
                e.printStackTrace();
            }
            finally {
                repo.endTrans();
            }
        }
    }
    
    public boolean enable(Node[] nodes) {
        if (nodes == null)
            return false;
        for (int i=0; i< nodes.length; i++) {
            if (!(nodes[i] instanceof LookNode))
                return false;
            if (!(((LookNode)nodes[i]).getRepresentedObject() instanceof xmlmodel.Node))
                return false;
        }
        return true;
    }
    
    public String getName() {
        return NbBundle.getMessage(DeleteAction.class, "TXT_Delete");
    }
    
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
    
    
    protected void deleteHierarchy(xmlmodel.ElementNode node) {
        Collection subNodes = node.getNodes();
        for (Iterator it = subNodes.iterator(); it.hasNext();) {
            xmlmodel.Node subNode = (xmlmodel.Node)it.next();
	    it.remove ();
            if (subNode instanceof xmlmodel.ElementNode)
                deleteHierarchy ((xmlmodel.ElementNode)subNode);
	    if (subNode != null)
		try {
        	 ((RefObject)subNode).refDelete();
		} catch (javax.jmi.reflect.InvalidObjectException invalidObjectExc) {
		    // Already deleted object
		}
        }
    }
}
