/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */


package org.foo.fftview;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.foo.wavsupport.api.WavCookie;
import org.foo.wavutils.FFTGraph;
import org.foo.wavutils.WrappedAudioInputStream;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.loaders.DataObject;
import org.openide.util.Lookup;

/**
 *
 * @author runger
 */
public class FFTComponent implements MultiViewElement {
    
    private DataObject dobj;
    private final FFTGraph graph = new FFTGraph();
    
    public FFTComponent(DataObject dobj)
    {
        super();
        
        this.dobj = dobj;
        final WavCookie c = (WavCookie)dobj.getCookie(WavCookie.class);
        assert(c != null);

        graph.createGraph(c.getAudioInputStream());
        
        c.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                if (evt.getPropertyName().equals(WavCookie.PROP_WAVEFORM))
                {
                    WrappedAudioInputStream wais = c.getAudioInputStream();
                    if (wais == null)
                        graph.clearGraph();
                    else
                        graph.createGraph(wais);
                }
            }
        });
    }

    public JComponent getVisualRepresentation() {
        return graph;
    }

    public JComponent getToolbarRepresentation() {
        return new JPanel();
    }

    public void setMultiViewCallback(MultiViewElementCallback multiViewElementCallback) {
        // do nothing (don't need the callback)
    }

    public CloseOperationState canCloseElement() {
        // The main wav component handles asking the user to save.
        // THIS component is OK whatever the outcome.
        return CloseOperationState.STATE_OK;
    }
    
    public void componentDeactivated() {}
    public void componentActivated() {}
    public void componentHidden() {}
    public void componentShowing() {}
    public void componentClosed() {}
    public void componentOpened() {}

    public Object writeReplace() {
        return null;
    }

    public Action[] getActions() {
        return new Action[0];
    }

    public Lookup getLookup() {
        return dobj.getNodeDelegate().getLookup();
    }

    public UndoRedo getUndoRedo() {
        return new UndoRedo.Manager();
    }
}
