/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */
package org.foo.wavsupport;

import java.lang.reflect.InvocationTargetException;
import javax.sound.sampled.AudioFormat;
import org.openide.ErrorManager;
import org.openide.loaders.DataNode;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.nodes.Node.Property;
import org.openide.nodes.Sheet;
import org.openide.util.NbBundle;

public class WavDataNode extends DataNode {
    
    private static final String IMAGE_ICON_BASE = "org/foo/wavutils/wavIconSmall.gif";
    
    public WavDataNode(WavDataObject obj) {
        super(obj, Children.LEAF);
        setIconBaseWithExtension(IMAGE_ICON_BASE);
    }
    
    /** Creates a property sheet. */
    protected Sheet createSheet() {
        Sheet s = super.createSheet();
        Sheet.Set ss = s.get(Sheet.PROPERTIES);
        if (ss == null) {
            ss = Sheet.createPropertiesSet();
            s.put(ss);
        }
        
        AudioFormat f = ((WavDataObject)getDataObject()).getAudioFormat();
        
        ss.put(new Encoding(f));
        ss.put(new Channels(f));
        ss.put(new FrameRate(f));
        ss.put(new FrameSize(f));
        ss.put(new SampleRate(f));
        ss.put(new SampleSize(f));
        ss.put(new Endian(f));
        return s;
    }
    
    private abstract class AudioFormatProperty extends Node.Property
    {
        private AudioFormat format;
        
        public AudioFormatProperty(AudioFormat f, Class clazz)
        {
            super(clazz);
            format = f;
        }
        
        protected AudioFormat getFormat()
        {
            return format;
        }
        
        public boolean canRead() { return true; }
        public boolean canWrite() { return false; }

        public void setValue(Object object) throws 
                IllegalAccessException, 
                IllegalArgumentException, 
                InvocationTargetException {}
    }
    
    private class SampleRate extends AudioFormatProperty
    {
        public SampleRate(AudioFormat f)
        {
            super(f, Float.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getSampleRate();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_SampleRate");
        }
    }
    
    private class Channels extends AudioFormatProperty
    {
        public Channels(AudioFormat f)
        {
            super(f, Integer.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getChannels();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_Channels");
        }
    }
    
    private class Encoding extends AudioFormatProperty
    {
        public Encoding(AudioFormat f)
        {
            super(f, String.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getEncoding().toString();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_Encoding");
        }
    }
    
    private class FrameRate extends AudioFormatProperty
    {
        public FrameRate(AudioFormat f)
        {
            super(f, Float.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getFrameRate();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_FrameRate");
        }
    }
    
    private class FrameSize extends AudioFormatProperty
    {
        public FrameSize(AudioFormat f)
        {
            super(f, Integer.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getFrameSize();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_FrameSize");
        }
    }
    
    private class SampleSize extends AudioFormatProperty
    {
        public SampleSize(AudioFormat f)
        {
            super(f, Integer.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().getSampleSizeInBits();
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_SampleSize");
        }
    }
    
    private class Endian extends AudioFormatProperty
    {
        public Endian(AudioFormat f)
        {
            super(f, String.class);
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException
        {
            return getFormat().isBigEndian() ? 
                NbBundle.getMessage(WavDataNode.class, "LBL_BigEndian") : 
                NbBundle.getMessage(WavDataNode.class, "LBL_LittleEndian");
        }

        public String getName() {
            return NbBundle.getMessage(WavDataNode.class, "LBL_Endian");
        }
    }
    
}
