/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */

package org.foo.wavutils;

import org.openide.util.TaskListener;

/**
 * An interface for the audio GUI to implement, for interfacing with
 * the javasound code.
 *
 * @author runger
 */
public interface AudioUI extends TaskListener {

    SamplingGraph getSamplingGraph();

    WrappedAudioInputStream getAudioInputStream();
    
    void setAudioInputStream(WrappedAudioInputStream stream);
    
    /**
     * Represents a selected portion of the audio stream.
     * Offset and length are indices into the audio data.
     */
    public static class Selection
    {
        public Selection( int offset, int length )
        {
            this.offset = offset;
            this.length = length;
        }
        
        /**
         * Gets the offset and length of the audio selected by the user
         * in the sampling graph.
         */
        public static Selection getSelection(AudioUI audioUI)
        {
            SamplingGraph samplingGraph = audioUI.getSamplingGraph();
            synchronized( samplingGraph )
            {
                double dStart = samplingGraph.getSelectionStart();
                double dEnd = samplingGraph.getSelectionEnd();

                // do we have a non-zero selection?
                if( Math.abs( dStart - dEnd ) == 0 )
                {
                    return null;
                }
                else
                {
                    // make sure they're in the right order
                    if( dEnd < dStart )
                    {
                        double d = dEnd;
                        dEnd = dStart;
                        dStart = d;
                    }

                    double dWidth = samplingGraph.getWidth();

                    WrappedAudioInputStream pcmStream = audioUI.getAudioInputStream();

                    if( pcmStream == null )
                    {
                        return null;
                    }
                    double dPromptLength = pcmStream.getLength();

                    int iBeginIndex = (int)( (dStart / dWidth) * dPromptLength );
                    int iEndIndex = (int)( (dEnd / dWidth) * dPromptLength );
                    int iLength = iEndIndex - iBeginIndex;

                    return new Selection( iBeginIndex, iLength );
                }
            }
        }

        public int offset;
        public int length;
    }

}
