/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cards.api;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * An instance of a Suit belonging to a Deck.
 *
 * @author Tim Boudreau
 */
public final class Suit {

    private final Suits type;
    private final List<Card> cards;
    private final Deck deck;

    Suit(Suits type, Deck deck) {
        this.type = type;
        this.deck = deck;
        cards = initCards();
    }

    public List<Card> getCards() {
        return cards;
    }

    public boolean belongsTo(Card card) {
        boolean result = type == card.getSuit();
        assert !result || cards.contains(card) : "Someone is cheating!";
        return result;
    }

    Card getCard(Rank id) {
        for (Card card : cards) {
            if (id == card.identity()) {
                return card;
            }
        }
        return null;
    }

    public Suits getType() {
        return type;
    }

    private List<Card> initCards() {
        List<Card> l = new ArrayList<Card>();
        l.add(new Card(Rank.ACE, type, deck));
        l.add(new Card(Rank.TWO, type, deck));
        l.add(new Card(Rank.THREE, type, deck));
        l.add(new Card(Rank.FOUR, type, deck));
        l.add(new Card(Rank.FIVE, type, deck));
        l.add(new Card(Rank.SIX, type, deck));
        l.add(new Card(Rank.SEVEN, type, deck));
        l.add(new Card(Rank.EIGHT, type, deck));
        l.add(new Card(Rank.NINE, type, deck));
        l.add(new Card(Rank.TEN, type, deck));
        l.add(new Card(Rank.JACK, type, deck));
        l.add(new Card(Rank.QUEEN, type, deck));
        l.add(new Card(Rank.KING, type, deck));
        return Collections.<Card>unmodifiableList(l);
    }
}
