/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cards.propertyeditors;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import org.netbeans.examples.cards.api.Suits;
import org.openide.util.NbBundle;

/**
 * A property editor for the org.netbeans.examples.cards.api.Suits enum
 *
 * @author Tim Boudreau
 */
public class SuitPropertyEditor implements PropertyEditor {

    private Suits suit;

    public SuitPropertyEditor() {
    }

    public void setValue(Object value) {
        suit = (Suits) value;
    }

    public Object getValue() {
        return suit;
    }

    public boolean isPaintable() {
        return true;
    }

    public void paintValue(Graphics g, Rectangle box) {
        Font old = g.getFont();
        try {
            Font f = new Font("SansSerif", Font.BOLD, old == null ? 12 : old.getSize());
            g.setFont(f);
            int offset = g.getFontMetrics().getMaxAscent();
            String s = getAsText();
            g.drawString(s, box.x, box.y + offset);
        } finally {
            g.setFont(old);
        }
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String getAsText() {
        return suit == null ? NbBundle.getMessage(SuitPropertyEditor.class,
                "LBL_UNKNOWN_SUIT") : suit.getUnicodeChar() + "";
    }

    public void setAsText(String text) throws IllegalArgumentException {
        throw new IllegalArgumentException();
    }

    public String[] getTags() {
        return null;
    }

    public Component getCustomEditor() {
        JLabel lbl = new JLabel();
        lbl.setFont(new Font("SansSerif", Font.BOLD, 32));
        lbl.setText(getAsText());
        lbl.setHorizontalTextPosition(SwingConstants.CENTER);
        return lbl;
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        //do nothing
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        //do nothing
    }
}
