/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.nbtidyintegration;

import java.awt.Component;
import java.util.prefs.Preferences;
import javax.swing.event.ChangeListener;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;

public class TidyConvertWizardPanel1 implements WizardDescriptor.Panel {

    /**
     * The visual component that displays this panel. If you need to access the
     * component from this class, just use getComponent().
     */
    private Component component;

    // Get the visual component for the panel. In this template, the component
    // is kept separate. This can be more efficient: if the wizard is created
    // but never displayed, or not all panels are displayed, it is better to
    // create only those which really need to be visible.
    public Component getComponent() {
        if (component == null) {
            component = new TidyConvertVisualPanel1();
        }
        return component;
    }

    public HelpCtx getHelp() {
        // Show no Help button for this panel:
        return HelpCtx.DEFAULT_HELP;
    // If you have context help:
    // return new HelpCtx(SampleWizardPanel1.class);
    }

    public boolean isValid() {
        // If it is always OK to press Next or Finish, then:
        return true;
    // If it depends on some condition (form filled out...), then:
    // return someCondition();
    // and when this condition changes (last form field filled in...) then:
    // fireChangeEvent();
    // and uncomment the complicated stuff below.
    }

    public final void addChangeListener(ChangeListener l) {
    }

    public final void removeChangeListener(ChangeListener l) {
    }
    /*
    private final Set<ChangeListener> listeners = new HashSet<ChangeListener>(1);
    public final void addChangeListener(ChangeListener l) {
    synchronized (listeners) {
    listeners.add(l);
    }
    }
    public final void removeChangeListener(ChangeListener l) {
    synchronized (listeners) {
    listeners.remove(l);
    }
    }
    protected final void fireChangeEvent() {
    Iterator<ChangeListener> it;
    synchronized (listeners) {
    it = new HashSet<ChangeListener>(listeners).iterator();
    }
    ChangeEvent ev = new ChangeEvent(this);
    while (it.hasNext()) {
    it.next().stateChanged(ev);
    }
    }
     */

    // You can use a settings object to keep track of state. Normally the
    // settings object will be the WizardDescriptor, so you can use
    // WizardDescriptor.getProperty & putProperty to store information entered
    // by the user.
    public void readSettings(Object settings) {
        TidyConvertVisualPanel1 c = (TidyConvertVisualPanel1) component;
        Preferences p = Preferences.userNodeForPackage(TidySettings.class);

        c.setEncoding(p.get(TidySettings.KEY_CONVERSION_ENCODING, TidySettings.DEFAULT_CONVERSION_ENCODING));
        c.setIndent(p.getBoolean(TidySettings.KEY_CONVERSION_INDENT, TidySettings.DEFAULT_CONVERSION_INDENT));
        c.setClean(p.getBoolean(TidySettings.KEY_CONVERSION_CLEAN, TidySettings.DEFAULT_CONVERSION_CLEAN));
        c.setBare(p.getBoolean(TidySettings.KEY_CONVERSION_BARE, TidySettings.DEFAULT_CONVERSION_BARE));
        c.setUpper(p.getBoolean(TidySettings.KEY_CONVERSION_UPPER, TidySettings.DEFAULT_CONVERSION_UPPER));
        c.setNumeric(p.getBoolean(TidySettings.KEY_CONVERSION_NUMERIC, TidySettings.DEFAULT_CONVERSION_NUMERIC));
        c.setOmit(p.getBoolean(TidySettings.KEY_CONVERSION_OMIT, TidySettings.DEFAULT_CONVERSION_OMIT));
        c.setXML(p.getBoolean(TidySettings.KEY_CONVERSION_XML, TidySettings.DEFAULT_CONVERSION_XML));
        c.setWrap(p.getBoolean(TidySettings.KEY_CONVERSION_WRAP, TidySettings.DEFAULT_CONVERSION_WRAP), p.getInt(TidySettings.KEY_CONVERSION_WRAP_COLUMN, TidySettings.DEFAULT_CONVERSION_WRAP_COLUMN));
        c.setAccessCheckLevel(p.getInt(TidySettings.KEY_CONVERSION_ACCESS_CHECK_LEVEL, TidySettings.DEFAULT_CONVERSION_ACCESS_CHECK_LEVEL));

        String formatOutput = p.get(TidySettings.KEY_CONVERSION_FORMAT_OUTPUT, TidySettings.DEFAULT_CONVERSION_FORMAT_OUTPUT);
        String selectedItem = TidySettings.CONVERSION_FORMAT_OUTPUT_ASIS;
        if ((formatOutput != null) && (formatOutput.length() > 0)) {
            if (formatOutput.toLowerCase().contains("xhtml")) {
                selectedItem = TidySettings.CONVERSION_FORMAT_OUTPUT_XHTML;
            } else if (formatOutput.toLowerCase().contains("html")) {
                selectedItem = TidySettings.CONVERSION_FORMAT_OUTPUT_HTML;
            }
        }
        c.setFormatOutput(selectedItem);
    }

    public void storeSettings(Object settings) {
        TidyConvertVisualPanel1 c = (TidyConvertVisualPanel1) component;
        Preferences p = Preferences.userNodeForPackage(TidySettings.class);
        p.putBoolean(TidySettings.KEY_CONVERSION_BARE, c.getBare());
        p.putBoolean(TidySettings.KEY_CONVERSION_UPPER, c.getUpper());
        p.putBoolean(TidySettings.KEY_CONVERSION_CLEAN, c.getClean());
        p.putBoolean(TidySettings.KEY_CONVERSION_NUMERIC, c.getNumeric());
        p.putBoolean(TidySettings.KEY_CONVERSION_OMIT, c.getOmit());
        p.putBoolean(TidySettings.KEY_CONVERSION_WRAP, c.getWrapLine());
        p.putInt(TidySettings.KEY_CONVERSION_WRAP_COLUMN, c.getWrapLineColumn());
        p.putBoolean(TidySettings.KEY_CONVERSION_XML, c.getXML());
        p.put(TidySettings.KEY_CONVERSION_ENCODING, c.getEncoding());
        p.put(TidySettings.KEY_CONVERSION_FORMAT_OUTPUT, c.getFormatOutput());
        p.putBoolean(TidySettings.KEY_CONVERSION_INDENT, c.getIndent());
        p.putInt(TidySettings.KEY_CONVERSION_ACCESS_CHECK_LEVEL, c.getAccessCheckLevel());

    }
}

