/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.pseudonavigator;

import java.awt.BorderLayout;
import java.io.Serializable;
import java.util.Collection;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import org.netbeans.api.editor.mimelookup.MimeLookup;
import org.netbeans.api.editor.mimelookup.MimePath;
import org.netbeans.api.pseudonavigator.ListModelProvider;
import org.openide.ErrorManager;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * Top component which displays something.
 */
final class PseudoNavigatorTopComponent extends TopComponent implements ListSelectionListener, LookupListener {

    private static PseudoNavigatorTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "org/netbeans/examples/pseudonavigator/mficon.png";
    private static final String PREFERRED_ID = "PseudoNavigatorTopComponent";
    private final JList list = new JList();

    private PseudoNavigatorTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(PseudoNavigatorTopComponent.class, "CTL_PseudoNavigatorTopComponent"));
        setToolTipText(NbBundle.getMessage(PseudoNavigatorTopComponent.class, "HINT_PseudoNavigatorTopComponent"));
        setIcon(ImageUtilities.loadImage(ICON_PATH, true));
        add(new JScrollPane(list), BorderLayout.CENTER);
        list.addListSelectionListener(this);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setLayout(new java.awt.BorderLayout());

    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized PseudoNavigatorTopComponent getDefault() {
        if (instance == null) {
            instance = new PseudoNavigatorTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the PseudoNavigatorTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized PseudoNavigatorTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find PseudoNavigator component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof PseudoNavigatorTopComponent) {
            return (PseudoNavigatorTopComponent) win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }
    private Lookup.Result <DataObject> lookupResult;

    @Override
    public void componentShowing() {
        lookupResult = Utilities.actionsGlobalContext().lookupResult (DataObject.class);
        lookupResult.addLookupListener(this);
        resultChanged(null);
    }

    @Override
    public void componentHidden() {
        lookupResult.removeLookupListener(this);
        lookupResult = null;
        setActivatedNodes(new Node[0]);
    }

    @SuppressWarnings("unchecked")
    public void valueChanged(ListSelectionEvent evt) {
        if (provider != null) {
            int index = list.getSelectionModel().getLeadSelectionIndex();
            if (index != -1 && index < list.getModel().getSize()) {
                Object obj = list.getModel().getElementAt(index);
                //This generates a warning, but no choice until
                //javax.swing.ListModel is generified
                provider.selected(obj);
            }
        }
    }

    public void resultChanged(LookupEvent evt) {
        Collection<? extends DataObject> objs = lookupResult.allInstances();
        DataObject dob = objs.isEmpty() ? null : objs.iterator().next();
        setDataObject(dob);
        //We're ignoring multiselection here
        setActivatedNodes(dob == null ? new Node[0] : new Node[]{dob.getNodeDelegate()});
    }

    private void setDataObject(DataObject obj) {
        ListModelProvider modelProvider = null;
        FileObject file = null;
        if (obj != null) {
            file = obj.getPrimaryFile();
            String mime = file.getMIMEType();
            Lookup lkp = MimeLookup.getLookup (MimePath.get(mime));
            Collection <? extends ListModelProvider> providers = lkp.lookupAll(
                    ListModelProvider.class);
            modelProvider = providers.isEmpty() ? null : providers.iterator().next();
        }
        setProvider(modelProvider, file);
    }
    private ListModelProvider provider;

    private void setProvider(ListModelProvider provider, FileObject file) {
        this.provider = provider;
        if (provider == null) {
            DefaultListModel mdl = new DefaultListModel();
            mdl.addElement(NbBundle.getMessage(PseudoNavigatorTopComponent.class,
                    "LBL_EMPTY")); //NOI18N

            list.setModel(mdl);
            list.setEnabled(false);
        } else {
            list.setEnabled(true);
            list.setModel(provider.getListModel(file));
        }
    }

    /** replaces this in object stream */
    @Override
    public Object writeReplace() {
        return new ResolvableHelper();
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    final static class ResolvableHelper implements Serializable {
        private static final long serialVersionUID = 1L;
        public Object readResolve() {
            return PseudoNavigatorTopComponent.getDefault();
        }
    }
}
