/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.urlsui;

import java.awt.event.ActionEvent;
import java.io.Serializable;
import java.net.URL;
import java.util.Collection;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import org.openide.ErrorManager;
import org.openide.awt.HtmlBrowser.URLDisplayer;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.Repository;
import org.openide.loaders.DataFolder;
import org.openide.loaders.FolderLookup;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * Top component which displays buttons containing URLs created
 * from instances of URL in the System Filesystem
 */
final class UrlFolderTopComponent extends TopComponent {

    private static UrlFolderTopComponent instance;
    private static final String PREFERRED_ID = "UrlFolderTopComponent";

    private UrlFolderTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(UrlFolderTopComponent.class, "CTL_UrlFolderTopComponent"));
        setToolTipText(NbBundle.getMessage(UrlFolderTopComponent.class, "HINT_UrlFolderTopComponent"));
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setLayout(new java.awt.FlowLayout());

    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link findInstance}.
     */
    public static synchronized UrlFolderTopComponent getDefault() {
        if (instance == null) {
            instance = new UrlFolderTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the UrlFolderTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized UrlFolderTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            ErrorManager.getDefault().log(ErrorManager.WARNING, "Cannot find UrlFolder component. It will not be located properly in the window system.");
            return getDefault();
        }
        if (win instanceof UrlFolderTopComponent) {
            return (UrlFolderTopComponent) win;
        }
        ErrorManager.getDefault().log(ErrorManager.WARNING, "There seem to be multiple components with the '" + PREFERRED_ID + "' ID. That is a potential source of errors and unexpected behavior.");
        return getDefault();
    }

    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }
    private static final String URLS_FOLDER_IN_SYSTEM_FILESYSTEM = "urls";

    public void componentOpened() {
        // TODO add custom code on component opening
        FileObject root = Repository.getDefault().getDefaultFileSystem().getRoot();
        FileObject urlsFolder = root.getFileObject(URLS_FOLDER_IN_SYSTEM_FILESYSTEM);
        DataFolder dataFolder = DataFolder.findFolder(urlsFolder);

        //Note if we were really doing this right, we would listen for
        //changes on the lookup
        FolderLookup lkp = new FolderLookup(dataFolder);

        //NetBeans 6 code
        //Collection <? extends URL> urls = lkp.getLookup().lookupAll(URL.class);

        //NetBeans 5.x and earlier:
        Collection<? extends URL> urls =
                lkp.getLookup().lookup(
                new Lookup.Template(URL.class)).allInstances();

        for (URL url : urls) {
            JButton jb = new JButton(new URLAction(url));
            add(jb);
        }
    }

    private static final class URLAction extends AbstractAction {

        private final URL url;

        URLAction(URL url) {
            this.url = url;
            putValue(NAME, url.toString());
        }

        public void actionPerformed(ActionEvent ae) {
            URLDisplayer.getDefault().showURL(url);
        }
    }

    public void componentClosed() {
        removeAll();
    }

    /** replaces this in object stream */
    public Object writeReplace() {
        return new ResolvableHelper();
    }

    protected String preferredID() {
        return PREFERRED_ID;
    }

    final static class ResolvableHelper implements Serializable {

        private static final long serialVersionUID = 1L;

        public Object readResolve() {
            return UrlFolderTopComponent.getDefault();
        }
    }
}
