/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.audioloader;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import javax.media.ControllerAdapter;
import javax.media.RealizeCompleteEvent;
import javax.media.bean.playerbean.MediaPlayer;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.text.CloneableEditorSupport;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.Mode;
import org.openide.windows.WindowManager;

// XXX override preferredID and getPersistenceType
public class AudioViewer extends CloneableTopComponent {
    private AudioDataObject obj;
    private MediaPlayer player;
    private static final long serialVersionUID = -4782743582015763601L;
    public AudioViewer() {}
    public AudioViewer(AudioDataObject obj) {
        super(/*obj*/);
        init(obj);
    }
    private void init(AudioDataObject obj) {
        this.obj = obj;
        setLayout(new BorderLayout());
        player = new MediaPlayer();
        FileObject fo = obj.getPrimaryFile();
        // Only file: URLs are accepted here. The following does not work:
        // player.setMediaLocation(NbfsURLConnection.encodeFileObject(fo).toString());
        File f = FileUtil.toFile(fo);
        if (f == null) {
            DialogDisplayer.getDefault().notify
                    (new NotifyDescriptor.Message(NbBundle.getBundle(AudioViewer.class)
                    .getString("MSG_local_only")));
            return;
        }
        player.setMediaLocation(f.toURI().toString());
        player.addControllerListener(new ControllerAdapter() {
            public void realizeComplete(RealizeCompleteEvent ev) {
                Component viz = player.getVisualComponent();
                if (viz != null) add(viz, BorderLayout.CENTER);
                Component ctrl = player.getControlPanelComponent();
                if (ctrl != null) add(ctrl, BorderLayout.SOUTH);
            }
        });
        add(player);
    }
    public boolean canClose() {
        boolean result = super.canClose();
        if (result) {
            player.stopAndDeallocate();
        }
        return result;
    }
    public HelpCtx getHelpCtx() {
        return new HelpCtx("org.netbeans.examples.modules.audioloader");
    }
    protected CloneableTopComponent createClonedObject() {
        return new AudioViewer(obj);
    }
    public void open() {
        super.open();
        Mode m = WindowManager.getDefault().findMode(CloneableEditorSupport.EDITOR_MODE);
        if (m != null) {
            m.dockInto(this);
        }
        player.start();
    }
    /**
     * @serialData Super, then store the AudioDataObject. */
    public void writeExternal(ObjectOutput oo) throws IOException {
        super.writeExternal(oo);
        oo.writeObject(obj);
    }
    /**
     * @serialData #see writeExternal */
    public void readExternal(ObjectInput oi) throws IOException, ClassNotFoundException {
        super.readExternal(oi);
        init((AudioDataObject) oi.readObject());
    }
}
