/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.customexplorer;

import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.openide.nodes.Node;

/**
 * This is the (somewhat contrived) business object that is represented by the
 * content of the ShapesDataObject.  It has nothing at all to do with NetBeans.
 * A ShapesBusinessObject contains some number of CustomRectangle obects, each
 * of which is represented by a child node of the ShapesDataNode.
 *
 *
 * @author twheeler
 */
public class ShapesBusinessObject implements Node.Cookie {
    // implements Node.Cookie so we can use CookieSet, for sake of simplicity,
    // instead of messing with InstanceContent and AbstractLookup.  Both ways
    // should work, but this one's easier if you don't mind the marker interface
    // in your code.

    private Set<CustomRectangle> rects;
    private List<ShapesContentListener> listeners;

    public ShapesBusinessObject() {
        rects = new HashSet<CustomRectangle>();
        listeners = new ArrayList<ShapesContentListener>();
    }

    public void add(CustomRectangle rect) {
        rects.add(rect);

        for (ShapesContentListener scl : listeners) {
            scl.shapesAdded();
        }
    }

    public boolean remove(CustomRectangle rect) {
        boolean wasRemoved = rects.remove(rect);

        if (wasRemoved) {
            for (ShapesContentListener scl : listeners) {
                scl.shapesRemoved();
            }
        }

        return wasRemoved;
    }

    public Set<CustomRectangle> getAllRectangles() {
        return Collections.unmodifiableSet(rects);
    }

    public void addShapesContentListener(ShapesContentListener scl) {
        listeners.add(scl);
    }

    public void removeShapesContentListener(ShapesContentListener scl) {
        listeners.remove(scl);
    }
}