/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.customexplorer.filetype;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import javax.swing.Action;
import org.openide.actions.CopyAction;
import org.openide.actions.CutAction;
import org.openide.actions.DeleteAction;
import org.openide.actions.PasteAction;
import org.openide.actions.PropertiesAction;
import org.openide.actions.RenameAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.lookup.Lookups;
import org.netbeans.examples.customexplorer.CustomRectangle;

/**
 *
 * @author twheeler
 */
public class RectangleNode extends AbstractNode {

    private CustomRectangle rect;

    private static final String IMAGE_ICON_BASE = "org/netbeans/examples/" +
            "customexplorer/filetype/rect.png";

    public RectangleNode(CustomRectangle rect) {
        super(Children.LEAF, Lookups.singleton(rect));
        setIconBaseWithExtension(IMAGE_ICON_BASE);
        this.rect = rect;
        setName(rect.getName());
    }

    @Override
    public Action[] getActions(boolean context) {
        return new Action[]{
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            SystemAction.get(PropertiesAction.class)
        };
    }

    @Override
    public Action getPreferredAction() {
        return SystemAction.get(PropertiesAction.class);
    }

    @Override
    protected Sheet createSheet() {
        Sheet s = super.createSheet();
        Sheet.Set ss = s.get(Sheet.PROPERTIES);
        if (ss == null) {
            ss = Sheet.createPropertiesSet();
            s.put(ss);
        }

        // add our properties
        ss.put(new XProperty());
        ss.put(new YProperty());
        ss.put(new WidthProperty());
        ss.put(new HeightProperty());

        // node names are a special case
        ss.put(new PropertySupport.Name(this));

        return s;
    }

    @Override
    public boolean canRename() {
        return true;
    }

    @Override
    public boolean canCut() {
        return true;
    }

    @Override
    public boolean canCopy() {
        return true;
    }

    @Override
    public boolean canDestroy() {
        return true;
    }

    private class XProperty extends PropertySupport.ReadWrite {

        @SuppressWarnings("unchecked")
        public XProperty() {
            super("X", double.class, "X", "X value for rectangle");
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return rect.getX();
        }

        public void setValue(Object o) {
            rect.setX((Double)o);
        }
    }

    @Override
    public Transferable clipboardCopy() throws IOException {
        System.out.println("clipboardCopy is called");

        Transferable deflt = super.clipboardCopy();
        ExTransferable enriched = ExTransferable.create(deflt);
        enriched.put(new ExTransferable.Single(CustomRectangle.DATA_FLAVOR) {

            @Override
            protected Object getData() throws IOException, UnsupportedFlavorException {
                return rect;
            }
        });

        System.out.println("clipboardCopy is finished");
        return enriched;
    }

    @Override
    public void destroy() throws IOException {
        super.destroy();
    }


    // NOTE: see NodeReaperAdapter in ShapesDataNode.ShapesChildren to see how
    // I am handling node deletion

    @Override
    public Transferable clipboardCut() throws IOException {
        System.out.println("clipboardCut is called");

        Transferable deflt = super.clipboardCopy();
        ExTransferable enriched = ExTransferable.create(deflt);
        enriched.put(new ExTransferable.Single(CustomRectangle.DATA_FLAVOR) {

            @Override
            protected Object getData() throws IOException, UnsupportedFlavorException {
                return rect;
            }
        });

        System.out.println("clipboardCut is finished");

        destroy();

        return enriched;
    }

    @Override
    public Transferable drag() throws IOException {
        System.out.println("drag is called");

        Transferable deflt = super.clipboardCopy();
        ExTransferable enriched = ExTransferable.create(deflt);
        enriched.put(new ExTransferable.Single(CustomRectangle.DATA_FLAVOR) {

            @Override
            protected Object getData() throws IOException, UnsupportedFlavorException {
                return rect;
            }
        });

        destroy();

        System.out.println("drag is finished");
        return enriched;
    }

    private class YProperty extends PropertySupport.ReadWrite {

        @SuppressWarnings("unchecked")
        public YProperty() {
            super("Y", double.class, "Y", "Y value for rectangle");
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return rect.getY();
        }

        public void setValue(Object o) {
            rect.setY((Double)o);
        }
    }

    private class WidthProperty extends PropertySupport.ReadWrite {

        public WidthProperty() {
            super("Width", double.class, "Width", "Width value for rectangle");
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return rect.getWidth();
        }

        public void setValue(Object o) {
            rect.setWidth((Double)o);
        }
    }

    private class HeightProperty extends PropertySupport.ReadWrite {

        public HeightProperty() {
            super("Height", double.class, "Height", "Height value for rectangle");
        }

        public Object getValue() throws IllegalAccessException, InvocationTargetException {
            return rect.getHeight();
        }

        public void setValue(Object o) {
            rect.setHeight((Double)o);
        }
    }
}