/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.Action;
import org.openide.ErrorManager;
import org.openide.actions.CopyAction;
import org.openide.actions.CutAction;
import org.openide.actions.DeleteAction;
import org.openide.actions.MoveDownAction;
import org.openide.actions.MoveUpAction;
import org.openide.actions.PasteAction;
import org.openide.actions.PropertiesAction;
import org.openide.actions.RenameAction;
import org.openide.actions.ToolsAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Index;
import org.openide.nodes.Node;
import org.openide.nodes.NodeEvent;
import org.openide.nodes.NodeListener;
import org.openide.nodes.NodeMemberEvent;
import org.openide.nodes.NodeOp;
import org.openide.nodes.NodeReorderEvent;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;
import org.openide.util.datatransfer.ExTransferable;
import org.openide.util.datatransfer.PasteType;

/** Node representing one note in a score.
 */
public class NoteNode extends AbstractNode implements NodeListener {
    private final Score.Note note;
    private boolean addedCookieListener;
    public NoteNode(Score.Note note) {
        super(Children.LEAF);
        this.note = note;
        super.setName(Score.TONES_SHORT[note.getTone()]);
        setShortDescription(NbBundle.getMessage(NoteNode.class, "HINT_note_node",
                            Score.TONES_LONG[note.getTone()],
                            Score.OCTAVES_LONG[note.getOctave()],
                            Score.DURATIONS_LONG[note.getDuration()]));
        setIconBaseWithExtension("org/netbeans/examples/modules/minicomposer/resources/NoteIcon.gif");
    }
    public Action[] getActions(boolean context) {
        return new Action[] {
            SystemAction.get(MoveUpAction.class),
            SystemAction.get(MoveDownAction.class),
            null,
            SystemAction.get(CutAction.class),
            SystemAction.get(CopyAction.class),
            SystemAction.get(PasteAction.class),
            null,
            SystemAction.get(DeleteAction.class),
            SystemAction.get(RenameAction.class),
            null,
            SystemAction.get(PropertiesAction.class),
            SystemAction.get(ToolsAction.class),
        };
    }
    public Node.Cookie getCookie(Class c) {
        if (c == Index.class || c == Index.Support.class) {
            return null;
        }
        Node parent = getParentNode();
        if (parent == null) return null;
        if (!addedCookieListener) {
            addedCookieListener = true;
            parent.addNodeListener(NodeOp.weakNodeListener(this, parent));
        }
        Node.Cookie cookie = parent.getCookie(c);
        return cookie;
    }
    public void propertyChange(PropertyChangeEvent evt) {
        if (Node.PROP_COOKIE.equals(evt.getPropertyName())) {
            fireCookieChange();
        }
    }
    public void childrenRemoved(NodeMemberEvent ev) {
        // ignore
    }
    public void childrenReordered(NodeReorderEvent ev) {
        // ignore
    }
    public void childrenAdded(NodeMemberEvent ev) {
        // ignore
    }
    public void nodeDestroyed(NodeEvent ev) {
        // ignore
    }
    public HelpCtx getHelpCtx() {
        return new HelpCtx("org.netbeans.examples.modules.minicomposer.explorer");
    }
    public boolean canRename() {
        return true;
    }
    public void setName(String nue) throws IllegalArgumentException {
        if (nue.equals(getName())) return;
        int v;
        for (v = 0; v < Score.TONES_SHORT.length; v++) {
            if (nue.equals(Score.TONES_SHORT[v])) break;
        }
        if (v == Score.TONES_SHORT.length) {
            IllegalArgumentException iae = new IllegalArgumentException("bad tone: " + nue);
            ErrorManager.getDefault().annotate(iae, ErrorManager.WARNING, null,
                NbBundle.getMessage(NoteNode.class, "EXC_bad_tone", nue), null, null);
            throw iae;
        }
        ScoreCookie cookie = (ScoreCookie)getCookie(ScoreCookie.class);
        try {
            Score s = cookie.getScore();
            int size = s.getSize();
            List notes = new ArrayList(Math.max(size, 1));
            for (int i = 0; i < size; i++) {
                Score.Note n = s.getNote(i);
                if (n == note) {
                    n = new Score.Note(v, n.getOctave(), n.getDuration());
                }
                notes.add(n);
            }
            cookie.setScore(new Score(notes));
        } catch (IOException ioe) {
            ErrorManager.getDefault().notify(ioe);
        }
    }
    public boolean canDestroy() {
        return true;
    }
    public void destroy() throws IOException {
        ScoreCookie cookie = (ScoreCookie)getCookie(ScoreCookie.class);
        if (cookie == null) throw new IOException();
        Score s = cookie.getScore();
        int size = s.getSize();
        List notes = new ArrayList(Math.max(size - 1, 1));
        for (int i = 0; i < size; i++) {
            Score.Note n = s.getNote(i);
            if (n != note) {
                notes.add(n);
            }
        }
        if (notes.size() != size - 1) throw new IOException("did not find note " + note + " among " + notes);
        cookie.setScore(new Score(notes));
    }
    public boolean canCopy() {
        return true;
    }
    public boolean canCut() {
        return true;
    }
    public Transferable clipboardCopy() throws IOException {
        Transferable deflt = super.clipboardCopy();
        ExTransferable enriched = ExTransferable.create(deflt);
        enriched.put(new ExTransferable.Single(NoteTransfer.SCORE_FLAVOR) {
            protected Object getData() {
                return new Score(Collections.singletonList(note));
            }
        });
        return enriched;
    }
    public Transferable clipboardCut() throws IOException {
        destroy();
        return clipboardCopy();
    }
    protected void createPasteTypes(Transferable t, List s) {
        super.createPasteTypes(t, s);
        PasteType p = NoteTransfer.createNotePasteType(t, (ScoreCookie)getCookie(ScoreCookie.class), note);
        if (p != null) {
            s.add(p);
        }
    }
    protected Sheet createSheet() {
        Sheet s = super.createSheet();
        Sheet.Set ss = Sheet.createPropertiesSet();
        ScoreCookie cookie = (ScoreCookie)getCookie(ScoreCookie.class);
        ss.put(new NoteProperty(cookie, note, 0));
        ss.put(new NoteProperty(cookie, note, 1));
        ss.put(new NoteProperty(cookie, note, 2));
        s.put(ss);
        return s;
    }
}
