/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.BeanInfo;
import java.beans.PropertyChangeEvent;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.ErrorManager;
import org.openide.loaders.DataObject;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Node;
import org.openide.nodes.NodeAdapter;
import org.openide.nodes.NodeListener;
import org.openide.nodes.NodeOp;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.Task;
import org.openide.util.TaskListener;
import org.openide.util.WeakListeners;
import org.openide.windows.CloneableTopComponent;

public class ScorePanel extends CloneableTopComponent implements MultiViewElement {
    private static final long serialVersionUID = 7204432764586558961L;
    private MultiDataObject.Entry entry;
    private ScoreCookie cookie;
    private Score score;
    private NodeListener listener;
    private final transient CloseOperationState cos;
    private transient MultiViewElementCallback callback;
    public ScorePanel(MultiDataObject.Entry entry, CloseOperationState cos) {
        this.entry = entry;
        this.cos = cos;
        init();
    }
    /** for externalization only */
    public ScorePanel() {
        cos = null; // XXX must be loaded from somewhere?
    }
    
    protected CloneableTopComponent createClonedObject() {
        return new ScorePanel(entry, cos);
    }
    /**
     * @serialData Super, then the MultiDataObject.Entry to represent. */
    public void writeExternal(ObjectOutput oo) throws IOException {
        // XXX is this really necessary??
        super.writeExternal(oo);
        oo.writeObject(entry);
    }
    /**
     * @serialData #see writeExternal */
    public void readExternal(ObjectInput oi) throws IOException, ClassNotFoundException {
        super.readExternal(oi);
        entry = (MultiDataObject.Entry)oi.readObject();
        init();
    }
    protected void updateNameAndIcon(DataObject o, Node n) {
        String displayName = n.getDisplayName();
        if (o.isModified()) {
            setName(NbBundle.getMessage(ScorePanel.class, "LBL_modified_name", displayName));
        } else {
            setName(displayName);
        }
        setIcon(n.getIcon(BeanInfo.ICON_COLOR_16x16));
    }
    public HelpCtx getHelpCtx() {
        return new HelpCtx("org.netbeans.examples.modules.minicomposer.visual");
    }
    private void init() {
        if (!SwingUtilities.isEventDispatchThread()) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    init();
                }
            });
            return;
        }
//        putClientProperty("PersistenceType", "OnlyOpened");
        // [PENDING] Should also fire Env.PROP_VALID when DataObject.PROP_VALID
        // changes. Presumably editor supports do this for you, somewhere, though
        // it is not clear where.
        final DataObject o = entry.getDataObject();
        final Node n = o.getNodeDelegate();
        updateNameAndIcon(o, n);
        listener = new NodeAdapter() {
            public void propertyChange(PropertyChangeEvent ev) {
                String prop = ev.getPropertyName();
                if (prop == null ||
                        prop.equals(Node.PROP_DISPLAY_NAME) ||
                        prop.equals(Node.PROP_ICON) ||
                        prop.equals(DataObject.PROP_MODIFIED)) {
                    updateNameAndIcon(o, n);
                }
            }
        };
        n.addNodeListener(NodeOp.weakNodeListener(listener, n));
        o.addPropertyChangeListener(WeakListeners.propertyChange(listener, o));
        setLayout(new BorderLayout());
        cookie = (ScoreCookie)entry.getDataObject().getCookie(ScoreCookie.class);
        if (cookie == null) {
            JLabel label = new JLabel(NbBundle.getMessage(ScorePanel.class, "LBL_cannot_load"));
            label.setHorizontalAlignment(SwingConstants.CENTER);
            add(label, BorderLayout.SOUTH);
            return;
        }
        Task t = cookie.prepare();
        t.addTaskListener(new DeferredRun());
        JLabel label = new JLabel(NbBundle.getMessage(ScorePanel.class, "LBL_loading"));
        label.setHorizontalAlignment(SwingConstants.CENTER);
        add(label, BorderLayout.SOUTH);
        // Just for safety:
        score = new Score(Collections.EMPTY_LIST);
    }
    private final class DeferredRun implements TaskListener, Runnable {
        public void taskFinished(Task t2) {
            SwingUtilities.invokeLater(this);
        }
        public void run() {
            init2();
        }
    }
    private void init2() {
        try {
            score = cookie.getScore();
        } catch (IOException ioe) {
            ErrorManager.getDefault().notify(ioe);
            return;
        }
        TableColumnModel columns = new DefaultTableColumnModel();
        TableColumn column = new TableColumn(0, 150, new PulldownRenderer(Score.TONES_LONG), new PulldownEditor(Score.TONES_LONG));
        column.setHeaderValue(NbBundle.getMessage(ScorePanel.class, "LBL_header_tone"));
        columns.addColumn(column);
        column = new TableColumn(1, 150, new PulldownRenderer(Score.OCTAVES_LONG), new PulldownEditor(Score.OCTAVES_LONG));
        column.setHeaderValue(NbBundle.getMessage(ScorePanel.class, "LBL_header_octave"));
        columns.addColumn(column);
        column = new TableColumn(2, 150, new PulldownRenderer(Score.DURATIONS_LONG), new PulldownEditor(Score.DURATIONS_LONG));
        column.setHeaderValue(NbBundle.getMessage(ScorePanel.class, "LBL_header_duration"));
        columns.addColumn(column);
        final JTable table = new JTable(new Model(), columns);
        removeAll();
        add(new JScrollPane(table), BorderLayout.CENTER);
        JButton add = new JButton(NbBundle.getMessage(ScorePanel.class, "LBL_add_row"));
        add.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                addRow();
            }
        });
        final JButton del = new JButton(NbBundle.getMessage(ScorePanel.class, "LBL_remove_row"));
        del.setEnabled(false);
        del.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                delRows(table.getSelectedRows());
            }
        });
        table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent ev) {
                del.setEnabled(table.getSelectedRows().length > 0);
            }
        });
        JPanel buttons = new JPanel();
        FlowLayout layout = new FlowLayout();
        layout.setAlignment(FlowLayout.CENTER);
        layout.setHgap(15);
        buttons.setLayout(layout);
        buttons.add(add);
        buttons.add(del);
        add(buttons, BorderLayout.SOUTH);
        revalidate();
    }
    private void addRow() {
        int size = score.getSize();
        List notes = new ArrayList(size + 1);
        for (int i = 0; i < size; i++) {
            notes.add(score.getNote(i));
        }
        notes.add(new Score.Note(Score.DEFAULT_TONE, Score.DEFAULT_OCTAVE, Score.DEFAULT_DURATION));
        try {
            cookie.setScore(new Score(notes));
        } catch (IOException ioe) {
            ErrorManager.getDefault().notify(ioe);
        }
    }
    private void delRows(int[] rows) {
        int size = score.getSize();
        int size2 = size - rows.length;
        List notes = new ArrayList(size2);
        KEEPROW: for (int i = 0; i < size; i++) {
            for (int j = 0; j < rows.length; j++) {
                if (rows[j] == i) {
                    continue KEEPROW;
                }
            }
            notes.add(score.getNote(i));
        }
        try {
            cookie.setScore(new Score(notes));
        } catch (IOException ioe) {
            ErrorManager.getDefault().notify(ioe);
        }
    }
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        // XXX necessary?
        this.callback = callback;
//        callback.updateTitle(getName());
    }
    public JComponent getVisualRepresentation() {
        return this;
    }

    public JComponent getToolbarRepresentation() {
        // XXX can put add/del buttons here
        return new JPanel();
    }
    public CloseOperationState canCloseElement() {
        if (entry.getDataObject().isModified()) {
           return cos;
        } else {
           return CloseOperationState.STATE_OK;
        }
    }
    public void componentDeactivated() {
        super.componentDeactivated();
    }
    public void componentActivated() {
        super.componentActivated();
    }
    public void componentHidden() {
        super.componentHidden();
    }
    public void componentShowing() {
        super.componentShowing();
    }
    public void componentClosed() {
        super.componentClosed();
    }
    public void componentOpened() {
        super.componentOpened();
    }
    private class Model extends AbstractTableModel implements ChangeListener {
        public Model() {
            cookie.addChangeListener(WeakListeners.change(this, cookie));
        }
        public void stateChanged(ChangeEvent ev) {
            try {
                score = cookie.getScore();
                fireTableDataChanged();
            } catch (IOException ioe) {
                // keep previous Score object instead
                if (entry.getDataObject().isValid()) {
                    ErrorManager.getDefault().notify(ioe);
                }
            }
        }
        public boolean isCellEditable(int row, int col) {
            return true;
        }
        public int getRowCount() {
            return score.getSize();
        }
        public int getColumnCount() {
            return 3;
        }
        public Object getValueAt(int row, int column) {
            if (column == 0)
                return new Integer(score.getNote(row).getTone());
            else if (column == 1)
                return new Integer(score.getNote(row).getOctave());
            else if (column == 2)
                return new Integer(score.getNote(row).getDuration());
            else
                throw new ArrayIndexOutOfBoundsException();
        }
        public void setValueAt(Object val, int row, int col) {
            if (val.equals(getValueAt(row, col))) {
                return;
            }
            int size = score.getSize();
            List notes = new ArrayList(size);
            for (int i = 0; i < size; i++) {
                Score.Note n = score.getNote(i);
                if (i == row) {
                    int x = ((Integer)val).intValue();
                    switch (col) {
                    case 0:
                        n = new Score.Note(x, n.getOctave(), n.getDuration());
                        break;
                    case 1:
                        n = new Score.Note(n.getTone(), x, n.getDuration());
                        break;
                    case 2:
                        n = new Score.Note(n.getTone(), n.getOctave(), x);
                        break;
                    default:
                        throw new ArrayIndexOutOfBoundsException();
                    }
                }
                notes.add(n);
            }
            Score nue = new Score(notes);
            try {
                cookie.setScore(nue);
            } catch (IOException ioe) {
                ErrorManager.getDefault().notify(ioe);
            }
        }
    }
    private static class PulldownRenderer extends DefaultTableCellRenderer {
        private String[] tokens;
        public PulldownRenderer(String[] tokens) {
            this.tokens = tokens;
        }
        protected void setValue(Object o) {
            int index = ((Integer)o).intValue();
            setText(tokens[index]);
        }
    }
    private static class PulldownEditor extends DefaultCellEditor {
        public PulldownEditor(String[] tokens) {
            super(makeComboBox(tokens));
        }
        private static JComboBox makeComboBox(final String[] tokens) {
            Object[] list = new Object[tokens.length];
            for (int i = 0; i < list.length; i++)
                list[i] = new Integer(i);
            JComboBox combo = new JComboBox(list);
            combo.setMaximumRowCount(list.length);
            combo.setRenderer(new DefaultListCellRenderer() {
                public Component getListCellRendererComponent(JList list2, Object value, int index, boolean isSelected, boolean cellHasFocus) {
                    int index2 = ((Integer)value).intValue();
                    return super.getListCellRendererComponent(list2, tokens[index2], index, isSelected, cellHasFocus);
                }
            });
            return combo;
        }
    }
}
