/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.modules.povfile;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.modules.examples.api.povray.MainFileProvider;
import org.netbeans.modules.examples.api.povray.RendererService;
import org.openide.ErrorManager;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 * A basic DataNode to represent .pov and .inc files - provided from
 * PovrayDataObject.getNodeDelegate();
 *
 * @author Tim Boudreau
 */
public class PovrayDataNode extends DataNode {

    private static final String IMAGE_ICON_BASE = "org/netbeans/examples/modules/povfile/povicon.gif";

    public PovrayDataNode(PovrayDataObject obj) {
        super(obj, Children.LEAF);
        setIconBaseWithExtension(IMAGE_ICON_BASE);
    }

    private FileObject getFile() {
        return getDataObject().getPrimaryFile();
    }

    private<T> T getFromProject (Class<T> clazz) {
        T result;
        Project p = FileOwnerQuery.getOwner(getFile());
        if (p != null) {
            result = p.getLookup().lookup (clazz);
        } else {
            result = null;
        }
        return result;
    }

    private boolean isMainFile() {
        MainFileProvider prov = 
            getFromProject (MainFileProvider.class);
        boolean result;
        if (prov == null) {
            result = false;
        } else {
            FileObject myFile = getFile();
            result = prov.isMainFile(myFile);
        }
        return result;
    }

    public String getHtmlDisplayName() {
        return isMainFile() ? "<b>" + getDisplayName() + "</b>" : null;
    }

    public Action[] getActions (boolean popup) {
        Action[] actions = super.getActions(popup);
        RendererService renderer = getFromProject (RendererService.class);
        Action[] result;
        if (renderer != null && actions.length > 0) { //should always be > 0
            Action rendererAction = new RendererAction (renderer, this);
            result = new Action[ actions.length + 2 ];
            result[0] = actions[0];
            result[1] = new SetMainFileAction();
            result[2] = rendererAction;
            System.arraycopy(actions, 1, result, 3, actions.length - 1);
        } else {
            //Isolated file in the favorites window or something
            result = actions;
        }
        return result;
    }


    private class SetMainFileAction extends AbstractAction {
        public SetMainFileAction() {
            putValue (NAME, NbBundle.getMessage(PovrayDataNode.class,
                    "LBL_MainFileAction"));
        }

        public void actionPerformed(ActionEvent e) {
            MainFileProvider prov = 
                getFromProject (MainFileProvider.class);
            if (prov != null) {
                FileObject oldMainFile = prov.getMainFile();
                prov.setMainFile(getFile());
                fireDisplayNameChange(getDisplayName(), 
                        getHtmlDisplayName());
                if (oldMainFile != null) {
                    DataObject ob;
                    try {
                        ob = DataObject.find(oldMainFile);
                        Node n = ob.getNodeDelegate();
                        if (n instanceof PovrayDataNode) {
                            ((PovrayDataNode)n).fireDisplayNameChange(null, 
                                    n.getDisplayName());
                        }
                    } catch (DataObjectNotFoundException ex) {
                        //Shouldn't happen
                        ErrorManager.getDefault().notify (ex);
                    }
                }
            }
        }

        public boolean isEnabled() {
            return !isMainFile() && getFromProject(MainFileProvider.class) !=
                    null;
        }
    }
}
