/*
DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.


The contents of this file are subject to the terms of either the GNU
General Public License Version 2 only ("GPL") or the Common
Development and Distribution License("CDDL") (collectively, the
"License"). You may not use this file except in compliance with the
License. You can obtain a copy of the License at
http://www.netbeans.org/cddl-gplv2.html
or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
specific language governing permissions and limitations under the
License.  When distributing the software, include this License Header
Notice in each file and include the License file at
nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
particular file as subject to the "Classpath" exception as provided
by Sun in the GPL Version 2 section of the License file that
accompanied this code. If applicable, add the following below the
License Header, with the fields enclosed by brackets [] replaced by
your own identifying information:
"Portions Copyrighted [year] [name of copyright owner]"

Contributor(s):

The Original Software is NetBeans. The Initial Developer of the Original
Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
Microsystems, Inc. All Rights Reserved.

If you wish your version of this file to be governed by only the CDDL
or only the GPL Version 2, indicate your decision by adding
"[Contributor] elects to include this software in this distribution
under the [CDDL or GPL Version 2] license." If you do not indicate a
single choice of license, a recipient has the option to distribute
your version of this file under either the CDDL, the GPL Version 2 or
to extend the choice of license to its licensees as provided above.
However, if you add GPL Version 2 code and therefore, elected the GPL
Version 2 license, then the option applies only if the new code is
made subject to such option by the copyright holder.
*/
package org.netbeans.modules.snipe;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.AbstractAction;
import javax.swing.text.DefaultEditorKit;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;

import org.openide.actions.*;
import org.openide.awt.UndoRedo;
import org.openide.util.SharedClassObject;
import org.openide.util.Utilities;
import org.openide.windows.OutputWriter;
import org.openide.windows.IOProvider;
import org.openide.windows.CloneableTopComponent;
import org.openide.windows.TopComponent;

/** 
 * A component for displaying/editing snipes.
 *
 * @author  Rich Unger
 */
public class SnipeTopComponent extends CloneableTopComponent 
{
    protected SnipeEditor m_snipeEditor;

    public SnipeTopComponent() 
    {
        super();
        initComponents();
    }
    
    public SnipeTopComponent(SnipeDataObject dobj)
    {
        super();
        initComponents();
        initData(dobj);
    }

    protected void initData(SnipeDataObject dobj)
    { 
        m_snipeEditor.setSnipe(dobj);
        TopComponent.NodeName.connect(this, dobj.getNodeDelegate());
    } 
    
    public SnipeEditor getSnipeEditor()
    {
        return m_snipeEditor;
    }

    public Image getIcon()
    {
        return Utilities.loadImage ("org/netbeans/modules/snipe/resources/snipe16.gif");
    }
    
    public void open()
    {
        super.open();
        if(m_snipeEditor != null)
            m_snipeEditor.open();
    }
    
    // PERSISTENCE
    // If you wish to keep any state between IDE restarts, put it here:
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException 
    {
        super.readExternal (in);
        SnipeDataObject snipe = (SnipeDataObject)in.readObject();
        initComponents();
        initData(snipe);
    }

    public void writeExternal (ObjectOutput out) throws IOException 
    {
        super.writeExternal (out);
        out.writeObject (m_snipeEditor.getSnipe());
    }

    public int getPersistenceType() { 
        return PERSISTENCE_ONLY_OPENED;
    } 

    protected String preferredID() { 
        return "snipe" + Math.random();
    } 
    
    /** To enable cloning of the top component */
    protected CloneableTopComponent createClonedObject()
    {
        return new SnipeTopComponent(m_snipeEditor.getSnipe());
    }

    /**
     * Called when the last instance of a SnipeTopComponent for a particular
     * SnipeDataObject is about to be closed.  If the object is not saved, the user
     * is presented with a dialog.  Then the SnipeDataObject is closed and taken out of
     * memory.
     */
    protected boolean closeLast()
    {
        return m_snipeEditor.close();
    }

    protected void initComponents()
    { 
        // lay out the JPanel
        setLayout(new BorderLayout());
        m_snipeEditor = new SnipeEditor();
        add(m_snipeEditor.getPanel(), BorderLayout.CENTER);

        // hook into the netbeans Edit menu
        Action cutPerformer = new SnipeCutAction();
        Action copyPerformer = new SnipeCopyAction();
        Action pastePerformer = new SnipePasteAction();
        Action deletePerformer = new SnipeDeleteAction();

        ActionMap map = this.getActionMap();

        map.put(DefaultEditorKit.copyAction, copyPerformer);
        map.put(DefaultEditorKit.cutAction, cutPerformer);
        map.put(DefaultEditorKit.pasteAction, pastePerformer);
        map.put("delete", deletePerformer);
    } 

    public UndoRedo getUndoRedo() { 
        return new SnipeUndoRedo();
    } 

    private void log(String str) { 
        OutputWriter out = IOProvider.getDefault().getStdOut();
        out.println(str);
        out.flush();
    } 

    public class SnipeDeleteAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Delete the Snipe");
        } 
    } 
    
    public class SnipeCutAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Cut the Snipe");
        } 
    } 
    
    public class SnipeCopyAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Copy the Snipe");
        } 
    } 
    
    public class SnipePasteAction extends AbstractAction { 
        public void actionPerformed(ActionEvent evt) { 
            log("Paste the Snipe");
        } 
    } 

    public class SnipeUndoRedo extends UndoRedo.Manager { 
        public String getUndoPresentationName() { 
            return "snipe edit";
        } 

        public String getRedoPresentationName() { 
            return "snipe edit";
        } 

        public boolean canUndo() { 
            return true;
        } 

        public boolean canRedo() { 
            return true;
        } 

        public void undo() throws CannotUndoException { 
            log("Undo the Snipe");
        } 

        public void redo() throws CannotRedoException { 
            log("Redo the Snipe");
        } 
    } 
    
    private static final long serialVersionUID = -4406860224835839120L;
}
