/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.ui;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;

/**
 * This converter is used to return a more human-readable value for the size of a file.
 * @author David Botterill
 */
public class SizeConverter implements Converter {
    
    /** 
     * Creates a new instance of SizeConverter 
     */
    public SizeConverter() {
    }
    
    /**
     * Convert the specified String into a model data object associated to the 
     * specified UIComponent.
     * @param context - FacesContext for the request being processed
     * @param component - UIComponent the Object is to be associated with
     * @param value - The String value to convert
     * @return - String value to be converted (may be null)
     */
    public Object getAsObject(FacesContext context, UIComponent component, String value) {
        return value;
    }
    
    /**
     * Convert the specified model object value, which is associated with the specified
     * UIComponent, into a String that is suitable for being included in the response 
     * generated during the Render Response phase of the request processing lifeycle.
     * @param context FacesContext for the request being processed
     * @param component UIComponent with which this model object value is associated
     * @param value Model object value to be converted (may be null)
     * @return a zero-length String if value is null, otherwise the result of the conversion
     */
    public String getAsString(FacesContext context, UIComponent component, Object value) {
        if(null == value) return "";
        if(value instanceof Integer) {
            String suffix="";
            int size = ((Integer)value).intValue();
            if(size > 1048576) {
                suffix = "MB";
                size = size/1048576;
                return Integer.toString(size) + " " + suffix;
            } else if(size > 1024) {
                suffix = "KB";
                size = size/1024;
                return Integer.toString(size) + " " + suffix;
            } else {
                suffix="Bytes";
                return Integer.toString(size) + " " + suffix;
            }
            
        } else if(value instanceof Long) {
            String suffix="";
            long size = ((Long)value).longValue();
            if(size > 1048576L) {
                suffix = "MB";
                size = size/1048576L;
                return Long.toString(size) + " " + suffix;
            } else if(size > 1024L) {
                suffix = "KB";
                size = size/1024L;
                return Long.toString(size) + " " + suffix;
            } else {
                suffix="Bytes";
                return Long.toString(size) + " " + suffix;
            }
            
        } else return value.toString();
    }
    
}
