/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.data;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import org.netbeans.pluginportal.system.Plugin;

/**
 * Entity class PluginImpl
 *
 * @author David Botterill
 */
@Entity
public class PluginImpl implements Serializable, Plugin {
    
    @OneToMany(cascade=CascadeType.ALL, mappedBy="plugin")
    private Collection<RatingImpl> ratings;
    @OneToMany(cascade=CascadeType.ALL, mappedBy = "plugin")
    private Collection<CommentImpl> comments;
    @OneToMany(cascade=CascadeType.ALL, mappedBy = "plugin")
    private Collection<OtherCategoryImpl> othercategories;
    
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long pluginid;
    
    /**
     * Creates a new instance of PluginImpl
     */
    public PluginImpl() {
    }
    
    /**
     * Gets the id of this PluginImpl.
     *
     * @return the id
     */
    public Long getPluginid() {
        return this.pluginid;
    }
    
    /**
     * Sets the id of this PluginImpl to the specified value.
     *
     * @param id the new id
     */
    public void setPluginid(Long pluginid) {
        this.pluginid = pluginid;
    }
    
    public Collection<RatingImpl> getRatings() {
        return ratings;
    }
    
    /**
     * Returns a hash code value for the object.  This implementation computes
     * a hash code value based on the id fields in this object.
     * @return a hash code value for this object.
     */
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.pluginid != null ? this.pluginid.hashCode() : 0);
        return hash;
    }
    
    /**
     * Determines whether another object is equal to this PluginImpl.  The result is
     * <code>true</code> if and only if the argument is not null and is a PluginImpl object that
     * has the same id field values as this object.
     *
     * @param object the reference object with which to compare
     * @return <code>true</code> if this object is the same as the argument;
     * <code>false</code> otherwise.
     */
    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof PluginImpl)) {
            return false;
        }
        PluginImpl other = (PluginImpl)object;
        if (this.pluginid != other.pluginid && (this.pluginid == null || !this.pluginid.equals(other.pluginid))) return false;
        return true;
    }
    
    /**
     * Returns a string representation of the object.  This implementation constructs
     * that representation based on the id fields.
     * @return a string representation of the object.
     */
    @Override
    public String toString() {
        return "org.netbeans.pluginportal.data.Plugin[id=" + pluginid + "]";
    }
    
    /**
     * Holds value of property plugin_name.
     */
    private String plugin_name;
    
    /**
     * Getter for property name.
     * @return Value of property name.
     */
    public String getPlugin_name() {
        return this.plugin_name;
    }
    
    /**
     * Setter for property name.
     * @param name New value of property name.
     */
    public void setPlugin_name(String plugin_name) {
        this.plugin_name = plugin_name;
    }
    
    /**
     * Holds value of property description.
     */
    @Column(length=2000)
    private String description;
    
    /**
     * Getter for property description.
     * @return Value of property description.
     */
    public String getDescription() {
        return this.description;
    }
    
    /**
     * Setter for property description.
     * @param description New value of property description.
     */
    public void setDescription(String description) {
        this.description = description;
    }
    
    /**
     * Holds value of property summary.
     */
    @Column(length=2000)
    private String summary;
    
    /**
     * Getter for property summary.
     * @return Value of property summary.
     */
    public String getSummary() {
        return this.summary;
    }
    
    /**
     * Setter for property summary.
     * @param summary New value of property summary.
     */
    public void setSummary(String summary) {
        this.summary = summary;
    }
    
    /**
     * Holds value of property home_page_url.
     */
    private String home_page_url;
    
    /**
     * Getter for property home_page_url.
     * @return Value of property home_page_url.
     */
    public String getHome_page_url() {
        return this.home_page_url;
    }
    
    /**
     * Setter for property home_page_url.
     * @param home_page_url New value of property home_page_url.
     */
    public void setHome_page_url(String home_page_url) {
        this.home_page_url = home_page_url;
    }
    
    /**
     * Holds value of property keywords.
     */
    private String keywords;
    
    /**
     * Getter for property keywords.
     * @return Value of property keywords.
     */
    public String getKeywords() {
        return this.keywords;
    }
    
    /**
     * Setter for property keywords.
     * @param keywords New value of property keywords.
     */
    public void setKeywords(String keywords) {
        this.keywords = keywords;
    }
    
    /**
     * Holds value of property author_userid.
     */
    private String author_userid;
    
    /**
     * Getter for property author_userid.
     * @return Value of property author_userid.
     */
    public String getAuthor_userid() {
        return this.author_userid;
    }
    
    /**
     * Setter for property author_userid.
     * @param author_userid New value of property author_userid.
     */
    public void setAuthor_userid(String author_userid) {
        this.author_userid = author_userid;
    }
    
    /**
     * Holds value of property license_type.
     */
    private String license_type;
    
    /**
     * Getter for property license_type.
     * @return Value of property license_type.
     */
    public String getLicense_type() {
        return this.license_type;
    }
    
    /**
     * Setter for property license_type.
     * @param license_type New value of property license_type.
     */
    public void setLicense_type(String license_type) {
        this.license_type = license_type;
    }
    
    /**
     * Holds value of property target_platforms.
     */
    private String target_platforms;
    
    /**
     * Getter for property target_platforms.
     * @return Value of property target_platforms.
     */
    public String getTarget_platforms() {
        return this.target_platforms;
    }
    
    /**
     * Setter for property target_platforms.
     * @param target_platforms New value of property target_platforms.
     */
    public void setTarget_platforms(String target_platforms) {
        this.target_platforms = target_platforms;
    }
    
    /**
     * Holds value of property versions_supported.
     */
    private String versions_supported;
    
    /**
     * Getter for property versions_supported.
     * @return Value of property versions_supported.
     */
    public String getVersions_supported() {
        return this.versions_supported;
    }
    
    /**
     * Setter for property versions_supported.
     * @param versions_supported New value of property versions_supported.
     */
    public void setVersions_supported(String versions_supported) {
        this.versions_supported = versions_supported;
    }
    
    /**
     * Holds value of property downloads.
     */
    private Long downloads;
    
    /**
     * Getter for property downloads.
     * @return Value of property downloads.
     */
    public Long getDownloads() {
        return this.downloads;
    }
    
    /**
     * Setter for property downloads.
     * @param downloads New value of property downloads.
     */
    public void setDownloads(Long downloads) {
        this.downloads = downloads;
    }
    
    /**
     * Holds value of property date_added.
     */
    @Temporal(TemporalType.DATE)
    private java.util.Date date_added;
    
    /**
     * Getter for property date_added.
     * @return Value of property date_added.
     */
    public java.util.Date getDate_added() {
        return this.date_added;
    }
    
    /**
     * Setter for property date_added.
     * @param date_added New value of property date_added.
     */
    public void setDate_added(java.util.Date date_added) {
        this.date_added = date_added;
    }
    
    /**
     * Holds value of property date_last_updated.
     */
    @Temporal(TemporalType.DATE)
    private java.util.Date date_last_updated;
    
    /**
     * Getter for property date_last_updated.
     * @return Value of property date_last_updated.
     */
    public java.util.Date getDate_last_updated() {
        return this.date_last_updated;
    }
    
    /**
     * Setter for property date_last_updated.
     * @param date_last_updated New value of property date_last_updated.
     */
    public void setDate_last_updated(java.util.Date date_last_updated) {
        this.date_last_updated = date_last_updated;
    }
    
    /**
     * Holds value of property nbm_filename.
     */
    private String nbm_filename;
    
    /**
     * Getter for property nbm_filename.
     * @return Value of property nbm_filename.
     */
    public String getNbm_filename() {
        return this.nbm_filename;
    }
    
    /**
     * Setter for property nbm_filename.
     * @param nbm_filename New value of property nbm_filename.
     */
    public void setNbm_filename(String nbm_filename) {
        this.nbm_filename = nbm_filename;
    }
    
    /**
     * Holds value of property quality_level.
     */
    private String quality_level;
    
    /**
     * Getter for property quality_level.
     * @return Value of property quality_level.
     */
    public String getQuality_level() {
        return this.quality_level;
    }
    
    /**
     * Setter for property quality_level.
     * @param quality_level New value of property quality_level.
     */
    public void setQuality_level(String quality_level) {
        this.quality_level = quality_level;
    }
    
    /**
     * Holds value of property version.
     */
    private String version;
    
    /**
     * Getter for property version.
     * @return Value of property version.
     */
    public String getVersion() {
        return this.version;
    }
    
    /**
     * Setter for property version.
     * @param version New value of property version.
     */
    public void setVersion(String version) {
        this.version = version;
    }
    
    /**
     * Holds value of property nbm_url.
     */
    private String nbm_url;
    
    /**
     * Getter for property nbm_url.
     * @return Value of property nbm_url.
     */
    public String getNbm_url() {
        return this.nbm_url;
    }
    
    /**
     * Setter for property nbm_url.
     * @param nbm_url New value of property nbm_url.
     */
    public void setNbm_url(String nbm_url) {
        this.nbm_url = nbm_url;
    }
    
    /**
     * Holds value of property categoryid.
     */
    private int categoryid;
    
    /**
     * Getter for property categoryid.
     * @return Value of property categoryid.
     */
    public int getCategoryid() {
        return this.categoryid;
    }
    
    /**
     * Setter for property categoryid.
     * @param categoryid New value of property categoryid.
     */
    public void setCategoryid(int categoryid) {
        this.categoryid = categoryid;
    }
    
    public Collection getComments() {
        return comments;
    }
    
    /**
     * Holds value of property download_size.
     */
    private Long download_size;
    
    /**
     * Getter for property download_size.
     * @return Value of property download_size.
     */
    public Long getDownload_size() {
        return this.download_size;
    }
    
    /**
     * Setter for property download_size.
     * @param download_size New value of property download_size.
     */
    public void setDownload_size(Long download_size) {
        this.download_size = download_size;
    }
    
    /**
     * Holds value of property image_filename.
     */
    private String image_filename;
    
    /**
     * Getter for property image_filename.
     * @return Value of property image_filename.
     */
    public String getImage_filename() {
        return this.image_filename;
    }
    
    /**
     * Setter for property image_filename.
     * @param image_filename New value of property image_filename.
     */
    public void setImage_filename(String image_filename) {
        this.image_filename = image_filename;
    }
    
    public Collection getOthercategories() {
        return othercategories;
    }
    
    public void setOthercategories(Collection othercategories) {
        this.othercategories = othercategories;
    }

    /**
     * Holds value of property new_version_comments.
     */
    private String new_version_comments;

    /**
     * Getter for property new_version_comments.
     * @return Value of property new_version_comments.
     */
    public String getNew_version_comments() {
        return this.new_version_comments;
    }

    /**
     * Setter for property new_version_comments.
     * @param new_version_comments New value of property new_version_comments.
     */
    public void setNew_version_comments(String new_version_comments) {
        this.new_version_comments = new_version_comments;
    }

    /**
     * Holds value of property average_rating.
     */
    private Double average_rating;

    /**
     * Getter for property average_rating.
     * @return Value of property average_rating.
     */
    public Double getAverage_rating() {
        return this.average_rating;
    }

    /**
     * Setter for property average_rating.
     * @param average_rating New value of property average_rating.
     */
    public void setAverage_rating(Double average_rating) {
        this.average_rating = average_rating;
    }

    /**
     * Holds value of property price.
     */
    private String price;

    /**
     * Getter for property price.
     * @return Value of property price.
     */
    public String getPrice() {
        return this.price;
    }

    /**
     * Setter for property price.
     * @param price New value of property price.
     */
    public void setPrice(String price) {
        this.price = price;
    }

    /**
     * Holds value of property fullsize_image_filename.
     */
    private String fullsize_image_filename;

    /**
     * Getter for property fullsize_image_filename.
     * @return Value of property fullsize_image_filename.
     */
    public String getFullsize_image_filename() {
        return this.fullsize_image_filename;
    }

    /**
     * Setter for property fullsize_image_filename.
     * @param fullsize_image_filename New value of property fullsize_image_filename.
     */
    public void setFullsize_image_filename(String fullsize_image_filename) {
        this.fullsize_image_filename = fullsize_image_filename;
    }

    /**
     * Holds value of property displaycategory.
     */
    private String displaycategory;

    /**
     * Getter for property displaycategory.
     * @return Value of property displaycategory.
     */
    public String getDisplaycategory() {
        return this.displaycategory;
    }

    /**
     * Setter for property displaycategory.
     * @param displaycategory New value of property displaycategory.
     */
    public void setDisplaycategory(String displaycategory) {
        this.displaycategory = displaycategory;
    }

    
}
