/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Query;
import org.netbeans.pluginportal.data.DisplayCategoryImpl;
import org.netbeans.pluginportal.system.CategoryCount;
import org.netbeans.pluginportal.system.Plugin;

/**
 *
 * @author David Botterill
 */
public class CategoryCountCache extends AbstractCache {
    
    private ArrayList<CategoryCount> countCache;
    private EntityManagerFactory emf;
    /** Creates a new instance of CategoryCountCache */
    public CategoryCountCache(EntityManagerFactory inEmf) {
        emf = inEmf;
        loadList();
        
    }
    
    protected synchronized void loadList() {
        loadDisplayCategories();
    }
    
    protected ArrayList<CategoryCount> getCache() {
        return getCountCache();
    }
    
    /**
     * This method loads the Display Categories that are shown in a module project.
     * These categories are stored in the displaycategoryimpl type.
     */
    private synchronized void loadDisplayCategories() {
        EntityManager em = emf.createEntityManager();
//        String select = "select id,displaycategory_name,p_info.categoryid,p_info.cat_count from displaycategoryimpl c," +
//                " (SELECT  categoryid, count(categoryid) as cat_count from pluginimpl p group by categoryid)as p_info" +
//                " where c.id = p_info.categoryid order by displaycategory_name";
//        Query query = em.createNativeQuery(select);
//        List resultList = query.getResultList();
//        Iterator resultIterator = resultList.iterator();
//        Vector currentResult = null;
        ArrayList<CategoryCount> newCache = new ArrayList();
//        while(resultIterator.hasNext()) {
//            currentResult = (Vector)resultIterator.next();
//            String categoryName = (String)currentResult.get(1); //displaycategory_name
//            Integer categoryCount = (Integer)currentResult.get(3); //displaycategory_name
//            CategoryCount currentRecord = new CategoryCount(categoryName, categoryCount.intValue());
//            newCache.add(currentRecord);
//        }
        
        String select = "select distinct o.categoryname, cc.cat_count from othercategoryimpl o," +
                " (select distinct categoryname,count(categoryname) as cat_count from othercategoryimpl cc group by categoryname) as cc" +
                " where o.categoryname=cc.categoryname order by o.categoryname";
        Query query = em.createNativeQuery(select);
        List resultList = query.getResultList();
        Iterator resultIterator = resultList.iterator();
        Vector currentResult = null;
        while(resultIterator.hasNext()) {
            currentResult = (Vector)resultIterator.next();
            String categoryName = (String)currentResult.get(0); //othercategory_name
            Integer categoryCount = (Integer)currentResult.get(1); //displaycategory_name
            CategoryCount currentRecord = new CategoryCount(categoryName, categoryCount.intValue());
            newCache.add(currentRecord);
        }
        
        setCountCache(newCache);
        
    }
    
    public ArrayList<CategoryCount> getCountCache() {
        return countCache;
    }
    
    public synchronized void setCountCache(ArrayList<CategoryCount> countCache) {
        this.countCache = countCache;
    }
    
    
    
}
