/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.qreporter;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.util.StringTokenizer;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;

/**
 * @author mk97936
 */
public class OracleReporterTask extends Task {
    
    private String [] components = null;
    private String [] subcomponents = null;
    private String [] versions = null;
    private String [] keywords = null;
    private String dateFrom;
    private String dateTill;
    private boolean enhancements;
    private boolean features;    
    private boolean tasks;
    private boolean patches;
    private boolean hiprios;
    private boolean listassignees;
    private File outputFile;
    
    public void setEnhancements(boolean b) {
        enhancements = b;
    }
    
    public void setFeatures(boolean b) {
        features = b;
    }
    
    public void setTasks(boolean b) {
        tasks = b;
    }
    
    public void setPatches(boolean b) {
        patches = b;
    }
    
    public void setListassignees(boolean b) {
        listassignees = b;
    }
    
    public void setHiprios(boolean b) {
        hiprios = b;
    }
    
    public void setListp12bugs(boolean b) {
        hiprios = b;
    }
    
    public void setModules(String s) {
        StringTokenizer st = new StringTokenizer(s, ",");
        if (st.countTokens() > 0) {
            components = new String[st.countTokens()];
            int index = 0;
            while (st.hasMoreTokens()) {
                components[index++] = st.nextToken().trim();
            }
        }
    }
    
    public void setSubcomponents(String s) {
        StringTokenizer st = new StringTokenizer(s, ",");
        if (st.countTokens() > 0) {
            subcomponents = new String[st.countTokens()];
            int index = 0;
            while (st.hasMoreTokens()) {
                subcomponents[index++] = st.nextToken().trim();
            }
        }
    }
    
    public void setVersions(String s) {
        StringTokenizer st = new StringTokenizer(s, ",");
        if (st.countTokens() > 0) {
            versions = new String[st.countTokens()];
            int index = 0;
            while (st.hasMoreTokens()) {
                versions[index++] = st.nextToken().trim();
            }
        }
    }
    
    public void setKeywords(String s) {
        StringTokenizer st = new StringTokenizer(s, ",");
        if (st.countTokens() > 0) {
            keywords = new String[st.countTokens()];
            int index = 0;
            while (st.hasMoreTokens()) {
                keywords[index++] = st.nextToken().trim();
            }
        }
    }
        
    public void setDateFrom(String s) {
        dateFrom = s;
    }
    
    public void setDateTill(String s) {
        dateTill = s;
    }
    
    public void setOutputfile(File f) {
        outputFile = f;
    }
    
    public void execute() throws BuildException {
        
        OracleReporter reporter = new OracleReporter();
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new FileOutputStream(outputFile));
        } catch (FileNotFoundException fnfe) {
            throw new BuildException(fnfe, location);
        }
        reporter.setOutput(pw, true);
        reporter.setDates(dateFrom, dateTill);
        reporter.setComponents(components);
        reporter.setSubcomponents(subcomponents);
        reporter.setVersions(versions);
        reporter.setKeywords(keywords);
        reporter.listEnhancements(enhancements);
        reporter.listTasks(tasks);
        reporter.listFeatures(features);
        reporter.listPatches(patches);
        reporter.listAssignees(listassignees);
        reporter.listHiprios(hiprios);
        reporter.getReport();
        
        pw.close();
        
    }
    
}
