/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.reports;

import java.util.*;

/**
 * Activity of some issue. Each item of this <code>List</code>
 * is instance of the class Change.
 *
 * @author Jan Stola
 */
public class IssueActivity extends LinkedList {
    
    /**
     * Determines whether the priority of the issue has changed since
     * given <code>date</code>.
     *
     * @param date reference date
     * @return change of the priority since given <code>date</code>.
     */
    int priorityChangeSince(Date date) {
        int oldPriority=0;
        int newPriority=0;
        Iterator iter=iterator();
        while (iter.hasNext()) {
            Change ch=(Change)iter.next();
            if (date.before(ch.when)&&(ch.what.equals(IssueQuery.PRIORITY))) {
                if (oldPriority==0) {
                    if (ch.oldValue.equals(IssueQuery.PRIORITY_P1)) {
                        oldPriority=1;
                    } else if (ch.oldValue.equals(IssueQuery.PRIORITY_P2)) {
                        oldPriority=2;
                    } else if (ch.oldValue.equals(IssueQuery.PRIORITY_P3)) {
                        oldPriority=3;
                    } else if (ch.oldValue.equals(IssueQuery.PRIORITY_P4)) {
                        oldPriority=4;
                    } else if (ch.oldValue.equals(IssueQuery.PRIORITY_P5)) {
                        oldPriority=5;
                    }
                }
                if (ch.newValue.equals(IssueQuery.PRIORITY_P1)) {
                    newPriority=1;
                } else if (ch.newValue.equals(IssueQuery.PRIORITY_P2)) {
                    newPriority=2;
                } else if (ch.newValue.equals(IssueQuery.PRIORITY_P3)) {
                    newPriority=3;
                } else if (ch.newValue.equals(IssueQuery.PRIORITY_P4)) {
                    newPriority=4;
                } else if (ch.newValue.equals(IssueQuery.PRIORITY_P5)) {
                    newPriority=5;
                }                
            }
        }
        return newPriority-oldPriority;
    }
    
}
