/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.qa.reports;

/**
 * Distribution of issues into several categories.
 *
 * @author Jan Stola
 */
public class IssueCounts {
    /** CQI value of P1 bug. */
    private static final int CQI_P1=100;
    /** CQI value of P2 bug. */
    private static final int CQI_P2=33;
    /** CQI value of P3 bug. */
    private static final int CQI_P3=10;
    /** CQI value of P4 bug. */
    private static final int CQI_P4=4;
    /** CQI value of P5 bug. */
    private static final int CQI_P5=2;
    /** Counts of issues by type. */
    public int[] types=new int[5];
    /** Counts of issues by status. */
    public int[] statuses=new int[7];
    /** Counts of issues by resolution. */
    public int[] resolutions=new int[8];
    /** Counts of issues by priorities. */
    public int[] priorities=new int[5];    
    
    /**
     * Returns number of issues with given <code>type</code>.
     *
     * @param type mask specifying type (see {@link Issue.TYPE_DEFECT} etc.).
     * @return number of issues with given <code>type</code>.
     */
    public int getCountByType(int type) {
        int count=0;
        
        if ((type&Issue.TYPE_DEFECT)!=0) count+=types[0];
        if ((type&Issue.TYPE_ENHANCEMENT)!=0) count+=types[1];
        if ((type&Issue.TYPE_FEATURE)!=0) count+=types[2];
        if ((type&Issue.TYPE_TASK)!=0) count+=types[3];
        if ((type&Issue.TYPE_PATCH)!=0) count+=types[4];
        return count;
    }
    
    /**
     * Returns number of issues with given <code>status</code>.
     *
     * @param status mask specifying status (see {@link Issue.STATUS_NEW} etc.).
     * @return number of issues with given <code>status</code>.
     */
    public int getCountByStatus(int status) {
        int count=0;
        
        if ((status&Issue.STATUS_UNCONFIRMED)!=0) count+=statuses[0];
        if ((status&Issue.STATUS_NEW)!=0) count+=statuses[1];
        if ((status&Issue.STATUS_ASSIGNED)!=0) count+=statuses[2];
        if ((status&Issue.STATUS_REOPENED)!=0) count+=statuses[3];
        if ((status&Issue.STATUS_RESOLVED)!=0) count+=statuses[4];
        if ((status&Issue.STATUS_VERIFIED)!=0) count+=statuses[5];
        if ((status&Issue.STATUS_CLOSED)!=0) count+=statuses[6];
        return count;
    }
    
    /**
     * Returns number of issues with given <code>resolution</code>.
     *
     * @param resolution mask specifying resolution (see {@link Issue.RESOLUTION_FIXED} etc.).
     * @return number of issues with given <code>resolution</code>.
     */
    public int getCountByResolution(int resolution) {
        int count=0;
        
        if ((resolution&Issue.RESOLUTION_FIXED)!=0) count+=resolutions[0];
        if ((resolution&Issue.RESOLUTION_INVALID)!=0) count+=resolutions[1];
        if ((resolution&Issue.RESOLUTION_WONTFIX)!=0) count+=resolutions[2];
        if ((resolution&Issue.RESOLUTION_LATER)!=0) count+=resolutions[3];
        if ((resolution&Issue.RESOLUTION_REMIND)!=0) count+=resolutions[4];
        if ((resolution&Issue.RESOLUTION_DUPLICATE)!=0) count+=resolutions[5];
        if ((resolution&Issue.RESOLUTION_WORKSFORME)!=0) count+=resolutions[6];
        if ((resolution&Issue.RESOLUTION_MOVED)!=0) count+=resolutions[7];        
        return count;
    }
    
    /**
     * Returns number of issues with given <code>priority</code>.
     *
     * @param priority mask specifying priority (see {@link Issue.PRIORITY_P1} etc.)
     * @return number of issue with given <code>priority</code>.
     */
    public int getCountByPriority(int priority) {
        int count=0;
        
        if ((priority&Issue.PRIORITY_P1)!=0) count+=priorities[0];
        if ((priority&Issue.PRIORITY_P2)!=0) count+=priorities[1];
        if ((priority&Issue.PRIORITY_P3)!=0) count+=priorities[2];
        if ((priority&Issue.PRIORITY_P4)!=0) count+=priorities[3];
        if ((priority&Issue.PRIORITY_P5)!=0) count+=priorities[4];
        return count;
    }
    
    /**
     * Counts CQI value of the issues for which this distribution was created.
     *
     * @return CQI value of the issues.
     */
    public int getCQI() {
        return priorities[0]*CQI_P1+priorities[1]*CQI_P2+priorities[2]*CQI_P3+
        priorities[3]*CQI_P4+priorities[4]*CQI_P5;
    }
}
