/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.SwingUtilities;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.events.PersistenceManager;
import org.netbeans.testspeceditor.options.Options;
import org.netbeans.testspeceditor.ui.MainFrame;
import org.netbeans.testspeceditor.xmlizable.XMLUtils;

/**
 *
 * @author ehucka
 */
public class Application {
    
    static String commitDate = "$Date$";
    static Date lastDate = null;
    public static SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");

    private static Application instance = null;
    
    private MainFrame ui;
    private Document document;
    
    /**
     * Creates a new instance of Application
     */
    public Application() {
        instance=this;
        //xml init
        XMLUtils.registerClassLoader("current", getClass().getClassLoader());
        XMLUtils.init("org.netbeans.testspeceditor.mappings.beans", true);
        XMLUtils.initParser("suites", "cases", "case-set", "completionTags", "workdirs"); //set scheme tags
        //document
        document=new Document();
        //ui
        ui=new MainFrame();
        document.addDocumentListener(ui);
        PersistenceManager.getInstance().addPersistenceListener(document);
    }
    
    public void start() {
        start(null);
    }
    
    public void start(String[] args) {
        
        boolean loadedFromArgs = false;
        
        if (args != null && args.length > 0 && args[0].length() > 0) {
            File file = new File(args[0]);
            
            if (file.exists()) {
                document.loadDocument(file.getAbsolutePath());
                loadedFromArgs = true;
            }
        }

        if (!loadedFromArgs)
            document.loadDocument(Options.getInstance().getOpenedFile());            
        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                ui.setVisible(true);
            }
        });
    }
    
    public static Application getInstance() {
        if (instance == null) {
            instance=new Application();
        }
        return instance;
    }
    
    public MainFrame getUi() {
        return ui;
    }
    
    public void setUi(MainFrame frm) {
        ui=frm;
    }
    
    public Document getDocument() {
        return document;
    }
    
    public void finish() {
        document.finish();
        Options.getInstance().setOpenedFile(document.getName());
        Options.getInstance().store();
        System.exit(0);
    }
    
    public static Date getLastChangeDate() {
        if (lastDate == null) {
            String s=commitDate.substring(commitDate.indexOf("Date:")+5);
            s=s.substring(0, s.indexOf('$'));
            s=s.trim();
            try {
                lastDate=format.parse(s);
            } catch (ParseException ex) {
                ex.printStackTrace();
            }
        }
        return lastDate;
    }
    
    public static String getBuildName() {
        try {
            BufferedReader br=new BufferedReader(new InputStreamReader(TestSpec.class.getResourceAsStream("/META-INF/version")));
            String line=br.readLine();
            br.close();
            return line;
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return "";
    }
}
