/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import javax.swing.JOptionPane;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.events.PersistenceListener;
import org.netbeans.testspeceditor.events.PersistenceManager;
import org.netbeans.testspeceditor.events.Persistent;
import org.netbeans.testspeceditor.ui.edits.MatchingWords;
import org.netbeans.testspeceditor.ui.status.StatusManager;
import org.netbeans.testspeceditor.undo.UndoManager;
import org.netbeans.testspeceditor.utils.FileUtil;
import org.netbeans.testspeceditor.utils.PostTransformationFilter;
import org.netbeans.testspeceditor.xmlizable.ObjectSource;
import org.netbeans.testspeceditor.xmlizable.XMLSource;
import org.netbeans.testspeceditor.xmlizable.XMLizableException;
import org.netbeans.testspeceditor.xmlparser.DocumentFactory;
import org.netbeans.testspeceditor.xmlparser.Special;
import org.netbeans.testspeceditor.xmlparser.Tag;
import org.netbeans.testspeceditor.xmlparser.TagFactory;
import org.netbeans.testspeceditor.xmlparser.XMLDocument;
/**
 *
 * @author ehucka
 */
public class Document implements PersistenceListener, Persistent, TextData {
    
    private File file;
    private XMLDocument document;
    private TestSpec testSpec;
    
    private String name;
    boolean stored=true;
    
    ArrayList<DocumentListener> listeners;
    
     
    public Document() {
        listeners=new ArrayList(4);
        //newDocument();
    }
    
    public void newDocument() {
        if (!stored && JOptionPane.showConfirmDialog(Application.getInstance().getUi(),
                "The document was not saved. Do you want to save it?", "Save",
                JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
            saveDocument();
        }
        PersistenceManager.getInstance().flush();
        TestSpec ts=new TestSpec();
        name="NewDocument";
        document=null;
        file=null;
        setTestSpec(ts);
        UndoManager.getInstance().flush();
        StatusManager.getInstance().setStatusText("New Document created.", 2);
    }
    
    public void saveDocument() {
        if (getFile() != null) {
            saveDocument(getFile(), false);
        } else {
            String s=FileUtil.openFileDialog(false, "Save As ...", false, new String[] {"xml"}, "XML");
            if (s != null) {
                if (!s.toLowerCase().endsWith(".xml"))
                    s=s+".xml";
                file=new File(s);
                saveDocument(getFile(), false);
            }
        }
        StatusManager.getInstance().setStatusText("Document saved.", 2);
    }
    
    public void saveAsDocument() {
        String file=FileUtil.openFileDialog(false, "Save As ...", false, new String[] {"xml"}, "XML");
        if (file != null) {
            if (!file.toLowerCase().endsWith(".xml"))
                file=file+".xml";
            saveDocument(new File(file), true);
        }
        StatusManager.getInstance().setStatusText("Document Saved As.", 2);
    }
    
    public void saveDocument(File file, boolean firsttime) {
        if (firsttime && file.exists()) {
            if (JOptionPane.showConfirmDialog(Application.getInstance().getUi(),
                    "The file already exists. Do you want to rewrite it?", "Rewrite",
                    JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION)
                return;
        }
        try {
            StatusManager.getInstance().setWaitCursor(true);
            StatusManager.getInstance().setStatusText("Saving Document...");
            if (document == null) {
                XMLSource src=new XMLSource(testSpec);
                Tag root = src.getTag();
                XMLDocument doc=DocumentFactory.createDocument(root);
                doc.addElement(new Special(""));//TODO add entity here
                document=doc;
            }
            document.print(file);
            name=file.getAbsolutePath();
            this.file=file;
            PersistenceManager.getInstance().flush();
            MatchingWords.getInstance().updateWords(testSpec);
        } catch (XMLizableException ex) {
            ex.printStackTrace();
        } finally {
            StatusManager.getInstance().setWaitCursor(false);
        }
    }
    
    public void loadDocument() {
        if (!stored && JOptionPane.showConfirmDialog(Application.getInstance().getUi(),
                "The document was not saved. Do you want to save it?", "Save",
                JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
            saveDocument();
        String s=FileUtil.openFileDialog(true, "Open", false, new String[] {"xml"}, "XML");
        loadDocument(s);
    }
    
    public void loadDocument(String s) {
        if (s != null) {
            try {
                StatusManager.getInstance().setWaitCursor(true);
                File f=new File(s);
                if (f.exists()) {
                    StatusManager.getInstance().setStatusText("Opening document ...");
                    XMLDocument doc=DocumentFactory.getDocument(f);
                    ObjectSource os=new ObjectSource(doc);
                    file=f;
                    name=getFile().getAbsolutePath();
                    setTestSpec((TestSpec)os.getObject());
                    document=doc;
                    PersistenceManager.getInstance().flush();
                    UndoManager.getInstance().flush();
                    MatchingWords.getInstance().updateWords(getTestSpec());
                    StatusManager.getInstance().setStatusText("Document opened.", 2);
                } else {
                    StatusManager.getInstance().setStatusText("File does not exist.", 2);
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            } finally {
                StatusManager.getInstance().setWaitCursor(false);
            }
        }
    }
    
    public void importOldFormatFiles(File testSpec, File caseSet) {
        if (!stored && JOptionPane.showConfirmDialog(Application.getInstance().getUi(),
                "The document was not saved. Do you want to save it?", "Save",
                JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
            saveDocument();
        try {
            StatusManager.getInstance().setStatusText("Importing Old Document.");
            XMLDocument doc1=DocumentFactory.getDocument(testSpec);
            XMLDocument doc2=DocumentFactory.getDocument(caseSet);
            
            XMLDocument doc=DocumentFactory.createDocument(doc1.getRoot());
            for (int i=0;i < doc1.getElementSize();i++) {
                if (doc1.getElement(i) != doc.getRoot())
                    doc.addElement(doc1.getElement(i));
            }
            //suites
            Tag suites=TagFactory.createTag("suites");
            doc.getRoot().addChild(suites);
            Tag[] suitearr=doc.getRoot().getChildren("suite");
            for (Tag ts:suitearr) {
                doc.getRoot().removeChild(ts);
                suites.addChild(ts);
            }
            //cases
            Tag cases=TagFactory.createTag("cases");
            doc.getRoot().addChild(cases);
            Tag[] casearr=doc2.getRoot().getChildren("case");
            for (Tag tc:casearr) {
                cases.addChild(tc);
            }
            //load
            ObjectSource os=new ObjectSource(doc);
            file=null;
            name="NewDocument";
            PersistenceManager.getInstance().flush();
            setTestSpec((TestSpec)os.getObject());
            document=null;
            UndoManager.getInstance().flush();
            StatusManager.getInstance().setStatusText("Document imported.", 2);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void transform(File htmlFile, InputStream xslstream, PostTransformationFilter filter) {
        try {
            //create temp document
            StatusManager.getInstance().setStatusText("Transforming To HTML...");
            File testSpecFile=File.createTempFile("testSpec", "transform");
            XMLDocument doc;
            if (document == null) {
                XMLSource src=new XMLSource(testSpec);
                Tag root = src.getTag();
                doc=DocumentFactory.createDocument(root);
            } else {
                doc=document;
            }
            doc.print(testSpecFile);
            //transform
            Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(xslstream));
            BufferedInputStream xmbis = new BufferedInputStream(new FileInputStream(testSpecFile));
            StreamSource xml = new StreamSource(xmbis);
            BufferedOutputStream hbos = new BufferedOutputStream(new FileOutputStream(htmlFile));
            StreamResult result = new StreamResult(hbos);
            transformer.transform(xml, result);
            xmbis.close();
            hbos.close();
            StatusManager.getInstance().setStatusText("Document transformed.", 2);
            testSpecFile.delete();
            if (filter != null) {
                filter.filter(htmlFile);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            try {
                xslstream.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
    
    public boolean canSave() {
        return (getFile() != null && !stored);
    }
    
    public boolean isStored() {
        return stored;
    }
    
    public TestSpec getTestSpec() {
        return testSpec;
    }
    
    public void setTestSpec(TestSpec testSpec) {
        this.testSpec = testSpec;
        fireDocumentEvent();
        PersistenceManager.getInstance().addPerisistenceEvent(this);
    }
    
    public String getName() {
        return name;
    }
    
    public void stored(org.netbeans.testspeceditor.events.Persistent source) {
        stored=true;
    }
    
    public void needStore(org.netbeans.testspeceditor.events.Persistent source) {
        document=null;
        stored=false;
    }
    
    public void addDocumentListener(DocumentListener l) {
        listeners.add(l);
    }
    
    public void removeDocumentListener(DocumentListener l) {
        listeners.remove(l);
    }
    
    public void fireDocumentEvent() {
        for (DocumentListener l:listeners) {
            l.documentChanged();
        }
    }
    
    public void finish() {
        if (!stored && JOptionPane.showConfirmDialog(Application.getInstance().getUi(),
                "The document was not saved. Do you want to save it?", "Save",
                JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
            saveDocument();
    }
    
    public File getFile() {
        return file;
    }
    
    public String[] getContents() {
        return testSpec.getWholeContents();
    }
}
