/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.options;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import org.netbeans.testspeceditor.xmlizable.XMLInputStream;
import org.netbeans.testspeceditor.xmlizable.XMLOutputStream;
import org.netbeans.testspeceditor.xmlizable.XMLizable;

/**
 *
 * @author ehucka
 */
public class Options implements XMLizable {
    
    static final String OPTIONS_FILE = ".testspec_options.xml";
    
    static Options instance = null;
    
    //ui
    private int width;
    private int height;
    private int x;
    private int y;
    private DividerLocations dividerLocations;
    //files
    ArrayList<Workdir> workdirs;
    private String openedFile;
    //completions
    ArrayList<CompletionTag> completionTags;
    
    String latestNode;
    
    transient HashMap<String, File> workdirsMap;
    
     
    public Options() {
        workdirsMap=new HashMap(4);
        workdirs=new ArrayList(4);
        setOpenedFile(null);
        dividerLocations=new DividerLocations();
        width=640;
        height=480;
        x=10;
        y=10;
        latestNode = null;
        completionTags=new ArrayList(4);
        completionTags.add(new CompletionTag("issue", "<a href=\"http://www.netbeans.org/issues/show_bug.cgi?id=|\">issue |</a>"));
    }
    
    public static Options getInstance() {
        if (instance == null) {
            instance=load();
        }
        return instance;
    }
    
    private static Options load() {
        File file=new File(System.getProperty("user.home"), OPTIONS_FILE);
        try {
            if (file.exists()) {
                XMLInputStream ins=new XMLInputStream(new BufferedInputStream(new FileInputStream(file)));
                Options opts=(Options)ins.readObject();
                opts.loadWorkdirs();
                return opts;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return new Options();
    }
    
    private void loadWorkdirs() {
        for (Workdir w:workdirs) {
            workdirsMap.put(w.getName(), new File(w.getPath()));
        }
    }
    
    public File getWorkdir(String fileType) {
        File f=workdirsMap.get(fileType);
        if (f == null) {
            f=new File(System.getProperty("user.home"));
            workdirsMap.put(fileType, f);
        }
        return f;
    }
    
    public void setWorkdir(String type, File dir) {
        workdirsMap.put(type, dir);
    }
    
    public void store() {
        workdirs.clear();
        for (String name:workdirsMap.keySet()) {
            Workdir w=new Workdir();
            w.setName(name);
            w.setPath(workdirsMap.get(name).getAbsolutePath());
            workdirs.add(w);
        }
        File file=new File(System.getProperty("user.home"), OPTIONS_FILE);
        try {
            XMLOutputStream out=new XMLOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
            out.write(this);
            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public int getWidth() {
        return width;
    }
    
    public void setWidth(int width) {
        this.width = width;
    }
    
    public int getHeight() {
        return height;
    }
    
    public void setHeight(int height) {
        this.height = height;
    }
    
    public int getX() {
        return x;
    }
    
    public void setX(int x) {
        this.x = x;
    }
    
    public int getY() {
        return y;
    }
    
    public void setY(int y) {
        this.y = y;
    }
    
    public DividerLocations getDividerLocations() {
        return dividerLocations;
    }
    
    public CompletionTag[] getCompletionTags() {
        return completionTags.toArray(new CompletionTag[completionTags.size()]);
    }
    
    public void initObject() {
        if (completionTags.size() > 1)
            completionTags.remove(0);
    }
    
    public String getOpenedFile() {
        return openedFile;
    }
    
    public void setOpenedFile(String openedFile) {
        this.openedFile = openedFile;
    }
    
    public String getLatestNode() {
        return latestNode;
    }

    public void setLatestNode(String latestNode) {
        this.latestNode = latestNode;
    }
}
