/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.actions;

import java.lang.reflect.Method;
import java.util.Arrays;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.undo.UndoManager;
import org.netbeans.testspeceditor.utils.Utils;

/**
 *
 * @author ehucka
 */
public class MoveDownAction implements NodeAction {
    
     
    public MoveDownAction() {
    }
    
    public void perform(Node[] nodes) {
        if (isEnabled(nodes)) {
            UndoManager.getInstance().startAtomic();
            Node n=nodes[0];
            Node parent=(Node)n.getParent();
            Sorting[] sort=new Sorting[nodes.length];
            for (int i = 0; i < sort.length; i++) {
                sort[i]=new Sorting(nodes[i], parent, i);
            }
            Arrays.sort(sort);
            Method[] ms=Utils.findActionAnnotatedMethods(n.getClass(), MoveDownAction.class);
            for (int i = sort.length-1;i >= 0;i--) {
                try {
                    ms[0].invoke(nodes[sort[i].arrayIndex]);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
            UndoManager.getInstance().endAtomic();
        }
    }
    
    public boolean isEnabled(Node[] nodes) {
        //all have the same parent
        if (nodes.length == 0) return false;
        Node parent=(Node)(nodes[0].getParent());
        Node n=nodes[0];
        int maxindex=parent.getIndex(n);
        for (int i = 1; i < nodes.length; i++) {
            if (nodes[i].getParent() != parent || n.getClass() != nodes[i].getClass()) {
                return false;
            }
            int ind=parent.getIndex(nodes[i]);
            if (ind > maxindex) {
                maxindex=ind;
            }
        }
        //find the next node
        if (parent.getChildCount()-1 == maxindex) return false;
        Node next=(Node)(parent.getChildAt(maxindex+1));
        Method[] ms=Utils.findActionAnnotatedMethods(next.getClass(), MoveUpAction.class);
        if (ms == null || ms.length != 1) return false;
        return true;
    }
    
    public String getName() {
        return "Move Down";
    }
    
    public String getHelpCtx() {
        return "Move Down selected nodes";
    }
    
    public char getMnemonic() {
        return 'd';
    }
    
    public javax.swing.KeyStroke getKeyStroke() {
        return javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_DOWN, java.awt.event.InputEvent.ALT_MASK);
    }
    
    public int compareTo(Object o) {
        return getName().compareTo(((Action)o).getName());
    }
    
    class Sorting implements Comparable {
        int index;
        public int arrayIndex;
        
        public Sorting(Node n, Node parent, int arrayIndex) {
            this.arrayIndex=arrayIndex;
            index=parent.getIndex(n);
        }
        
        public int compareTo(Object o) {
            Sorting s=(Sorting)o;
            return ((s.index < index) ? 1:-1);
        }
    }
}
