/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import javax.swing.Icon;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestResult;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.actions.ActionAnnotation;
import org.netbeans.testspeceditor.ui.actions.ActionsManager;
import org.netbeans.testspeceditor.ui.actions.CopyAsTextAction;

/**
 *
 * @author ehucka
 */
public class TestResultNode<A extends TestResult> extends DataNode<A> {
    
    public static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(TestResultNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/result.png"));
        ActionsManager.getInstance().registerActions(TestResultNode.class, new CopyAsTextAction());
    }
    
    
    public TestResultNode(TestCaseNode root, A data) {
        super(data, root, false);
        getTestResult().getBeanSupport().addBeanChangeListener(this);
    }
    
    public TestResult getTestResult() {
        return getData();
    }
    
    public String toString() {
        return TestActionNode.getCatString("Result", getTestResult().getContent());
    }
    
    @ActionAnnotation(actionClass=CopyAsTextAction.class)
    public void copyAsText() {
        String cont = ((TestAction)getTestResult()).getContent();
        StringSelection ss = new StringSelection(cont);
        Toolkit.getDefaultToolkit().getSystemClipboard().setContents(ss, null);
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (propertyName.equals("content")) {
            //getTreeModel().nodeChanged(this);
            treeNodesChanged(getNodeEvent());
        }
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public void release() {
        getTestResult().getBeanSupport().removeBeanChangeListener(this);
    }
}
