/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;

import javax.swing.Icon;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.ui.actions.ActionsManager;
import org.netbeans.testspeceditor.ui.actions.NewAction;
import org.netbeans.testspeceditor.ui.actions.NewTestCaseAction;
import org.netbeans.testspeceditor.ui.actions.NewTestSuiteAction;
import org.netbeans.testspeceditor.ui.actions.PackCasesAction;

/**
 *
 * @author ehucka
 */
public class TestSpecNode<T extends TestSpec> extends DataNode<T> {
    
    public static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(TestSpecNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/testspec.png"));
        ActionsManager.getInstance().registerActions(TestSpecNode.class,
                new NewAction(new NewTestCaseAction(), new NewTestSuiteAction()), null,
                new PackCasesAction());
    }
    
    SuitesNode suites;
    CasesNode cases;
    TSTreeModel treemodel;
    
    
    public TestSpecNode(T ts, TSTreeModel model) {
        super(ts, null, true);
        this.treemodel=model;
        init();
        ts.getBeanSupport().addBeanChangeListener(this);
    }
    
    private void init() {
        suites=new SuitesNode(this);
        addChild(suites);
        cases=new CasesNode(this);
        addChild(cases);
    }
    
    public TestSpec getTestSpec() {
        return getData();
    }
    
    public TSTreeModel getTreeModel() {
        return treemodel;
    }
    
    public String toString() {
        return getTestSpec().getModule();
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (propertyName.equals("module")) {
            //getTreeModel().nodeChanged(this);
            treeNodesChanged(getNodeEvent());
        }
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public TreePath findNode(Bean bean) {
        if (bean == getTestSpec()) {
            return getTreePath();
        }
        TreePath path = suites.findNode(bean);
        if (path != null) return path;
        return cases.findNode(bean);
    }
    
    public void release() {
        getTestSpec().getBeanSupport().removeBeanChangeListener(this);
    }
}
