/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree.find;

import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.find.FoundBean;
import org.netbeans.testspeceditor.find.FoundData;
import org.netbeans.testspeceditor.ui.tree.AbstractTreeModel;
import org.netbeans.testspeceditor.ui.tree.DataNode;
import org.netbeans.testspeceditor.ui.tree.SuitesNode;

/**
 *
 * @author ehucka
 */
public class FoundDataNode<T extends FoundData> extends DataNode<T> {
    
    static Icon icon;
    
    static {
        icon = new javax.swing.ImageIcon(SuitesNode.class.getResource("/org/netbeans/testspeceditor/ui/icons/find.gif"));
    }
    
    FoundTreeModel treemodel;
    String label=null;
    /**
     * Creates a new instance of FoundDataNode
     */
    public FoundDataNode(T ts, FoundTreeModel model) {
        super(ts, null, true);
        this.treemodel=model;
        init();
        ts.getBeanSupport().addBeanChangeListener(this);
    }
    
    private void init() {
        for (int i = 0; i < getFoundData().getFoundBeans().size(); i++) {
            FoundBeanNode n1=new FoundBeanNode(this, getFoundData().getFoundBeans().get(i), treemodel);
            addChild(n1);
        }
    }
    
    public FoundData getFoundData() {
        return getData();
    }
    
    public AbstractTreeModel getTreeModel() {
        return treemodel;
    }
    
    public String toString() {
        if (label == null) {
            createLabel();
        }
        return label;
    }
    
    private void createLabel() {
        label="Found "+String.valueOf(getFoundData().getOccurances())+" occurance(s)";
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
        if (source == getFoundData() && child instanceof FoundBean) {
            if (newIndex == -1) { //old
                FoundBeanNode<FoundBean> node=(FoundBeanNode)(getChildAt(oldIndex));
                if (node != null && node.getFoundBean() == child) {
                    int inx=remove(node);
                    node.release();
                    treeNodesRemoved(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{inx}, new Object[] {node}));
                }
            }
        }
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
        if (propertyName.equals("occurances")) {
            createLabel();
            treeNodesChanged(getNodeEvent());
        }
    }
    
    public Icon getIcon() {
        return icon;
    }
    
    public void release() {
        getFoundData().getBeanSupport().removeBeanChangeListener(this);
    }
}
