/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.undo;

import java.lang.reflect.Method;
import org.netbeans.testspeceditor.events.Bean;

/**
 * This action assumes source methods:
 *
 * Bean get(int index);
 * void add(Bean child, int index)
 * Bean remove(int index)
 *
 * These should be signed by UndoAnnotation.
 *
 * @author ehucka
 */
public class BeanStructureUndoAction implements UndoAction {
    
    Bean source;
    Bean child;
    int oldIndex;
    int newIndex;
    
     
    public BeanStructureUndoAction(Bean source, Bean child, int oldIndex, int newIndex) {
        this.source = source;
        this.child = child;
        this.oldIndex = oldIndex;
        this.newIndex = newIndex;
    }
    
    public void undo() throws UndoException {
        //System.err.println("try undo "+source+" child "+child+" old="+oldIndex+" new="+newIndex);
        try {
            Method get, add, rem;
            get=getGetOnIndexMethod();
            add=getAddOnIndexMethod();
            rem=getRemoveFromIndexMethod();
            if (newIndex >= 0) {
                Object o=get.invoke(source, newIndex);
                if (o != child) {
                    throw new IllegalStateException("Cannot make undo - there is different object.");
                }
            }
            if (newIndex >= 0) //add or move
                rem.invoke(source, newIndex);
            if (oldIndex >= 0)//move or remove
                add.invoke(source, child, oldIndex);
        } catch (Exception ex) {
            throw new UndoException(ex);
        }
    }
    
    public void redo() throws UndoException {
        //System.err.println("try redo "+source+" child "+child+" old="+oldIndex+" new="+newIndex);
        try {
            Method get, add, rem;
            get=getGetOnIndexMethod();
            add=getAddOnIndexMethod();
            rem=getRemoveFromIndexMethod();
            if (oldIndex >= 0) { //remove from previous position
                Object o=get.invoke(source, oldIndex);
                if (o != child) {
                    throw new IllegalStateException("Cannot make undo - there is different object.");
                }
                rem.invoke(source, oldIndex);
            }
            if (newIndex >= 0) {
                add.invoke(source, child, newIndex);
            }
        } catch (Exception ex) {
            throw new UndoException(ex);
        }
    }
    
    protected Method getGetOnIndexMethod() throws Exception { //heuristic
        Method[] ms=source.getClass().getMethods();
        for (Method m:ms) {
            if (m.isAnnotationPresent(UndoAnnotation.class)) {
                if (((UndoAnnotation)m.getAnnotation(UndoAnnotation.class)).method() == UndoMethods.GET &&
                        m.getReturnType().isAssignableFrom(child.getClass())) {
                    return m;
                }
            }
        }
        return null;
    }
    
    protected Method getAddOnIndexMethod() throws Exception { //heuristic
        Method[] ms=source.getClass().getMethods();
        for (Method m:ms) {
            if (m.isAnnotationPresent(UndoAnnotation.class)) {
                Class[] parm=m.getParameterTypes();
                if (((UndoAnnotation)m.getAnnotation(UndoAnnotation.class)).method() == UndoMethods.ADD &&
                        parm[0].isAssignableFrom(child.getClass())) {
                    return m;
                }
            }
        }
        return null;
    }
    
    protected Method getRemoveFromIndexMethod() throws Exception { //heuristic
        Method[] ms=source.getClass().getMethods();
        for (Method m:ms) {
            if (m.isAnnotationPresent(UndoAnnotation.class)) {
                if (((UndoAnnotation)m.getAnnotation(UndoAnnotation.class)).method() == UndoMethods.REMOVE &&
                        m.getReturnType().isAssignableFrom(child.getClass())) {
                    return m;
                }
            }
        }
        return null;
    }

    public String toString () {
        return "Structure: "+source.toString()+" child "+child.toString()+" oldi "+oldIndex+" newi "+newIndex;
    }
}
