/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.testspeceditor.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.options.Options;

/**
 *
 * @author ehucka
 */
public class FileUtil {

    public FileUtil() {
    }

    public static String openFileDialog(boolean open, String title,
            boolean dirsOnly, final String[] filters, final String description) {
        JFileChooser fch;
        File file = null;
        File current = Options.getInstance().getWorkdir(description);
        fch = new JFileChooser(current);
        if (!dirsOnly) {
            if (filters != null) {
                fch.setFileFilter(new FileFilter() {

                            public boolean accept(File f) {
                                String s = f.getName().substring(f.getName().lastIndexOf('.') + 1).toLowerCase();
                                if (f.isDirectory()) {
                                    return true;
                                }
                                for (int i = 0; i < filters.length; i++) {
                                    if (s.compareTo(filters[i]) == 0) {
                                        return true;
                                    }
                                }
                                return false;
                            }

                            public String getDescription() {
                                return description;
                            }
                        });
            } else {
                fch.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            }
        } else {
            fch.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        }
        if (open) {
            if (title == null) {
                fch.setDialogTitle("Open");
            } else {
                fch.setDialogTitle(title);
            }
            if (fch.showOpenDialog(Application.getInstance().getUi()) == JFileChooser.APPROVE_OPTION) {
                file = fch.getSelectedFile();
            }
        } else {
            if (title == null) {
                fch.setDialogTitle("Save As ...");
            } else {
                fch.setDialogTitle(title);
            }
            if (fch.showSaveDialog(Application.getInstance().getUi()) == JFileChooser.APPROVE_OPTION) {
                file = fch.getSelectedFile();
            }
        }
        if (file == null) {
            return null;
        }
        Options.getInstance().setWorkdir(description, file.getParentFile());
        return file.getAbsolutePath();
    }

    public static void copyFiles(File file, File copyTo) throws Exception {
        FileChannel fch1 = null;
        FileChannel fch2 = null;
        try {
            fch1 = new FileInputStream(file).getChannel();
            fch2 = new FileOutputStream(copyTo).getChannel();
            fch1.transferTo(0, fch1.size(), fch2);
        } finally {
            if (fch1 != null) {
                try {
                    fch1.close();
                } catch (IOException iOException) {
                    iOException.printStackTrace();
                }
            }
            if (fch2 != null) {
                try {
                    fch2.close();
                } catch (IOException iOException) {
                    iOException.printStackTrace();
                }
            }
        }
    }

    public static void safelyRenameFiles(File file, File renameTo) throws Exception {
        // move temporay file to final one
        if (!file.renameTo(renameTo)) {
            // renameTo() may fail when the destination already exists
            renameTo.delete();
            if (!file.renameTo(renameTo)) {
                // renameTo() may fail again, if temporary file is in 
                // another slice on Solaris. We have to copy manually...
                copyFiles(file, renameTo);
                file.delete();
            }
        }
    }
}
