/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.utils;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import org.netbeans.testspeceditor.ui.actions.ActionAnnotation;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.xmlparser.Tag;

/**
 *
 * @author ehucka
 */
public class Utils {
    
    static int caseLastID = 0;
    static int suiteLastID = 0;
    
     
    public Utils() {
    }
    
    public static String getID(String name, int id) {
        StringBuilder sb=new StringBuilder(name.length());
        boolean und=false;
        for (int i = 0; i < name.length(); i++) {
            if (Character.isLetterOrDigit(name.charAt(i))) {
                sb.append(Character.toLowerCase(name.charAt(i)));
                und=false;
            } else {
                if (!und) {
                    sb.append('_');
                    und=true;
                }
            }
        }
        sb.append('_');
        sb.append(String.valueOf(id));
        return sb.toString();
    }
    
    public static Method[] findAnnotatedMethods(Class cl, Class annot) {
        Method[] ms=cl.getMethods();
        ArrayList<Method> methods=new ArrayList(ms.length);
        for (Method m:ms) {
            if (m.getAnnotation(annot) != null) {
                methods.add(m);
            }
        }
        return methods.toArray(new Method[methods.size()]);
    }
    
    public static Method[] findActionAnnotatedMethods(Class cl, Class actionClass) {
        Method[] ms=cl.getMethods();
        ArrayList<Method> methods=new ArrayList(ms.length);
        for (Method m:ms) {
            Annotation a=m.getAnnotation(ActionAnnotation.class);
            if (a != null && ((ActionAnnotation)a).actionClass() == actionClass) {
                methods.add(m);
            }
        }
        return methods.toArray(new Method[methods.size()]);
    }
    
    public static String getValueNotNull(Tag tag, String child) {
        Tag tch=tag.getChild(child);
        if (tch != null) {
            return tch.getValue();
        }
        return "";
    }
    
    public static Node[] getSelectedNodes(JTree tree) {
        Node[] selected = null;
        TreePath[] selpaths = tree.getSelectionPaths();
        if (selpaths == null) return null;
        selected = new Node[selpaths.length];
        for (int i = 0; i < selected.length; i++) {
            selected[i] = (Node) (selpaths[i].getLastPathComponent());
        }
        return selected;
    }
    
    public static Node[] getSelectedNodesOneInPath(Node[] nodes) {
        ArrayList<Node> select=new ArrayList(nodes.length);
        
        for (int i = 0; i < nodes.length; i++) {
            select.add(nodes[i]);
        }
        for (int i = 0; i < nodes.length; i++) {
            if (parentSelected(nodes[i], select)) {
                select.remove(nodes[i]);
            }
        }
        return select.toArray(new Node[select.size()]);
    }
    
    protected static boolean parentSelected(Node n, ArrayList<Node> selected) {
        Node par=(Node)n.getParent();
        while (par != null) {
            if (selected.contains(par)) {
                return true;
            }
            par=(Node)par.getParent();
        }
        return false;
    }
    
    public static String getGetterName(String propertyName) {
        StringBuilder sb=new StringBuilder(propertyName.length()+3);
        sb.append("get");
        sb.append(Character.toUpperCase(propertyName.charAt(0)));
        sb.append(propertyName.substring(1));
        return sb.toString();
    }
    
    public static String getSetterName(String propertyName) {
        StringBuilder sb=new StringBuilder(propertyName.length()+3);
        sb.append("set");
        sb.append(Character.toUpperCase(propertyName.charAt(0)));
        sb.append(propertyName.substring(1));
        return sb.toString();
    }
    
    public static String clearTags(String s) {
        StringBuilder sb = new StringBuilder();
        boolean tag = false;
        for (int i=0;i < s.length();i++) {
            if (tag && s.charAt(i) == '>') {
                tag = false;
            } else if (s.charAt(i) == '<') {
                tag = true;
            } else if (!tag) {
                sb.append(s.charAt(i));
            }
        }
        return sb.toString();
    }
}
