/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.quiz;

import java.io.*;

import java.util.Collections;
import javax.persistence.EntityManager;
import javax.servlet.*;
import javax.servlet.http.*;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.Response;

/**
 *
 * @author Jindrich Sedek
 */
public class QuizServlet extends HttpServlet {
   
    /** 
    * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
    * @param request servlet request
    * @param response servlet response
    */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        String quizIdParam = request.getParameter("quizid");
        if (quizIdParam == null){
            quizIdParam = request.getParameter("quizId");
        }
        String userId = request.getParameter("userid");
        RequestDispatcher dispatcher = request.getRequestDispatcher("quiz.jsp");
        PersistenceUtils pu = PersistenceUtils.getInstance();
        Quiz quiz = null;
        Integer quizId = null;
        if (quizIdParam != null){
            quizId = Integer.parseInt(quizIdParam);
        }
        if (quizId == null){
            quiz = pu.getActualQuiz();
            if (quiz == null){
                request.getRequestDispatcher("quizfinished.jsp").forward(request, response);
                return ;
            }
            quizId = quiz.getId();
        }
        
        EntityManager em = pu.createEntityManager();
        if (quiz == null){
            quiz = Quiz.findById(em, quizId);
        }
        if (userId != null){
            NbUser user = NbUser.findById(em, userId);
            if (user == null){
                request.setAttribute("notification", true);
            }else{
                if (Response.isAlreadySubmitted(user, quiz)){
                    PersistenceUtils.addOpenQuizes(user, request);
                    dispatcher = request.getRequestDispatcher("alreadysubmitted.jsp");
                }else{
                    boolean notification = false;
                    if (user.getSendNotification() > 0){
                        notification = true;
                    }
                    request.setAttribute("notification", notification);
                    request.setAttribute("username", user.getUsername());
                    request.setAttribute("email", user.getEmail());
                    request.setAttribute("university", user.getUniversity());
                }
            }
        }
        em.close();
        
        String content = pu.getQuizContent(quiz);
        request.setAttribute("countries", Collections.singletonList("India"));
        request.setAttribute("formContent", content);
        if (userId != null){
            request.setAttribute("userid", userId);
        }
        request.setAttribute("quizid", quiz.getId());
        dispatcher.forward(request, response);
    } 

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
    * Handles the HTTP <code>GET</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    } 

    /** 
    * Handles the HTTP <code>POST</code> method.
    * @param request servlet request
    * @param response servlet response
    */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
    * Returns a short description of the servlet.
    */
    public String getServletInfo() {
        return "Short description";
    }
    // </editor-fold>
}
