/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.quiz.entities;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;

/**
 *
 * @author Jindrich Sedek
 */
@Entity
@Table(name = "RESPONSE_ITEM")
@NamedQueries({@NamedQuery(name = "ResponseItem.findByResponse", query = "SELECT r FROM ResponseItem r WHERE r.ResponseItemPK.response = :response"), 
@NamedQuery(name = "ResponseItem.findByItemIndex", query = "SELECT r FROM ResponseItem r WHERE r.ResponseItemPK.itemIndex = :itemIndex"), 
@NamedQuery(name = "ResponseItem.findByItemText", query = "SELECT r FROM ResponseItem r WHERE r.itemText = :itemText"),
@NamedQuery(name = "ResponseItem.findByQuiz", query = "SELECT ri FROM ResponseItem ri JOIN ri.response r WHERE r.quiz = :quiz"),
@NamedQuery(name = "Leaders.all", query = "SELECT NEW org.netbeans.modules.quiz.Leaders$Leader(ri.response.nbuser, count(ri), sum(ri.correct), count(distinct ri.response)) FROM ResponseItem ri " +
                "WHERE (SELECT sum(ri1.correct) FROM ResponseItem ri1 where ri1.response.nbuser = ri.response.nbuser) > :level " +
                "GROUP BY ri.response.nbuser ORDER BY ri.response.nbuser.id"),
@NamedQuery(name = "Leaders.byCategoryId", query = "SELECT NEW org.netbeans.modules.quiz.Leaders$Leader(ri.response.nbuser, count(ri), sum(ri.correct), count(distinct ri.response)) FROM QuizQuestion q, ResponseItem ri WHERE q.category.id=:category AND " +
    "ri.ResponseItemPK.itemIndex = q.quizQuestionPK.record AND ri.response.quiz = q.quiz AND " +
    "((SELECT sum(ri1.correct) FROM ResponseItem ri1, QuizQuestion q1 where q1.category.id=:category AND ri1.response.nbuser = ri.response.nbuser AND ri1.ResponseItemPK.itemIndex = q1.quizQuestionPK.record AND ri1.response.quiz = q1.quiz) > :level) " +
    "GROUP BY ri.response.nbuser ORDER BY ri.response.nbuser.id"),
@NamedQuery(name = "ResponseItem.statsForCorrectQuestions", 
               query= "SELECT NEW org.netbeans.modules.quiz.stats.QuestionItem(q.quizQuestionPK.quizId, q.quizQuestionPK.record, q.question, count(ri)) " +
                      "FROM QuizQuestion q, ResponseItem ri WHERE " +
                      "ri.ResponseItemPK.itemIndex = q.quizQuestionPK.record " +
                      "AND ri.response.quiz = q.quiz " +
                      "AND ri.correct = 1 " +
                      "GROUP BY q.quizQuestionPK.quizId, q.quizQuestionPK.record, q.question " +
                      "ORDER BY q.quizQuestionPK.quizId, q.quizQuestionPK.record")
})        
public class ResponseItem implements Serializable {
    private static final long serialVersionUID = 1L;
    @EmbeddedId
    protected ResponseItemPK ResponseItemPK;
    @Column(name = "ITEM_TEXT")
    private String itemText;
    @Column(name = "IS_CORRECT")
    private int correct;
    @JoinColumn(name = "RESPONSE", referencedColumnName = "ID", insertable = false, updatable = false)
    @ManyToOne
    private Response response;

    public ResponseItem() {
    }

    public ResponseItem(ResponseItemPK ResponseItemPK) {
        this.ResponseItemPK = ResponseItemPK;
    }

    public ResponseItem(Response response, int itemIndex, String itemText, boolean isCorrect) {
        this.ResponseItemPK = new ResponseItemPK(response.getId(), itemIndex);
        this.response = response;
        this.itemText = itemText;
        setCorrect(isCorrect);
    }

    public ResponseItemPK getResponseItemPK() {
        return ResponseItemPK;
    }

    public void setResponseItemPK(ResponseItemPK ResponseItemPK) {
        this.ResponseItemPK = ResponseItemPK;
    }

    public String getItemText() {
        return itemText;
    }

    public void setItemText(String itemText) {
        this.itemText = itemText;
    }

    public Response getResponse() {
        return response;
    }

    public void setResponse(Response response) {
        this.response = response;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (ResponseItemPK != null ? ResponseItemPK.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof ResponseItem)) {
            return false;
        }
        ResponseItem other = (ResponseItem) object;
        if ((this.ResponseItemPK == null && other.ResponseItemPK != null) || (this.ResponseItemPK != null && !this.ResponseItemPK.equals(other.ResponseItemPK))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.quiz.entities.ResponseItem[ResponseItemPK=" + ResponseItemPK + "]";
    }
    
    public boolean isCorrect() {
        return correct != 0;
    }

    public void setCorrect(boolean correct) {
        if (correct){
            this.correct = 1;
        }else{
            this.correct = 0;
        }
    }
}
