/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.EntityManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.modules.quiz.entities.Category;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.QuizQuestion;
import org.netbeans.modules.quiz.entities.Response;
import org.netbeans.modules.quiz.entities.ResponseItem;
import org.netbeans.modules.quiz.entities.Winners;
import static org.junit.Assert.*;

/**
 *
 * @author pblaha
 */
public class ResponseItemTest {

    private static PersistenceUtils pu;
    private static EntityManager em;

    public ResponseItemTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        PersistenceUtils.setDefaultPu(PersistenceUtils.TEST_PU);
        pu = PersistenceUtils.getInstance();
        em = pu.createEntityManager();

        // create data and new Quiz & User
        NbUser user1 = new NbUser("nbId", "Tonda", "email", "address", "university", "ip");

        Category cat1 = new Category(1);
        cat1.setName("Basic");
        Category cat2 = new Category(2);
        cat2.setName("NetBeans");
        
        Quiz quiz = new Quiz(Quiz.suppressId(), new Date(2008, 11, 15), new Date(2006, 10, 14), "Prize");

        QuizQuestion question = new QuizQuestion(quiz, 1, QuestionType.CHOICE, "a");
        question.setQuestion("First question");
        question.setCategory(cat1);
        QuizQuestion question2 = new QuizQuestion(quiz, 2, QuestionType.WORD, "method = a + b");
        question2.setCategory(cat2);
        QuizQuestion question3 = new QuizQuestion(quiz, 3, QuestionType.CHOICE, "d");
        question2.setCategory(cat2);

        ArrayList<QuizQuestion> questions = new ArrayList();
        questions.add(question);
        questions.add(question2);
        questions.add(question3);
        quiz.setQuizQuestionCollection(questions);


        Response resp = new Response(user1, quiz);
        ResponseItem response = new ResponseItem(resp, 1, "a", true); // correct
        ResponseItem response2 = new ResponseItem(resp, 2, "method=a+b", true); // correct
        ResponseItem response3 = new ResponseItem(resp, 3, "a", false); //false

        
        ArrayList<ResponseItem> responses = new ArrayList();
        responses.add(response);
        responses.add(response2);
        responses.add(response3);
        resp.setResponseItemCollection(responses);
        
        Winners winner = new Winners(quiz, user1);

        em.getTransaction().begin();
        em.persist(cat1);
        em.persist(cat2);
        em.persist(user1);
        em.persist(quiz);
        em.persist(resp);
        em.persist(winner);

        em.getTransaction().commit();
        em.close();

    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    @Test
    public void testCorrectResponses() {
        em = pu.createEntityManager();
        List<ResponseItem> results = pu.getAll(em, ResponseItem.class);
        for (ResponseItem responseItem : results) {

            switch (responseItem.getResponseItemPK().getItemIndex()) {
                case (1):
                    assertTrue(responseItem.isCorrect());
                    break;
                case (2):
                    assertTrue(responseItem.isCorrect());
                    break;
                case (3):
                    assertFalse(responseItem.isCorrect());
                    break;
            }
        }
        em.close();
    }
    
    @Test
    public void testCorrectResponseCollection() {
        em = pu.createEntityManager();
        Response response = em.find(Response.class, 1);
        response.getCorrectResponseItemCollection();
        assertEquals(2, response.getCorrectResponseItemCollection().size());
        em.close();
    }
    
    @Test 
    public void testLeaders() throws Exception{
        List<Leaders.Leader> data = Leaders.getLeaders(null);
        Leaders.Leader leader = data.get(0);
        assertEquals("Tonda",leader.getName());
        assertEquals(3,leader.getAnswers());
        assertEquals(2,leader.getCorrectAnswers());

        data = Leaders.getLeaders("all");
        leader = data.get(0);
        assertEquals("Tonda",leader.getName());
        assertEquals(3,leader.getAnswers());
        assertEquals(2,leader.getCorrectAnswers());

        data = Leaders.getLeaders("1");
        leader = data.get(0);
        assertEquals("Tonda",leader.getName());
        assertEquals(1,leader.getAnswers());
        assertEquals(1,leader.getCorrectAnswers());
        
        data = Leaders.getLeaders("2");
        leader = data.get(0);
        assertEquals("Tonda",leader.getName());
        assertEquals(1,leader.getAnswers());
        assertEquals(1,leader.getCorrectAnswers());
    }
    
}

