/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import javax.swing.*;
import java.util.*;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.HelpCtx;
import org.netbeans.modules.regsup.auted.actions.*;
import org.netbeans.modules.regsup.debug.*;
import org.netbeans.modules.regsup.util.Util;

/**
 *
 * @author  Petr Slegr
 * @version 24.6.01/2
 */
// !!! TODO: Better decision about separators in popup menu
public class AutomatonInnerPanel extends JPanel {
    
    /** Container with data to be displayed by this AutomatonPanel
     */
    private VisualAutomatonContainer container;
    
    /** Cache. It would be possible to get editor via container from MainManager
     */
    private AutomatonEditor editor;
    
    /** Debugger for a top-level super container of container from this.
     */
    // !!!! zmenit typ na automaton debugger
    private RexDebugger debugger;
    
    private AutomatonPaintPolicy paintPolicy = AutomatonPaintPolicy.DEFAULT;
    
    private boolean dragOccurred;
    
    private boolean dragStarted;
    
    private VisualAutomatonElement draggedElement;
    
    private Point draggedElementStartPosition;
    
    /** Should be debug actions present in popup even if debugger is not running?
     */
    private static final boolean ADD_DEBUG_ACTIONS_ALWAYS = false;
    
    private static final boolean ADD_SPECIAL_ACTIONS_ALWAYS = true;
    
    
    /** Creates new AutomatonPanel
     */
    AutomatonInnerPanel (VisualAutomatonContainer container) {
        this (container, null);
    }
    
    // !!!! zmenit typ debuggeru
    AutomatonInnerPanel (VisualAutomatonContainer container, RexDebugger debugger) {
        if (null == container) {
            throw new IllegalArgumentException ("Container cannot be null");
        }
        this.container = container;
        this.debugger = debugger;
        editor = MainManager.getDefault ().createEditor (container);
        editor.addAutomatonEditorListener (new AutomatonEditorListenPoint ());
        //
        setBackground (Color.white); //! does not work here, why?
        MouseListenPoint listener = new MouseListenPoint ();
        addMouseListener (listener);
        addMouseMotionListener (listener);
        //setBackground (Color.white); // ! doesn't work here
        // debug section
        //paintPolicy = new AutomatonPaintPolicy.TestPolicy ();
        // !!! divny
        setPreferredSize (new Dimension (500, 400)); // !! const
        //setOpaque (true);
        //System.out.println("pre-set-background: " + getBackground ());
        //setBackground (Color.RED);
        //System.out.println("set-background: " + getBackground ());
    }
    
    // !!!!!!!!!!!!! je potreba?
    public void update (Graphics graphics) {
        paint (graphics);
    }
    
    private Set prepareSet (int type) {
        Set res;
        switch (type) {
            case PaintDescriptor.SELECTED:
                res = editor.getSelected ();
                break;
            case PaintDescriptor.ERROR:
                res = editor.errorElements ();
                break;
                // !!!!! nejakou const
            case 0:
                // vsechno
                res = editor.getContent ().getElements ();
                break;
            default:
                throw new IllegalStateException ("Unknown type: " + type);
        }
        return res;
    }
    
    private void _paintSelected (Graphics2D g, Set elements) {
        Iterator it = elements.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            elem.getGeometry ().paintSelected (g);
        }
    }
    
    private void _paintError (Graphics2D g, Set elements) {
        Iterator it = elements.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            elem.getGeometry ().paintError (g);
        }
    }
    
    private void _paintActive (Graphics2D g, Set elements) {
        //System.out.println ("paintActive");
        for (Iterator it = elements.iterator (); it.hasNext (); ) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            elem.getGeometry ().paintActive (g);
        }
    }
    
    private void _paintBreakpoint (Graphics2D g, Set elements) {
        //System.out.println ("paintActive");
        for (Iterator it = elements.iterator (); it.hasNext (); ) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            elem.getGeometry ().paintBreakpoint (g);
        }
    }
    
    
    private void _paintNormal (Graphics2D g, Set elements) {
        Iterator it = elements.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            elem.getGeometry ().paint (g); // was bug by copy :o(
            //System.out.println("painted geometry: " + elem.getGeometry ());
        }
    }
    
    /**
     */
    private void _paintUnsorted (Graphics2D g, Set elements) {
        // elements can be e.g. selected and error at the same time
        //System.out.println("paintUnsorted");
        Set painted = new HashSet ();
        Set selected = editor.getSelected ();
        selected.retainAll (elements);
        _paintSelected (g, selected);
        painted.addAll (selected);  // adds "selected" elements that has been already painted
        Set error = editor.errorElements ();
        error.retainAll (elements);
        _paintError (g, error);
        painted.addAll (error);
        Set breakpointSet = prepareBreakpointSet ();
        breakpointSet.retainAll (elements);
        _paintBreakpoint (g, breakpointSet);
        //
        if (debugger != null) {
            Set actives = debugger.getCurrentState ().getPureNodes ();
            //System.out.println ("actives: " + actives);
            _paintActive (g, actives);
        }
        //
        elements.removeAll (painted);
        _paintNormal (g, elements); // rest
    }
    
    // !!!!???? predelat na reflection
    private void _paintSet (Graphics2D g, Set set, int type) {
        switch (type) {
            case PaintDescriptor.SELECTED:
                _paintSelected (g, set);
                break;
            case PaintDescriptor.ERROR:
                _paintError (g, set);
                break;
            default:
                throw new IllegalStateException ("Unknown type: " + type);
        }
    }
    
    /** Returns Set of elements to be painted as "with breakpoint"
     */
    // !!! hack-bad per4mance
    private Set prepareBreakpointSet () {
        Set breakpoints = RexDebugger.getBreakpoints (container);
        Set elems = new HashSet ();
        for (Iterator it = breakpoints.iterator (); it.hasNext (); ) {
            RexBreakpoint bre = (RexBreakpoint) it.next ();
            if (bre instanceof AutomatonBreakpoint) {
                VisualAutomatonElement elem = ((AutomatonBreakpoint) bre).getElement ();
                elems.add (elem);
            }
        }
        return elems;
    }
    
    /**
     */
    // !!!!! prejmenovat na paintComponent
    public void paint (Graphics graphics) {
        //System.out.println ("paint XXX");
        super.paint (graphics);
        Graphics2D go; // for double buffering
        Graphics2D gg;
        if (graphics instanceof Graphics2D) {
            gg = (Graphics2D) graphics;
        } else {
            throw new IllegalStateException ("Not 2D graphic");
        }
        Dimension dim = getSize ();
        Image image = createImage ((int) dim.getWidth (), (int) dim.getHeight ());
        go = (Graphics2D) image.getGraphics ();
        go.setRenderingHint (RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        // ! hack - because setBackground() on panel does not work
        Color old = go.getColor ();
        go.setColor (Color.white);
        go.fillRect (0, 0, (int) dim.getWidth (), (int) dim.getHeight ());
        go.setColor (old);
        // end of hack
        //
        VisualAutomatonContainer container = editor.getContent ();
        
        Set elements = container.elements ();
        Set selected = editor.getSelected ();
        
        PaintDescriptor[] descArr = paintPolicy.getPolicy ();
        //int[] flags = new int [descArr.length];
        Set[] sets = new Set[descArr.length];
        for (int i = 0; i < descArr.length; i++) {
            PaintDescriptor desc = descArr[i];
            //System.out.println ("mode: " + desc.mode);
            Set set = prepareSet (desc.mode);
            if (desc.clazz != null) {
                Set s2 = new HashSet ();
                Util.cutFromToByClass (desc.clazz, set, s2);
                sets[i] = s2;
            } else {
                sets[i] = set;
            }
        }
        //
        for (int i = 0; i < descArr.length; i++) {
            elements.removeAll (sets[i]);
        }
        // "elements" should now contain only elems that are not specified explicitly by policy
        if (paintPolicy.getMode () == AutomatonPaintPolicy.MODE_END) {
            _paintUnsorted (go, elements);
        }
        for (int i = 0; i < descArr.length; i++) {
            _paintSet (go, sets[i], descArr[i].mode);
        }
        if (paintPolicy.getMode () == AutomatonPaintPolicy.MODE_START) {
            _paintUnsorted (go, elements);
        }
        //
        /*
        Iterator it = container.elements ().iterator ();
        while (it.hasNext ()) {
            VisualAutomatonElement elem = (VisualAutomatonElement) it.next ();
            ElementGeometry geometry = elem.getGeometry ();
            // element can be selected and active at the same time
            boolean pure = true;
            if (editor.containsSelectedElement (elem)) {
                geometry.paintSelected (go);
                pure = false;
            }
            if (editor.containsActiveElement (elem)) {
                geometry.paintActive (go);
                pure = false;
            }
            if (editor.containsActiveInContainer (elem)) {
                // ! this will be safe if no other object than VAC will be at activeInContainers set
                ((VisualAutomatonContainer.ContainerGeometry) geometry).paintActiveIn (go);
            }
            if (pure) {
                geometry.paint (go);
            }
        }
         */
        // !!!!!?????
        gg.drawImage (image, 0, 0, image.getWidth (this), image.getHeight (this),
        Color.white, this); // ! does not work
        // !!!!!?????
    }
    
    /** Prepares popup menu
     */
    private void tryShowPopup (MouseEvent mouseEvent) {
        JPopupMenu popup = new JPopupMenu ();
        boolean enabled = false;
        JMenuItem item;
        //
        boolean debugged = MainManager.getDefault ().existsDebuggerFor (editor);
        boolean debugging;
        // action is added into popup IF
        // a) is instanceof DebuggingAction OR
        // b) is NOT instance of DebuggingAction and debugger NOT exists
        // (debugging == debugged)
        // AND finally action must be enabled in both cases
        int debugCount = 0;
        if (ADD_DEBUG_ACTIONS_ALWAYS || debugged) {
            for (Iterator it = MainManager.getDefault ().getDebugActions ().iterator ();
            it.hasNext (); ) {
                CallableSystemAction action = (CallableSystemAction) it.next ();
                enabled = action.isEnabled ();
                item = action.getPopupPresenter ();
                item.setEnabled (enabled);
                popup.add (item);
                debugCount++;
            }
        }
        if (debugCount > 0) {
            popup.addSeparator ();
        }
        int elemCount = 0;
        VisualAutomatonElement[] selected = editor.getSelectedElements ();
        /*
        if (selected.length > 0) {
            // set only when some element is selected - was bug
            MainManager.getDefault ().setSelectedElements (selected);
        } else {
            selected = MainManager.getDefault ().getSelectedElements (); // was bug - missing
        }
         */
        //
        Iterator it = MainManager.getDefault ().getElementActions ().iterator ();
        while (it.hasNext ()) {
            ElementAction action = (ElementAction) it.next ();
            debugging = action instanceof DebuggingAction;
            // ! possible to make checking method
            if ((debugging || debugging == debugged) && action.isEnabled (selected)) {
                //
                if (! (action instanceof AddAction)) {
                    if (! MainManager.getDefault ().specialActions ().contains (action)) {
                        popup.add (action.getPopupPresenter ());
                        elemCount++;
                    }
                }
            }
        }
        //
        it = MainManager.getDefault ().getAddActions ().iterator ();
        JMenu submenu = new JMenu ();
        int addCount = 0;
        while (it.hasNext ()) {
            // !!!! every AddAction must be ElementAction
            ElementAction action = (ElementAction) it.next ();
            debugging = action instanceof DebuggingAction;
            if ((debugging || debugging == debugged) && action.isEnabled (selected)) {
                /* !!! for now only - replace by templates
                if (action instanceof AddContainerAction) {
                    //if (templatesPopupSubmenu.getComponents ().length > 0) {
                        submenu.add (templatesPopupSubmenu);
                    //}
                } else {
                 */
                submenu.add (action.getPopupPresenter ());
                /* !!! for now only } */
                addCount++;
            }
        }
        if (addCount > 0) {
            if (elemCount > 0) {
                popup.addSeparator ();
            }
            submenu.setText ("Add"); // !! bundle
            popup.add (submenu);
        }
        // next, all actions are editing
        //if (! debugged) {
        // individual actions
        if (addCount + elemCount > 0) {
            popup.addSeparator ();
        }
        // was bug: missing enable check
        DeleteAction delete = (DeleteAction) MainManager.getDefault ().getAction (DeleteAction.class);
        enabled = ! debugged && delete.isEnabled (selected);
        if (ADD_SPECIAL_ACTIONS_ALWAYS || enabled) {
            popup.add (Util.preparePopupPresenter (delete, enabled));
        }
        popup.addSeparator ();
        //
        EditAction edit = (EditAction) MainManager.getDefault ().getAction (EditAction.class);
        enabled = ! debugged && edit.isEnabled (selected);
        if (ADD_SPECIAL_ACTIONS_ALWAYS || enabled) {
            popup.add (Util.preparePopupPresenter (edit, enabled));
        }
        //
        if (popup.getComponents ().length == 0) {
            // empty popup
            return;
        }
        popup.show (mouseEvent.getComponent (), mouseEvent.getX () + 15,
        mouseEvent.getY () + 15); // !! const
    }
    
    /**
     */
    private void tryStartDND (Point position) {
        dragStarted = false;
        draggedElement = editor.getContent ().findElementOn (position);
        if (draggedElement == null) {
            return;
        }
        if (! draggedElement.isDragable ()) {
            return;
        }
        dragStarted = true;
        draggedElementStartPosition = draggedElement.getPosition ();
        //System.out.println ("drag started: " + draggedElementStartPosition);
    }
    
    /**
     */
    private void tryDoDND (Point position) {
        if (! dragStarted) {
            return;
        }
        draggedElement.handleDragTo (position);
        //System.out.println("drag did: " + position);
        //
        double width = getSize ().getWidth ();
        double height = getSize ().getHeight ();
        // !!! udelat rozsirovani a naopak setrasani pri zmene pozice
        // !!! elementu odkudkoliv
        // !! const
        boolean enlarged = false;
        if (position.getX () + 40 > width) {
            width += 40;
            enlarged = true;
            //System.out.println("SIIIIIIRKA");
        }
        if (position.getY () + 40 > height) {
            height += 40;
            enlarged = true;
            //System.out.println("VYYYYYYSKA");
        }
        if (enlarged) {
            //Rectangle rect = new Rectangle (getPreferredSize ());
            //System.out.println("position to scrol: " + position);
            /*
            Rectangle rect = new Rectangle ((int) position.getX (), (int) position.getY (), 50, 50);
            scrollRectToVisible (rect); // !! const
             */
            // !! NOTE !! does not work if not both sets performed
            /*
            setSize ((int) width, (int) height);
            setPreferredSize (new Dimension ((int) width, (int) height));
             */
            //System.out.println ("size: " + getSize ());
            //revalidate ();
        }
    }
    
    /**
     */
    private void tryStopDND (Point position) {
        if (! dragStarted) {
            return;
        }
        if (dragOccurred) {
            draggedElement.handleDropTo (draggedElementStartPosition, position);
        }
        draggedElement = null;
        dragStarted = false;
        dragOccurred = false;
        //System.out.println ("drag stopped: " + draggedElementStartPosition + " : " + position);
    }
    
    private void prepareToolTip (Point position) {
        // !!!! hazi NPE
        /*
        VisualAutomatonElement elem = MainManager.getDefault ().getCurrentEditor ().
        getDocument ().getContent ().findElementOn (position);
        if (null == elem) {
            setToolTipText (null);
            return;
        } else {
            setToolTipText (elem.getHumanName ());
        }
         */
    }
    
    /**
     * @return the text of the tool tip
     */
    /*
    public String getToolTipText () {
        String retValue = "cau";
     
        return retValue;
    }
     
    public String getToolTipText (MouseEvent event) {
        return "cau - event";
    }
     */
    
    public AutomatonEditor getEditor () {
        return editor;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class MouseListenPoint extends MouseAdapter implements MouseMotionListener {
        
        public void mousePressed (MouseEvent mouseEvent) {
            //System.out.println("mousePressed: " + mouseEvent.getPoint ());
            if (SwingUtilities.isLeftMouseButton (mouseEvent)) {
                tryStartDND (mouseEvent.getPoint ());
            }
        }
        
        public void mouseClicked (MouseEvent mouseEvent) {
            //System.out.println ("mouse clicked");
            Point currentPosition = mouseEvent.getPoint ();
            //AutomatonEditor editor = MainManager.getDefault ().getCurrentEditor ();
            editor.setCurrentPosition (currentPosition);
            VisualAutomatonContainer content = editor.getContent ();
            //
            // "nothing to do" means that no changes are done in selectedElements set
            if (SwingUtilities.isLeftMouseButton (mouseEvent)) {
                if (content.isElementOn (currentPosition)) {
                    VisualAutomatonElement elem = content.findElementOn (currentPosition);
                    if (editor.containsSelectedElement (elem)) {
                        if (mouseEvent.isControlDown ()) {
                            editor.removeSelectedElement (elem);
                        } else {
                            editor.resetSelectedElements ();
                            editor.addSelectedElement (elem);
                        }
                    } else {
                        if (mouseEvent.isControlDown ()) {
                            editor.addSelectedElement (elem);
                        } else {
                            editor.resetSelectedElements ();
                            editor.addSelectedElement (elem);
                        }
                    }
                } else {
                    if (mouseEvent.isControlDown ()) {
                        // nothing to do
                    } else {
                        editor.resetSelectedElements ();
                        setActivatedElements ();
                    }
                }
            }
            //
            if (SwingUtilities.isRightMouseButton (mouseEvent)) {
                if (content.isElementOn (currentPosition)) {
                    VisualAutomatonElement elem = content.findElementOn (currentPosition);
                    if (editor.containsSelectedElement (elem)) {
                        if (mouseEvent.isControlDown ()) {
                            // nothing to do
                        } else {
                            // nothing to do
                        }
                    } else {
                        if (mouseEvent.isControlDown ()) {
                            // nothing to do
                        } else {
                            editor.resetSelectedElements ();
                            editor.addSelectedElement (elem);
                        }
                    }
                } else {
                    if (mouseEvent.isControlDown ()) {
                        editor.resetSelectedElements ();
                        setActivatedElements ();
                    } else {
                        editor.resetSelectedElements ();
                        setActivatedElements ();
                    }
                }
                // !!!!! tady volat repaint
                tryShowPopup (mouseEvent);
            }
            /*
            if (content.isElementOn (currentPosition)) {
                if (rightButton) {
                }
                else if (! mouseEvent.isControlDown ()) {
                    editor.resetSelectedElement ();
                }
                editor.addSelectedElement (content.findElementOn (currentPosition));
            } else {
                editor.resetSelectedElement ();
            }
            // popup invocation
            if (SwingUtilities.isRightMouseButton (mouseEvent)) {
                // !! NOTE !! mouseEvent.isPopupTrigger () does not work
                tryShowPopup (mouseEvent);
            }
             */
        }
        
        private void setActivatedElements () {
            MainManager.getDefault ().setSelectedElements (new VisualAutomatonElement [] { container });
            //System.out.println("activated elems seted: " + MainManager.getDefault ().getSelectedElements ());
        }
        
        public void mouseReleased (MouseEvent mouseEvent) {
            //System.out.println("mouseReleased: " + mouseEvent.getPoint ());
            if (SwingUtilities.isLeftMouseButton (mouseEvent)) {
                tryStopDND (mouseEvent.getPoint ());
            }
            editor.setCurrentPosition (mouseEvent.getPoint ());
        }
        
        public void mouseDragged (MouseEvent mouseEvent) {
            //System.out.println("mouseDragged: " + mouseEvent.getPoint ());
            if (SwingUtilities.isLeftMouseButton (mouseEvent)) {
                dragOccurred = true;
                tryDoDND (mouseEvent.getPoint ());
            }
        }
        
        public void mouseMoved (MouseEvent e) {
            prepareToolTip (e.getPoint ());
        }
        
    }
    
    
    ////////////////////////////////////////////////////////////////////////////
    private class AutomatonEditorListenPoint implements AutomatonEditorListener {
        
        public void selectedElementsChanged (AutomatonEditorEvent e) {
            repaint ();
        }
        
        public void errorElementsChanged (AutomatonEditorEvent e) {
            repaint ();
        }
        
    }
    
}

