/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import org.openide.util.actions.CallableSystemAction;
import java.awt.event.MouseMotionListener;
import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import org.netbeans.modules.regsup.auted.actions.*;
import java.awt.event.ActionEvent;
import org.openide.util.HelpCtx;
import org.netbeans.modules.regsup.debug.RexDebugger;
import org.netbeans.modules.regsup.util.Util;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import org.netbeans.modules.regsup.logic.LogicAutomaton;

/**
 *
 * @author  Petr Slegr
 * @version 24.6.01/2
 */
public class AutomatonPanel extends JPanel {
    
    /** Stores created panels.
     */
    private static Set panels = new HashSet ();
    
    /** Container with data to be displayed by this AutomatonPanel
     */
    private VisualAutomatonContainer container;
    
    /** Cache. It would be possible to get editor via container from MainManager
     */
    // !!!!! zrusit
    private AutomatonEditor editor;
    
    /** Debugger for a top-level super container of container from this.
     */
    // !!!! zmenit typ na automaton debugger
    private RexDebugger debugger;
    
    // !!!!! docasne ne-private
    AutomatonInnerPanel innerPanel;
    
    private AutomatonEditorStatusBar statusBarPanel;
    
    private AutomatonEditorPanel parent;
    
    private JScrollPane automatonScrollPane;
    
    /** Is status bar displayed or not?
     */
    private boolean statusBar = true;
    
    /*
    public static AutomatonPanel find (VisualAutomatonContainer container) {
        return (AutomatonPanel) panels.get (container);
    }
    
    public static AutomatonPanel find (RexDebugger debugger) {
        return (AutomatonPanel) panels.get (debugger);
    }
     */
    
    /**
     */
    public static AutomatonPanel createFor (VisualAutomatonContainer container) {
        return createFor (container, null);
    }
    
    /**
     */
    public synchronized static AutomatonPanel createFor (VisualAutomatonContainer container, 
    RexDebugger debugger) {
        for (Iterator it = panels.iterator (); it.hasNext (); ) {
            AutomatonPanel panel = (AutomatonPanel) it.next ();
            // debugger may be null
            if (panel.getContainer ().equals (container) && panel.getDebugger () == debugger) {
                return panel;
            }
        }
        AutomatonPanel res = new AutomatonPanel (container, debugger);
        panels.add (res);
        return res;
    }
    
    /** Creates new AutomatonPanel
     */
    private AutomatonPanel (VisualAutomatonContainer container) {
        this (container, null);
    }
    
    // !!!! zmenit typ debuggeru
    /**
     * @param container need not be the same as root container of debuuger
     */
    private AutomatonPanel (VisualAutomatonContainer container, RexDebugger debugger) {
        if (null == container) {
            throw new IllegalArgumentException ("Container cannot be null");
        }
        this.container = container;
        this.debugger = debugger;
        //
        automatonScrollPane = new JScrollPane ();
        innerPanel = new AutomatonInnerPanel (container, debugger);
        innerPanel.setBackground (Color.white);
        innerPanel.setOpaque (true);
        automatonScrollPane.setViewportView (innerPanel);
        //automatonScrollPane.setPreferredSize (new Dimension (500, 400));
        //innerPanel.setPreferredSize (new Dimension (500, 300));
        setLayout (new GridBagLayout ());
        GridBagConstraints constr = new GridBagConstraints ();
        constr.fill = GridBagConstraints.BOTH;
        constr.weightx = 1.0;
        constr.weighty = 1.0;
        constr.gridx = 0;
        constr.gridy = 0;
        add (automatonScrollPane, constr);
        setStatusBar (statusBar);
        innerPanel.setBackground (Color.white);
    }
    
    void setParent (AutomatonEditorPanel parent) {
    }
    
    void setStatusBar (boolean set) {
        if (set) {
            statusBarPanel = new AutomatonEditorStatusBar (container.getLogicAutomaton ());
            statusBarPanel.addDFAButtonListener (new DFAButtonListener ());
            GridBagConstraints constr = new GridBagConstraints ();
            constr.fill = GridBagConstraints.HORIZONTAL;
            constr.gridx = 0;
            constr.gridy = 1;
            constr.weightx = 1;
            add (statusBarPanel, constr);
        } else {
            remove (statusBarPanel);
        }
        revalidate ();
        repaint ();
    }
    
    /** Returns editor for container presented by this panel.
     */
    public AutomatonEditor getEditor () {
        return innerPanel.getEditor ();
    }
    
    /** Returns container for which this panel was open.
     */
    public VisualAutomatonContainer getContainer () {
        return container;
    }
    
    /** Returns debugger for which this panel was open.
     * @return may return null
     */
    public RexDebugger getDebugger () {
        return debugger;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class AutomatonEditorListenPoint implements AutomatonEditorListener {
        
        public void selectedElementsChanged (AutomatonEditorEvent e) {
            repaint ();
        }
        
        public void errorElementsChanged (AutomatonEditorEvent e) {
            repaint ();
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class DFAButtonListener implements ActionListener {
        
        /**
         * Invoked when an action occurs.
         */
        public void actionPerformed (ActionEvent e) {
            LogicAutomaton auto = container.getLogicAutomaton ();
            auto.setShouldDetermin (! auto.getShouldDetermin ());
        }
        
    }
    
}

