/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.debug.*;
import java.util.*;
import javax.swing.DefaultListModel;
import org.netbeans.modules.regsup.auted.actions.StepAction;
import java.awt.Color;
import javax.swing.text.*;
import javax.swing.event.DocumentListener;
import org.netbeans.modules.regsup.auted.ui.*;
import java.awt.GridBagConstraints;
import java.awt.event.*;
import javax.swing.ImageIcon;
import javax.swing.JLabel;

/**
 *
 * @author  Petr Slegr
 * @version 20.3.02
 */
public class DebugPanel extends javax.swing.JPanel {
    
    private static final String LABEL_ACCEPTED = "ACCEPTED";
    
    private static final String LABEL_NOT_STARTED = "NOT STARTED";
    
    private static final String LABEL_STARTED = "STARTED";
    
    private static final String LABEL_HANGED = "CANNOT CONTINUE";
    
    private static final String LABEL_SUSPENDED = "SUSPENDED";
    
    private static final String LABEL_BREAKPOINT = "BREAKPOINT";
    
    private BreakpointableTextField inputTextField;
    
    private RexDebugger debugger;
    
    /** Current nodes holded in panel for switching between them
     */
    private ArrayList currentNodes;
    
    /** Index to currentNodes seted by user
     */
    private int currentNodeIndex;
    
    // ! in order to call repaint
    AutomatonEditorPanel parent;
    
    /** Creates new form DebugPanel */
    public DebugPanel (RexDebugger debugger) {
        this.debugger = debugger;
        debugger.addRexDebuggerListener (new DebuggerListener ());
        initComponents ();
        //
        inputTextField = new BreakpointableTextField ();
        inputTextField.setMinimumSize (new java.awt.Dimension (300, 21));
        inputTextField.setPreferredSize (new java.awt.Dimension (300, 21));
        GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints ();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets (4, 4, 4, 4);
        inputPanel.add (inputTextField, gridBagConstraints);
        toggleBreakpointButton.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                toggleBreakpointButtonActionPerformed (e);
            }
        });
        //debuggerStateLabel.setText (LABEL_NOT_STARTED);
        expressionTextField.setText (debugger.getEditor ().getContent ().
        getLogicAutomaton ().getExpression ().toString ());
        Set brks = debugger.getEditor ().getBreakpoints ();
        DefaultListModel model = new DefaultListModel ();
        Iterator it = brks.iterator ();
        while (it.hasNext ()) {
            model.addElement (it.next ());
        }
        breakpointsList.setModel (model);
        //breakpointsList.setCellRenderer (new BreakpointListRenderer ());
        //
        /*
        JLabel label = new JLabel ();
        label.setText ("Input Position Breakpoint: 9");
        label.setIcon (new ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/breakpointx.gif")));
         *
         */
        /*
        JLabel jLabel1 = new JLabel ();
        jLabel1.setIcon (new javax.swing.ImageIcon (getClass ().getResource ("/org/netbeans/modules/regsup/resources/icons/breakpoint2.gif")));
        jLabel1.setText ("Input Position Breakpoint: Position 5");
        model.addElement (jLabel1);
         */
        //
        // for debugging
        inputTextField.setText ("ahoj");
        Document doc = inputTextField.getDocument ();
        /* JDK 1.4
        if (doc instanceof AbstractDocument) {
            ((AbstractDocument) doc).setDocumentFilter (new InputTextFieldDocumentFilter ());
        } else {
            throw new IllegalStateException ();
        }
         */
        // ! no solution know with NavigationFilter
        //inputTextField.setNavigationFilter (new InputTextFieldNavigationFilter ());
        doc.addDocumentListener (new DebugPanel.InputTextFieldDocumentListener ());
        debugger.setInput (getInputString ());
    }
    
    /** Empty version of DebugPanel
     */
    public DebugPanel (AutomatonEditorPanel parent) {
        this.parent = parent;
        initComponents ();
    }
    
    // !!!
    /*
    void setWindow (EditorWindow window) {
        this.window = window;
    }
     */
    
    // !!!???
    void setExpressionString (String str) {
        //expressionTextField.setText (str);
    }
    
    // !!!???
    String getInputString () {
        return inputTextField.getText ();
    }
    
    /**
     */
    private void setStateCount (int count) {
        // !!! complements by empty space from left, try use java.text package
        stateCountLabel.setText (" " + String.valueOf (count) + " ");
    }
    
    /**
     */
    private void selectReadedInput () {
        // ! without this does not work
        inputTextField.getCaret ().setSelectionVisible (true);
        inputTextField.setSelectionColor (Color.green); // !! const
        inputTextField.setSelectionStart (0);
        inputTextField.setSelectionEnd (debugger.getStepNumber ());
    }
    
    /** Updates UI according to debugger state change
     */
    private void updateUI (RexDebuggerEvent e) {
        //RexDebugger debugger = e.getDebugger ();
        boolean back = debugger.isMoveBackwardPossible ();
        moveBackButton.setEnabled (back);
        backButton.setEnabled (back);
        backCharButton.setEnabled (back);
        boolean forw = debugger.isMoveForwardPossible ();
        // ! it is necessary to let buttons enabled TILL INPUT IS SETED BY USER
        boolean emptyInput = debugger.getInput ().equals ("");
        System.out.println ("CAN MOVE FORWARD: " + forw);
        System.out.println ("EMPTY INPUT: " + emptyInput);
        /*
        if (forw || emptyInput) {
            stepButton.setEnabled (forw);
            stepCharButton.setEnabled (forw);
            runButton.setEnabled (forw);
        } else {
            stepButton.setEnabled (false);
            stepCharButton.setEnabled (false);
            runButton.setEnabled (false);
        }
         */
        if (debugger.isHanged () || debugger.isAccepted ()) {
            stepButton.setEnabled (false);
            stepCharButton.setEnabled (false);
            runButton.setEnabled (false);
        } else {
            stepButton.setEnabled (true);
            stepCharButton.setEnabled (true);
            runButton.setEnabled (true);
        }
    }
    
    ///////////////////////////////////////////////////////////////
    private class DebuggerListener extends RexDebuggerAdapter {
        
        public void breakpointApplied (RexDebuggerEvent e) {
            debuggerStateLabel.setText (LABEL_BREAKPOINT);
        }
        
        /** Invoked when input has not been completely read but next move it is not
         * possible.
         */
        public void hanged (RexDebuggerEvent e) {
            debuggerStateLabel.setText (LABEL_HANGED);
            updateUI (null);
            System.out.println ("set as hanged");
        }
        
        /** Invoked when input has been read and a final state has been reached
         */
        public void inputAccepted (RexDebuggerEvent e) {
            //stepButton.setEnabled (false);
            debuggerStateLabel.setText (LABEL_ACCEPTED);
            updateUI (null);
            System.out.println ("set as accepted");
        }
        
        /** Invoked when SUSPEND breakpoint is applied
         */
        public void suspended (RexDebuggerEvent e) {
            debuggerStateLabel.setText (LABEL_SUSPENDED);
            currentNodes = new ArrayList (e.getDebugger ().getCurrentState ().getNodes ());
            System.out.println ("set as suspended");
        }
        
        public void started (RexDebuggerEvent e) {
            debuggerStateLabel.setText (LABEL_STARTED);
            // ! also possible to set to 1 fixedly
            setStateCount (e.getDebugger ().getCurrentState ().getNodes ().size ());
            //System.out.println("STARTED");
        }
        
        public void characterReaded (RexDebuggerEvent e) {
            /*
            int selectionEnd = e.getDebugger ().getStepNumber ();
            //System.out.println("selectionEnd: " + selectionEnd);
            inputTextField.setSelectionStart (0);
            inputTextField.setSelectionEnd (selectionEnd);
             */
            selectReadedInput ();
            setStateCount (e.getDebugger ().getCurrentState ().getNodes ().size ());
            AutomatonEditor editor = MainManager.getDefault ().getCurrentEditor ();
            VisualAutomatonElement elem = (VisualAutomatonElement) e.getDebugger ().
            getCurrentState ().getNodes ().iterator ().next ();
        }
        
        public void characterUnReaded (RexDebuggerEvent e) {
            selectReadedInput ();
            setStateCount (e.getDebugger ().getCurrentState ().getNodes ().size ());
        }
        
        public void containerStepedOut (VisualAutomatonContainer container, RexDebuggerEvent event) {
            // !!!!!x
            AutomatonEditor superEditor = MainManager.getDefault ().createEditor (container.getContainer ());
            // docasne
            //MainManager.getDefault ().getCurrentWindow ().changeViewTo (superEditor);
            //System.out.println("container steped out");
        }
        
        public void containerStepedIn (VisualAutomatonContainer container, RexDebuggerEvent event) {
            AutomatonEditor subEditor = MainManager.getDefault ().createEditor (container);
            // docasne
            //MainManager.getDefault ().getCurrentWindow ().changeViewTo (subEditor);
            //System.out.println("container steped in");
        }
        
        /**
         */
        public void stageChanged (RexDebuggerEvent e) {
            parent.repaint ();
            updateUI (e);
            // !!! redundance
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    /*
    private class InputTextFieldDocumentFilter extends DocumentFilter {
     
        public void insertString (javax.swing.text.DocumentFilter.FilterBypass filterBypass, int offset, String str, javax.swing.text.AttributeSet attributeSet) throws javax.swing.text.BadLocationException {
            //System.out.println ("insert");
            if (offset >= debugger.getStepNumber ()) {
                super.insertString (filterBypass, offset, str, attributeSet);
            }
        }
     
        public void replace (javax.swing.text.DocumentFilter.FilterBypass filterBypass, int offset, int param2, String str, javax.swing.text.AttributeSet attributeSet) throws javax.swing.text.BadLocationException {
            //System.out.println ("replace");
            if (offset >= debugger.getStepNumber ()) {
                super.replace (filterBypass, offset, param2, str, attributeSet);
            }
        }
     
        public void remove (javax.swing.text.DocumentFilter.FilterBypass filterBypass, int offset, int param2) throws javax.swing.text.BadLocationException {
            //System.out.println ("remove");
            if (offset >= debugger.getStepNumber ()) {
                super.remove (filterBypass, offset, param2);
            }
        }
    }
     */
    
    /*
    ////////////////////////////////////////////////////////////////////////////
    private class InputTextFieldNavigationFilter extends NavigationFilter {
     
        public void moveDot (javax.swing.text.NavigationFilter.FilterBypass filterBypass, int param, javax.swing.text.Position.Bias bias) {
        }
     
        public void setDot (javax.swing.text.NavigationFilter.FilterBypass filterBypass, int param, javax.swing.text.Position.Bias bias) {
            if (param >= debugger.getStepNumber ()) {
                super.setDot (filterBypass, param, bias);
            }
            //System.out.println("set");
        }
     
    }
     */
    
    ////////////////////////////////////////////////////////////////////////////
    private class InputTextFieldDocumentListener implements DocumentListener {
        
        public void changedUpdate (javax.swing.event.DocumentEvent documentEvent) {
        }
        
        public void insertUpdate (javax.swing.event.DocumentEvent documentEvent) {
            //System.out.println("inputFromTextField: " + inputTextField.getText ());
            debugger.setInput (inputTextField.getText ());
            // ! problematic
            //selectReadedInput ();
        }
        
        public void removeUpdate (javax.swing.event.DocumentEvent documentEvent) {
            //System.out.println("inputFromTextField: " + inputTextField.getText ());
            debugger.setInput (inputTextField.getText ());
            // ! problematic
            //selectReadedInput ();
        }
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jSplitPane1 = new javax.swing.JSplitPane();
        dbgPanel = new javax.swing.JPanel();
        inputPanel = new javax.swing.JPanel();
        expressionTextField = new javax.swing.JTextField();
        expressionLabel = new javax.swing.JLabel();
        inputLabel = new javax.swing.JLabel();
        inputMoreCheckBox = new javax.swing.JCheckBox();
        inputBreakpointsPanel = new javax.swing.JPanel();
        toggleBreakpointButton = new javax.swing.JButton();
        clearBreakpointsButton = new javax.swing.JButton();
        controlPanel = new javax.swing.JPanel();
        runButton = new javax.swing.JButton();
        stepButton = new javax.swing.JButton();
        stepCharButton = new javax.swing.JButton();
        moveBackButton = new javax.swing.JButton();
        backButton = new javax.swing.JButton();
        backCharButton = new javax.swing.JButton();
        rightPanel = new javax.swing.JPanel();
        breakpointsLabel = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        breakpointsList = new javax.swing.JList();
        statusBarPanel = new javax.swing.JPanel();
        debuggerStateLabel = new javax.swing.JLabel();
        stateCountLabel = new javax.swing.JLabel();
        previousStateButton = new javax.swing.JButton();
        nextStateButton = new javax.swing.JButton();
        empyLabel = new javax.swing.JLabel();

        setLayout(new java.awt.GridBagLayout());

        setEnabled(false);
        jSplitPane1.setMinimumSize(new java.awt.Dimension(0, 0));
        jSplitPane1.setOneTouchExpandable(true);
        dbgPanel.setLayout(new java.awt.GridBagLayout());

        dbgPanel.setMinimumSize(new java.awt.Dimension(0, 0));
        inputPanel.setLayout(new java.awt.GridBagLayout());

        inputPanel.setBorder(new javax.swing.border.EtchedBorder());
        inputPanel.setMinimumSize(new java.awt.Dimension(0, 0));
        expressionTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        expressionTextField.setMinimumSize(new java.awt.Dimension(300, 21));
        expressionTextField.setPreferredSize(new java.awt.Dimension(300, 21));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        inputPanel.add(expressionTextField, gridBagConstraints);

        expressionLabel.setText("Expression");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        inputPanel.add(expressionLabel, gridBagConstraints);

        inputLabel.setText("Input String");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        inputPanel.add(inputLabel, gridBagConstraints);

        inputMoreCheckBox.setSelected(true);
        inputMoreCheckBox.setText("More");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        inputPanel.add(inputMoreCheckBox, gridBagConstraints);

        inputBreakpointsPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        inputBreakpointsPanel.setBorder(new javax.swing.border.TitledBorder(null, "Input String Breakpoints", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.TOP));
        toggleBreakpointButton.setText("Toggle");
        toggleBreakpointButton.setToolTipText("Toggle breakpoint on input text field");
        inputBreakpointsPanel.add(toggleBreakpointButton);

        clearBreakpointsButton.setText("Clear All");
        clearBreakpointsButton.setToolTipText("Removes all breakpoint on input text field.");
        inputBreakpointsPanel.add(clearBreakpointsButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        inputPanel.add(inputBreakpointsPanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        dbgPanel.add(inputPanel, gridBagConstraints);

        controlPanel.setLayout(new java.awt.GridBagLayout());

        runButton.setText("Continue");
        runButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                runButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        controlPanel.add(runButton, gridBagConstraints);

        stepButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/step.gif")));
        stepButton.setText("Step");
        stepButton.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        stepButton.setIconTextGap(2);
        stepButton.setPreferredSize(new java.awt.Dimension(81, 26));
        stepButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        controlPanel.add(stepButton, gridBagConstraints);

        stepCharButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/stepRead.gif")));
        stepCharButton.setText("Step & Read");
        stepCharButton.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
        stepCharButton.setIconTextGap(2);
        stepCharButton.setPreferredSize(new java.awt.Dimension(137, 26));
        stepCharButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stepCharButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        controlPanel.add(stepCharButton, gridBagConstraints);

        moveBackButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/rollback.gif")));
        moveBackButton.setText("Revert");
        moveBackButton.setToolTipText("Moves backward to previous state where debugger was suspended");
        moveBackButton.setIconTextGap(2);
        moveBackButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveBackButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        controlPanel.add(moveBackButton, gridBagConstraints);

        backButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/back.gif")));
        backButton.setText("Back");
        backButton.setToolTipText("");
        backButton.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
        backButton.setIconTextGap(2);
        backButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        controlPanel.add(backButton, gridBagConstraints);

        backCharButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/backUnRead.gif")));
        backCharButton.setText("Back & UnRead");
        backCharButton.setHorizontalTextPosition(javax.swing.SwingConstants.RIGHT);
        backCharButton.setIconTextGap(2);
        backCharButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                backCharButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        controlPanel.add(backCharButton, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        dbgPanel.add(controlPanel, gridBagConstraints);

        jSplitPane1.setLeftComponent(dbgPanel);

        rightPanel.setLayout(new java.awt.GridBagLayout());

        breakpointsLabel.setIcon(new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/regsup/resources/icons/breakpoint.gif")));
        breakpointsLabel.setText("Breakpoints");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        rightPanel.add(breakpointsLabel, gridBagConstraints);

        jScrollPane1.setPreferredSize(new java.awt.Dimension(258, 130));
        breakpointsList.setPreferredSize(new java.awt.Dimension(259, 131));
        jScrollPane1.setViewportView(breakpointsList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        rightPanel.add(jScrollPane1, gridBagConstraints);

        jSplitPane1.setRightComponent(rightPanel);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jSplitPane1, gridBagConstraints);

        statusBarPanel.setLayout(new java.awt.GridBagLayout());

        statusBarPanel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        debuggerStateLabel.setText("UNKNOWN");
        debuggerStateLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(debuggerStateLabel, gridBagConstraints);

        stateCountLabel.setText("     0 ");
        stateCountLabel.setToolTipText("Number of current nodes");
        stateCountLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(stateCountLabel, gridBagConstraints);

        previousStateButton.setText("P");
        previousStateButton.setToolTipText("Show previous node from current ones");
        previousStateButton.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        previousStateButton.setMaximumSize(new java.awt.Dimension(26, 26));
        previousStateButton.setMinimumSize(new java.awt.Dimension(26, 26));
        previousStateButton.setPreferredSize(new java.awt.Dimension(26, 26));
        previousStateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                previousStateButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(previousStateButton, gridBagConstraints);

        nextStateButton.setText("N");
        nextStateButton.setToolTipText("Show next node from current ones");
        nextStateButton.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        nextStateButton.setPreferredSize(new java.awt.Dimension(26, 26));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(nextStateButton, gridBagConstraints);

        empyLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 5;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        statusBarPanel.add(empyLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.weightx = 1.0;
        add(statusBarPanel, gridBagConstraints);

    }//GEN-END:initComponents
    
    private void backCharButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backCharButtonActionPerformed
        // !!!!! predelat na backUnRead
        debugger.back ();
    }//GEN-LAST:event_backCharButtonActionPerformed
    
    private void backButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backButtonActionPerformed
        debugger.back ();
    }//GEN-LAST:event_backButtonActionPerformed
    
    private void toggleBreakpointButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_toggleBreakpointButtonActionPerformed
        inputTextField.toggleBreakpoint ();
    }//GEN-LAST:event_toggleBreakpointButtonActionPerformed
    
    private void previousStateButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_previousStateButtonActionPerformed
        /*
        System.out.println("previousPerformed");
        System.out.println("currentNodes: " + currentNodes);
        // !!!! testovat stav debuggeru, resp. enablovat jen ve spravnych chvilich
        int n = currentNodes.size ();
        if (n == 0) {
            return;
        }
        // moves to left (cyclically)
        currentNodeIndex = (currentNodeIndex + n - 1) % n;
        VisualAutomatonNode node = (VisualAutomatonNode) currentNodes.get (currentNodeIndex);
        AutomatonEditor currentEditor = window.getCurrentEditor ();
        AutomatonEditor editor = MainManager.getDefault ().createEditor (node.getContainer ());
        if (! editor.equals (currentEditor)) {
            // docasne
            //window.changeViewTo (editor);
        }
        // docasne
        //editor.makeElementVisible (node);
        System.out.println("new current node: " + node);
         */
    }//GEN-LAST:event_previousStateButtonActionPerformed
    
    private void stepCharButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stepCharButtonActionPerformed
        debugger.stepAndRead ();
        updateUI (null);
    }//GEN-LAST:event_stepCharButtonActionPerformed
    
    private void moveBackButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moveBackButtonActionPerformed
        debugger.moveBack ();
        updateUI (null);
    }//GEN-LAST:event_moveBackButtonActionPerformed
    
    private void runButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_runButtonActionPerformed
        debugger.run ();
        updateUI (null);
    }//GEN-LAST:event_runButtonActionPerformed
    
    private void stepButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stepButtonActionPerformed
        debugger.step ();
        //updateUI (null);
    }//GEN-LAST:event_stepButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel expressionLabel;
    private javax.swing.JButton backButton;
    private javax.swing.JButton toggleBreakpointButton;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel breakpointsLabel;
    private javax.swing.JButton nextStateButton;
    private javax.swing.JButton backCharButton;
    private javax.swing.JButton previousStateButton;
    private javax.swing.JPanel inputBreakpointsPanel;
    private javax.swing.JButton moveBackButton;
    private javax.swing.JPanel controlPanel;
    private javax.swing.JButton clearBreakpointsButton;
    private javax.swing.JLabel debuggerStateLabel;
    private javax.swing.JList breakpointsList;
    private javax.swing.JButton stepButton;
    private javax.swing.JButton runButton;
    private javax.swing.JPanel dbgPanel;
    private javax.swing.JPanel inputPanel;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JLabel stateCountLabel;
    private javax.swing.JTextField expressionTextField;
    private javax.swing.JButton stepCharButton;
    private javax.swing.JLabel inputLabel;
    private javax.swing.JPanel rightPanel;
    private javax.swing.JLabel empyLabel;
    private javax.swing.JPanel statusBarPanel;
    private javax.swing.JCheckBox inputMoreCheckBox;
    // End of variables declaration//GEN-END:variables
    
}
