/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted;

import javax.swing.undo.*;
import java.util.Vector;
import org.netbeans.modules.regsup.util.Util;
import java.lang.reflect.Field;

/**
 *
 * @author  Petr Slegr
 * @version 12.12.01
 */
public class UndoManagerPlus extends UndoManager {
    
    protected UndoableEdit lastEditBeforeSave;
    
    public UndoManagerPlus () {
    }
    
    /** Tells to this UndoManager that save was performed.
     */
    public void notifySave () {
        lastEditBeforeSave = lastEdit ();
    }
    
    /** Checks if this UndoManager contains an structural edit
     * since last notifySave () calling.
     */
    public boolean containsStructuralEdit () {
        Vector edits = this.edits;
        if (null == lastEdit ()) {
            // no edits before save
            return false;
        }
        //
        int last;
        if (null == lastEditBeforeSave) {
            // save not performed yet
            last = -1;  // was BUG - 0 instead (how typical :-()
        } else {
            last = edits.lastIndexOf (lastEditBeforeSave);
        }
        for (int i = edits.size () - 1; i > last; i--) {
            UndoableEdit edit = (UndoableEdit) edits.get (i);
            if (edit instanceof VisualAutomatonEdit) {
                VisualAutomatonEdit visEdit = (VisualAutomatonEdit) edit;
                if (visEdit.isStructural ()) {
                    //System.out.println ("struct edit found: " + visEdit);
                    if (hasBeenDone (edit)) {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    /** Checks if this UndoManager contains ANY edit
     * since last notifySave () calling.
     */
    public boolean containsEdit () {
        if (lastEdit () == null) {
            // no edits before save
            return false;
        }
        /*
        if (lastEdit ().equals (lastEditBeforeSave)) {
            return false;
        }
         */
        int last;
        if (null == lastEditBeforeSave) {
            // save not performed yet
            last = -1;  // was BUG: 0 instead (how typical :-()
        } else {
            last = edits.lastIndexOf (lastEditBeforeSave);
        }
        for (int i = edits.size () - 1; i > last; i--) {
            UndoableEdit edit = (UndoableEdit) edits.get (i);
            if (hasBeenDone (edit)) {
                return true;
            }
        }
        return false;
    }
    
    public boolean addEdit (UndoableEdit edit) {
        //System.out.println ("before add: " + edits);
        boolean res = super.addEdit (edit);
        //System.out.println ("after add: " + edits);
        return res;
    }
    
    public static boolean hasBeenDone (UndoableEdit edit) {
        Class clazz = edit.getClass ();
        Field field = null;
        try {
            field = Util.findField (clazz, "hasBeenDone");
            field.setAccessible (true);
        }
        catch (SecurityException e) {
            e.printStackTrace ();
        }
        catch (NoSuchFieldException e) {
            e.printStackTrace ();
        }
        Object val = null;
        try {
            val = field.get (edit);
        }
        catch (IllegalArgumentException e) {
            e.printStackTrace ();
        }
        catch (IllegalAccessException e) {
            e.printStackTrace ();
        }
        return ((Boolean) val).booleanValue ();
    }
    
    public boolean canRedo () {
//        Thread.dumpStack();
        return super.canRedo ();
    }
}
