/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import java.awt.*;
import java.lang.reflect.Method;
import javax.swing.undo.*;
import java.beans.*;
import java.lang.reflect.InvocationTargetException;
import org.netbeans.modules.regsup.util.Util;

/**
 *
 * @author  Petr Slegr
 * @version 21.6.01
 */
public abstract class VisualAutomatonElement {
    
    /** This field is assigned during inserting element
     * into a container.
     * It serves for firing of change and undo events.
     * In the current impl assignment is performed
     * in VisualAutomatonContainer.addElement() method.
     */
    protected VisualAutomatonContainer container;
    
    /**
     */
    protected String humanName;
    
    protected Point position;
    
    /** The last factory used for this class
     */
    private GeometryFactory lastFactory;
    
    protected ElementGeometry geometry;
    
    //protected static ElementGeometryFactory geometryFactory;
    
    protected PropertyChangeSupport propChangeSup;
    
    // !!!????
    private VisualAutomatonContainer rootContainerCache;
    
    public static final String PROP_POSITION = "position";
    
    public static final String PROP_HUMAN_NAME = "humanName";
    
    public static final String STRUCTURAL = "structural";
    
    /**
     */
    // !!!?
    protected VisualAutomatonElement () {
        propChangeSup = new PropertyChangeSupport (this);
        position = new PointPlus (0, 0);
        humanName = "noname";
    }
    
    // !!! couldn't be public?
    ElementGeometry getGeometry () {
        GeometryFactory current = (GeometryFactory) GeometryFactoryRegistry.getFactory (getClass ());
        //System.out.println("last factory: " + last);
        //System.out.println("current factory: " + current);
        // !!!!! temp hack
        if (! getClass ().equals (VisualAutomatonNode.class)) {
            return geometry;
        }
        // !!!! end temp hack
        if (lastFactory == null || ! lastFactory.equals (current)) {
            geometry = ((GeometryFactory) current).create (this);
            //System.out.println("new geometry created");
        }
        return geometry;
    }
    
    /** Creates proper ElementGeometry using current factory
     * @return created geometry or null if factory is not set
     * @side assigns <code>geometry</code> field
     */
    protected ElementGeometry prepareGeometry () {
        GeometryFactory factory = GeometryFactoryRegistry.getFactory (getClass ());
        if (factory != null) {
            geometry = factory.create (this);
            return geometry;
        }
        throw new IllegalStateException ("Missing geometry factory for class: " + this.getClass ());
    }
    
    /**
     * @return human name of this element
     */
    public String getHumanName () {
        return humanName;
    }
    
    /** Sets a human name of this element.
     */
    public void setHumanName (String humanName) {
        //System.out.println("humanName (old): " + humanName);
        String oldValue = this.humanName;
        String newValue = humanName;
        if (oldValue.equals (newValue)) {
            // new position is the same as the current one
            return;
        }
        this.humanName = humanName;
        prepareGeometry ();
        // !!!!
        // following causes NPE
        /*
        prepareGeometry ();
        //
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_HUMAN_NAME,
        oldValue, newValue);
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
        // !!! udelat, aby container poslouchal jen na property a firoval undo a change sam?
        // !!! ... container bude poslouchat na vsech propeties svych elementu
        // !!!???
        propChangeSup.firePropertyChange (pce);
         */
    }
    
    void setHumanNameAsSystem (String humanName) {
        String oldHumanName = this.humanName;
        this.humanName = humanName;
        geometry = prepareGeometry ();
        propChangeSup.firePropertyChange (PROP_HUMAN_NAME, oldHumanName, humanName);
    }
    
    /**
     */
    public abstract boolean isOn (Point p);
    
    /**
     */
    public abstract boolean isOnForSelection (Point p);
    
    /** Return upper-left position of this element
     */
    public Point getPosition () {
        return position;
    }
    
    /**
     * @return true by default
     */
    public boolean isDragable () {
        return true;
    }
    
    /**
     */
    void handleDragTo (Point position) {
        Object oldPosition = getPosition ();
        setPositionAsSystem (position);
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_POSITION,
        oldPosition, position);
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        container.fireContentChanged (edit);
    }
    
    /**
     */
    void handleDropTo (Point startPosition, Point endPosition) {
        setPositionAsSystem (endPosition);
        geometry = prepareGeometry ();
        //
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_POSITION,
        startPosition, endPosition);
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
    }
    
    /// ??? vsechny public nebo vsechny package-private
    public void handleDeleteOn (Point position) {
    }
    
    // !!!!???? proc protected? snad public, ne?
    protected void addPropertyChangeListener (PropertyChangeListener pcl) {
        propChangeSup.addPropertyChangeListener (pcl);
    }
    
    protected void removePropertyChangeListener (PropertyChangeListener pcl) {
        propChangeSup.removePropertyChangeListener (pcl);
    }
    
    protected void addPropertyChangeListener (String name, PropertyChangeListener pcl) {
        propChangeSup.addPropertyChangeListener (name, pcl);
    }
    
    protected void removePropertyChangeListener (String name, PropertyChangeListener pcl) {
        propChangeSup.removePropertyChangeListener (name, pcl);
    }
    
    /**
     * @return null if this container is top-level container
     */
    public VisualAutomatonContainer getContainer () {
        return container;
    }
    
    /**
     * @return null if this container is top-level container
     */
    public VisualAutomatonContainer getSuperContainer () {
        if (null == container) {
            return null;
        }
        return container.getContainer ();
    }
    
    /**
     * @return <code>this</code> if this container is top-level container
     */
    // !t
    public VisualAutomatonContainer getRootContainer () {
        // this is top-level container
        if (container == null && this instanceof VisualAutomatonContainer) {
            return (VisualAutomatonContainer) this;
        }
        // result from cache
        if (rootContainerCache != null) {
            return rootContainerCache;
        }
        VisualAutomatonContainer cont = container;
        VisualAutomatonContainer upper = container.getContainer ();
        while (upper != null) {
            container = upper;
            upper = upper.getContainer ();
        }
        rootContainerCache = cont;
        return rootContainerCache;
    }
    
    /**
     */
    void setContainer (VisualAutomatonContainer container) {
        this.container = container;
    }
    
    /**
     */
    public void setPosition (Point position) {
        Point oldValue = this.position;
        Point newValue = position;
        if (oldValue.equals (newValue)) {
            // new position is the same as the current one
            return;
        }
        this.position = newValue;
        geometry = prepareGeometry ();
        //
        PropertyChangeEvent pce = new PropertyChangeEvent (this, PROP_POSITION,
        oldValue, newValue);
        VisualAutomatonChange edit = new VisualAutomatonChange (
        container, pce);
        //container.someChange ();
        container.fireUndoableEdit (edit);
        container.fireContentChanged (edit);
        // !!! co to udelat, aby container poslouchal jen na property a firoval undo a change sam?
        // !!!! jo udelat to... container bude poslouchat na vsech propeties svych elementu
        propChangeSup.firePropertyChange (pce);
    }
    
    /** Is called by undo/redo.
     * Is called in d'n'd handling.
     */
    void setPositionAsSystem (Point position) {
        Point oldPosition = this.position;
        this.position = position;
        geometry = prepareGeometry ();
        propChangeSup.firePropertyChange (PROP_POSITION, oldPosition, position);
    }
    
    
    ////////////////////////////////////////////////////////////////////////////
    class VisualAutomatonChange extends VisualAutomatonContainerEdit {
        
        private Method systemSetter;
        
        protected VisualAutomatonElement element;
        
        protected String property;
        
        protected Object oldValue;
        
        protected Object newValue;
        
        /** This class is instantiated also in case when it is not added into UNDO
         */
        // !!! predelat
        public VisualAutomatonChange (VisualAutomatonContainer container,
        PropertyChangeEvent event
        /* VisualAutomatonElement element, String property, Object oldValue, Object newValue */) {
            super (container);
            this.element = (VisualAutomatonElement) event.getSource ();
            this.property = event.getPropertyName ();
            this.oldValue = event.getOldValue ();
            this.newValue = event.getNewValue ();
            //System.out.println("undo created from: " + element + " with old: " + oldValue + " new: " + newValue);
            //
            Class clazz = element.getClass ();
            //
            BeanInfo bi = null;
            try {
                bi = Introspector.getBeanInfo (clazz);
            }
            catch (IntrospectionException e) {
                e.printStackTrace ();
            }
            PropertyDescriptor[] pds = bi.getPropertyDescriptors ();
            PropertyDescriptor propDes = null;
            for (int i=0; i<pds.length; i++) {
                if (pds[i].getName ().equals (property)) {
                    propDes = pds[i];
                    break;
                }
            }
            //
            /*
            setter = propDes.getWriteMethod ();
             */
            Class arg = propDes.getPropertyType ();
            //System.out.println ("methodParam: " + arg);
            //System.out.println("isPrimitive: " + arg.isPrimitive ());
            Class[] args = { arg };
            String methodName = "set" + property.substring (0, 1).toUpperCase () +
            property.substring (1) + "AsSystem";
            //System.out.println ("methodName: " + methodName);
            try {
                //systemSetter = clazz.getMethod (methodName, args);
                systemSetter = Util.findMethod (clazz, methodName, args);
            }
            catch (SecurityException e) {
                e.printStackTrace ();
            }
            catch (NoSuchMethodException e) {
                e.printStackTrace ();
            }
            //
            Boolean val = (Boolean) propDes.getValue (VisualAutomatonElement.STRUCTURAL);
            if (null != val) {
                if (val.equals (Boolean.FALSE)) {
                    setStructural (false);
                }
            }
        }
        
        /**
         */
        public void redo () throws CannotRedoException {
            super.redo ();
            Object[] args = { newValue };
            try {
                systemSetter.invoke (element, args);
                /*
                setter.invoke (element, args);
                 */
                //System.out.println("redo of: " + element + " again to new: " + newValue);
            }
            catch (InvocationTargetException e) {
                e.printStackTrace ();
            }
            catch (IllegalArgumentException e) {
                e.printStackTrace ();
            }
            catch (IllegalAccessException e) {
                e.printStackTrace ();
            }
            this.container.fireContentChanged (this);
        }
        
        /**
         */
        public void undo () throws CannotUndoException {
            super.undo ();
            Object[] args = { oldValue };
            //System.out.println("oldvalue.class: " +  oldValue.getClass ());
            try {
                systemSetter.invoke (element, args);
                /*
                setter.invoke (element, args);
                 */
                //System.out.println("undo of: " + element + " back to old: " + oldValue);
            }
            catch (InvocationTargetException e) {
                e.printStackTrace ();
            }
            catch (IllegalArgumentException e) {
                e.printStackTrace ();
            }
            catch (IllegalAccessException e) {
                e.printStackTrace ();
            }
            this.container.fireContentChanged (this);
        }
        
        /**
         */
        public String getPresentationName () {
            return "Change of " + humanName + " " + property + " property"; // !! not L10N capable
        }
        
        /**
         */
        public String getUndoPresentationName () {
            return "Undo " + getPresentationName ();
        }
        
        /**
         */
        public String getRedoPresentationName () {
            return "Redo " +  getPresentationName ();
        }
        
    }
    
}
