/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted.actions;

import javax.swing.*;
import org.openide.util.actions.CallableSystemAction;
import org.openide.util.HelpCtx;
import org.netbeans.modules.regsup.auted.*;
import java.awt.Point;
import org.openide.util.NbBundle;
import org.netbeans.modules.regsup.logic.RexEdgeElement;
import org.netbeans.modules.regsup.util.Util;

/**
 *
 * @author  Petr Slegr
 * @version 16.9.01
 */
public class EditAction extends ElementAction {
    
    public void performAction (VisualAutomatonElement selected[]) {
        VisualAutomatonElement elem = selected[0];
        AutomatonEditor editor = MainManager.getDefault ().createEditor (elem.getContainer ());
        Point pos = editor.getCurrentPosition ();
        // only temporary, do it via customizers
        if (elem instanceof VisualAutomatonEdge) {
            VisualAutomatonEdge edge = (VisualAutomatonEdge) elem;
            RexEdgeElement edgeValue = UserSupport.letUserEditEdgeValue (
            edge.getLogicEdge ().getEdgeValue (), pos);
            //System.out.println("returned value: " + edgeValue);
            edge.setEdgeValue (edgeValue);
        }
        else if (elem instanceof VisualAutomatonContainer) {
            String value = UserSupport.letUserEditString (elem.getHumanName (), "Human name", pos);
            if (value != null) {
                elem.setHumanName (value);
            }
        }
        else if (elem instanceof VisualAutomatonNode){
            VisualAutomatonNode node = (VisualAutomatonNode) elem;
            String value = UserSupport.letUserEditString (String.valueOf (node.getID ()), "Node ID", pos);
            if (value != null) {
                try {
                    node.setID (Integer.parseInt (value));
                }
                catch (NumberFormatException e) {
                    throw new IllegalArgumentException (e.toString ());
                }
            }
        }
    }
    
    public String getName () {
        return NbBundle.getMessage (this.getClass (), "CTL_EDIT_ACTION");
    }
    
    protected String iconResource () {
        return "ActionIcon.gif";
    }
    
    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (AddNodeActionAction.class);
    }
    
    /** Test whether the action is currently enabled.
     * @return <code>true</code> if so
     */
    public boolean isEnabled (VisualAutomatonElement[] selected) {
        if (selected.length != 1) {
            return false;
        }
        // !!!! testovat jestli pro element existuje Customizer
        // !!!! zatim jen pro edge
        /*
        if (! (selected[0] instanceof VisualAutomatonEdge)) {
            return false;
        }
        boolean b = ! Util.containsTopLevelContainer (selected);
        return b;
         */
        return true;
    }
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
     * protected void initialize () {
     * super.initialize ();
     * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (AddNodeActionAction.class, "HINT_Action"));
     * }
     */
    
}
