/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.ide;

import java.io.IOException;

import org.openide.actions.*;
import org.openide.filesystems.*;
import org.openide.loaders.*;
import org.openide.util.NbBundle;
import org.openide.util.actions.SystemAction;

/** Recognizes single files in the Repository as being of a certain type.
 *
 * @author Petr Slegr
 * @version 01.05.02
 */
public class RegsupDataLoader extends UniFileLoader {
    
    public RegsupDataLoader () {
        this ("org.netbeans.modules.regsup.ide.RegsupDataObject");
    }
    
    // Can be useful for subclasses:
    protected RegsupDataLoader (String recognizedObjectClass) {
        super (recognizedObjectClass);
    }
    
    protected String defaultDisplayName () {
        return NbBundle.getMessage (RegsupDataLoader.class, "LBL_loaderName");
    }
    
    protected void initialize () {
        
        super.initialize ();
        
        ExtensionList extensions = new ExtensionList ();
        extensions.addExtension ("aut");
        setExtensions (extensions);
        
    }
    
    protected SystemAction[] defaultActions () {
        return new SystemAction[] {
            SystemAction.get (OpenAction.class),
            // SystemAction.get (CustomizeBeanAction.class),
            null,
            SystemAction.get (SaveAction.class),
            null,
            SystemAction.get (FileSystemAction.class),
            null,
            /*
            SystemAction.get (CompileAction.class),
            null,
            SystemAction.get (BuildAction.class),
            null,
            SystemAction.get (ExecuteAction.class),
            null,
             */
            SystemAction.get (CutAction.class),
            SystemAction.get (CopyAction.class),
            SystemAction.get (PasteAction.class),
            null,
            SystemAction.get (DeleteAction.class),
            SystemAction.get (RenameAction.class),
            null,
            SystemAction.get (SaveAsTemplateAction.class),
            null,
            SystemAction.get (ToolsAction.class),
            SystemAction.get (PropertiesAction.class),
        };
    }
    
    protected MultiDataObject createMultiObject (FileObject primaryFile)
    throws DataObjectExistsException, IOException {
        return new RegsupDataObject (primaryFile, this);
    }
    
    // Additional user-configurable properties:
    /*
    public String getMyProp () {
        return (String) getProperty ("myProp");
    }
    public void setMyProp (String nue) {
        putProperty ("myProp", nue, true);
    }
    public void writeExternal (ObjectOutput out) throws IOException {
        super.writeExternal (out);
        out.writeUTF (getMyProp ());
    }
    public void readExternal (ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal (in);
        setMyProp (in.readUTF ());
    }
     */
    
}
