/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic;

import java.util.*;
import java.beans.*;

/**
 *
 * @author  Petr Slegr
 * @version 24.6.01
 */
// !!! Event about adding edge is not fired twice times...:o)
public class LogicAutomatonNode /* implements Cloneable */ {
    
    public NodeID id;
    
    protected RexElement innnerElement;
    
    private Set nexts;
    
    private Set prevs;
    
    private LogicAutomatonEdge selfEdge;
    
    private boolean finalNode = false;
    
    /** LogicAutomaton where this node is. It is set by LogicAutomaton.addNode ()
     */
    //LogicAutomaton automaton;
    
    /*
    private boolean isolated = true;
     */
    
    // !!!! je to k necemu?
    boolean included = true;
    
    private boolean starting = false;
    
    /** Is this node deterministic, i.e. includes output edges with
     * disjoint edge elems only
     * By default true because no output edge is deterministic.
     */
    private boolean determin = true;
    
    public static final String PROP_FINAL_NODE = new String ("finalNode");
    
    public static final String PROP_ISOLATED = new String ("isolated");
    
    public static final String PROP_STARTING = new String ("starting");
    
    private PropertyChangeSupport propChangeSup = new PropertyChangeSupport (this);
    
    private Set nextEdgesCache;
    
    private char nextEdgesCacheKey;
    
    /*
    private LogicAutomaton automaton;
     */
    
    /** Creates new LogicAutomatonNode
     * Directly to use only for debug purposes.
     */
    LogicAutomatonNode () {
        //id = new NodeID ();
        //id = -1; // !! const unAdded node ID
        nexts = new HashSet ();
        prevs = new HashSet ();
    }
    
    // !!!!! zatim k nicemu
    /*
    void setAutomaton (LogicAutomaton automaton) {
        if (automaton != null) {
            throw new IllegalStateException ("Automaton already seted");
        }
        this.automaton = automaton;
    }
     */
    
    public int getID () {
        if (id == null) {
            return -1; // !! const unAdded node ID
        }
        return id.getID ();
    }
    
    void setID (NodeID nodeID) {
        if (id == null) {
            id = nodeID;
        } else {
            // !!! zatim nevm co s tim
            // !!! kontrolovat jestli node patri do automatu?
            //throw new IllegalStateException ("NodeID already initialized: " + id);
        }
    }
    
    /** Returns inner RexElement of CompoundLogicAutomatonNodes.
     * For simple LogicAutomatonNodes returns null.
     */
    public RexElement getInnerElement () {
        return innnerElement; // BUG - was null and this method wasn't overrided in CLAN
    }
    
    
    
    /** Creates edge from this node to the target node and
     * then registers newly created edge within this node and target node both.
     * @return null if output edge with the same target node already exists
     * @sidefect
     */
    public synchronized LogicAutomatonEdge createOuptputEdgeTo (
    LogicAutomatonNode target,
    RexEdgeElement value) {
        if (containsEdgeByOutputNode (target)) {
            return null;
        }
        LogicAutomatonEdge edge = new LogicAutomatonEdge (this, target, value);
        // has no sense to check if currently is non-determin
        if (determin) {
            checkDetermin (edge);
        }
        boolean res = nexts.add (edge);
        // checking of res is not necessary
        target.registerInputEdge (edge);
        fireEdgeChanged ();
        return edge;
    }
    
    /** NOTE: This method is NOT part of public API. It is "public" only
     * to make itself accessible in different package (auted)
     *
     * Registers edge within this node and target node both.
     */
    public void addOutputEdge (LogicAutomatonEdge edge) {
        if (containsEdgeByOutputNode (edge.getTarget ())) {
            throw new IllegalStateException ("output edge with the same target node already present: " + edge);
        }
        // has no sense to check if currently is non-determin
        if (determin) {
            checkDetermin (edge);
        }
        boolean res = nexts.add (edge);
        // !!! checkovat res
        edge.getTarget ().registerInputEdge (edge);
        fireEdgeChanged ();
    }
    
    /** Supposes that current set of output edges (nexts) is disjoint.
     * It is also called from LogicAutomatonEdge after edge value change.
     * //@return true if after addition the edge node will be still determin
     * @side sets the determin flag for this node
     */
    void checkDetermin (LogicAutomatonEdge edge) {
        //System.out.println("checking node: " + this + " against: " + edge);
        boolean oldDetermin = determin;
        RexEdgeElement rex = edge.getEdgeValue ();
        Iterator it = nexts.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge e = (LogicAutomatonEdge) it.next ();
            // hack: edge may be contained in nexts set (in case of checking after edge value check)
            // so it is necessary to skip this element
            if (edge.equals (e)) {
                continue;
            }
            RexEdgeElement r = e.getEdgeValue ();
            if (r.intersects (rex)) {
                //System.out.println ("intersects: " + r + " with: " + rex);
                determin = false;
                if (determin != oldDetermin) {
                    propChangeSup.firePropertyChange (LogicAutomaton.PROP_DETERMIN, null, null);
                }
                return; // ! was missing - painful bug
            }
        }
        determin = true;
        if (determin != oldDetermin) {
            propChangeSup.firePropertyChange (LogicAutomaton.PROP_DETERMIN, null, null);
        }
    }
    
    /** Checks every edge values each to other and if two of them intersects returns
     * false.
     * @return false if this node is NOT deterministic
     * @return true otherwise
     */
    private boolean _checkDetermin () {
        for (Iterator it1 = nexts.iterator (); it1.hasNext (); ) {
            RexEdgeElement elem1 = ((LogicAutomatonEdge) it1.next ()).getEdgeValue ();
            for (Iterator it2 = nexts.iterator (); it2.hasNext (); ) {
                RexEdgeElement elem2 = ((LogicAutomatonEdge) it2.next ()).getEdgeValue ();
                if (! elem1.equals (elem2)) {
                    if (elem1.intersects (elem2)) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    
    /** Checks and if desired fires event.
     */
    private void checkDetermin () {
        boolean newDet = _checkDetermin ();
        System.out.println("newDetermin: " + newDet);
        System.out.println("currentDetermin: " + determin);
        if (newDet) {
            if (! determin) {
                System.out.println("uuuuuuuuuuuuuuuuu");
                //
                determin = true; 
                // ! was bug: was missing, i.e. event was fired but node
                // was still in the same state - non-det :o)
                propChangeSup.firePropertyChange (LogicAutomaton.PROP_DETERMIN, null, null);
            } else {
                // was det, is det
            }
        } else {
            if (determin) {
                throw new IllegalStateException ("After removal of edge node cannot become nondeterministic");
            } else {
                // was not det, is not det
            }
        }
    }
    
    /**
     */
    private void registerInputEdge (LogicAutomatonEdge edge) {
        // !!!!
        prevs.add (edge);
        //checkIsolated ();
    }
    
    /**
     */
    private void registerOutputEdge (LogicAutomatonEdge edge) {
        // !!!!
        nexts.add (edge);
        //checkIsolated ();
    }
    
    
    /** NOT API
     * @sidefect
     */
    public synchronized LogicAutomatonEdge addInputEdge (
    LogicAutomatonNode source,
    RexEdgeElement value) {
        /////
        if (containsEdgeByInputNode (source))
            throw new IllegalStateException (
            "input edge with the same source node already present");
        LogicAutomatonEdge edge = new LogicAutomatonEdge (source, this, value);
        boolean res = prevs.add (edge);
        source.registerOutputEdge (edge);
        fireEdgeChanged ();
        return edge;
    }
    
    
    /**
     */
    public Set getOutputEdges () {
        return new HashSet (nexts);
    }
    
    /**
     */
    public Set getInputEdges () {
        return new HashSet (prevs);
    }
    
    /** Returns set of output edges which acts on (i.e. transition is qualified by accepting)
     * specified char.
     */
    public Set getOutputEdges (char ch) {
        Set res = new HashSet ();
        Iterator it = nexts.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge edge = (LogicAutomatonEdge) it.next ();
            if (edge.getEdgeValue ().includes (ch)) {
                res.add (edge);
            }
        }
        return res;
    }
    
    /** @return true when edge with the same target node as specified is
     * present in output edges set
     */
    public boolean containsEdgeByOutputNode (LogicAutomatonNode node) {
        Iterator it = nexts.iterator ();
        boolean match = false;
        while (it.hasNext ()) {
            LogicAutomatonEdge e = (LogicAutomatonEdge) it.next ();
            if (e.getTarget ().equals (node)) {
                match = true;
                return match;
            }
        }
        return false;
    }
    
    public boolean containsEdgeByInputNode (LogicAutomatonNode node) {
        Iterator it = prevs.iterator ();
        boolean match = false;
        while (it.hasNext ()) {
            LogicAutomatonEdge e = (LogicAutomatonEdge) it.next ();
            if (e.getSource ().equals (node)) {
                match = true;
                return match;
            }
        }
        return false;
    }
    
    /** Test whether this node contains specified edge (no matter
     * if input or output).
     */
    public boolean containsEdge (LogicAutomatonEdge edge) {
        return prevs.contains (edge) || nexts.contains (edge);
    }
    
    /** Returns Set of both input and output edges.
     */ 
    public Set getEdges () {
        Set res = new HashSet (prevs);
        res.addAll (nexts);
        return res;
    }
    
    /**
     */
    public boolean hasSelfEdge () {
        return selfEdge != null;
    }
    
    /** Creates and sets self-edge from this node.
     * When you want modify the self-edge, call removeSelfEdge first and then this
     * method again
     * // !! zmenit radu
     * @param selfEdge null is allowed
     * @return created edge
     */
    public LogicAutomatonEdge setSelfEdge (RexEdgeElement rexValue) {
        /*
        if (! selfEdge.getSource ().equals (selfEdge.getTarget ())) {
            throw new IllegalArgumentException ("Cannot set non-self-edge");
        }
         */
        if (null != selfEdge)
            throw new IllegalStateException ("SelfEdge already set");
        selfEdge = new LogicAutomatonEdge (this, rexValue);
        return selfEdge;
    }
    
    
    /**
     * @return null if node has not selfe edge
     */
    public LogicAutomatonEdge getSelfEdge () {
        return selfEdge;
    }
    
    
    /**
     */
    private void __removeOutputEdge (LogicAutomatonEdge edge) {
        boolean res = nexts.remove (edge);
        if (!res)
            throw new IllegalStateException ("output edge not found");
        fireEdgeChanged ();
    }
    
    
    /** API
     */
    public synchronized void removeOutputEdge (LogicAutomatonEdge edge) {
        boolean res = nexts.remove (edge);
        if (!res)
            throw new IllegalStateException ("output edge not found");
        edge.getTarget ().__removeInputEdge (edge);
        checkDetermin ();
    }
    
    /**
     */
    private void __removeInputEdge (LogicAutomatonEdge edge) {
        boolean res = prevs.remove (edge);
        if (!res)
            throw new IllegalStateException ("input edge not found");
        fireEdgeChanged ();
    }
    
    /** API
     */
    public synchronized void removeInputEdge (LogicAutomatonEdge edge) {
        boolean res = prevs.remove (edge);
        if (!res)
            throw new IllegalStateException ("input edge not found");
        edge.getSource ().__removeOutputEdge (edge);
        checkDetermin ();
        //checkIsolated ();
    }
    
    /**
     */
    int prevCount () {
        return prevs.size ();
    }
    
    /**
     */
    int nextCount () {
        return nexts.size ();
    }
    
    /** Add a PropertyChangeListener to the listener list.
     * @param l The listener to add.
     */
    public void addPropertyChangeListener (PropertyChangeListener l) {
        propChangeSup.addPropertyChangeListener (l);
    }
    
    /** Removes a PropertyChangeListener from the listener list.
     * @param l The listener to remove.
     */
    public void removePropertyChangeListener (PropertyChangeListener l) {
        propChangeSup.removePropertyChangeListener (l);
    }
    
    /** Getter for property isolated.
     * @return Value of property isolated.
     */
    public boolean isIsolated () {
        return nextCount () == 0 && prevCount () == 0;
    }
    
    /**
     */
    public boolean isStarting () {
        return starting;
    }
    
    /**
     */
    public boolean isDetermin () {
        return determin;
    }
    
    /** Setter for property isolated.
     * @param isolated New value of property isolated.
     */
    public void setStarting (boolean starting) {
        if (this.starting != starting) {
            boolean oldStarting = this.starting;
            this.starting = starting;
            propChangeSup.firePropertyChange (PROP_STARTING, new Boolean (oldStarting), new Boolean (starting));
        }
    }
    
    /** Getter for property finalNode.
     * @return Value of property finalNode.
     */
    public boolean isFinalNode () {
        return finalNode;
    }
    
    /** Setter for property finalNode.
     * @param finalNode New value of property finalNode.
     */
    public void setFinalNode (boolean finalNode) {
        if (this.finalNode != finalNode) {
            boolean oldFinalNode = this.finalNode;
            this.finalNode = finalNode;
            propChangeSup.firePropertyChange (PROP_FINAL_NODE, new Boolean (oldFinalNode), new Boolean (finalNode));
        }
    }
    
    
    /** Returns set of nodes that are available from this node by accepting the
     * char ch.
     * @side sets the edge-cache
     */
    // !!!!!? nebyl by problem s ch == 0?
    public Set getNextNodes (char ch) {
        //        if (isFinalNode()) {
        //            return Collections.EMPTY_SET;
        //        } else {
        // was bug: final node can have next nodes...:-)
        Set nextNodes = new HashSet ();
        nextEdgesCache = new HashSet ();
        Iterator it = nexts.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge edge = (LogicAutomatonEdge) it.next ();
            if (edge.getEdgeValue ().includes (ch)) {
                nextNodes.add (edge.getTarget ());
                nextEdgesCache.add (edge); // was bug: was outside this if-block
            }
        }
        nextEdgesCacheKey = ch;
        return nextNodes;
        //        }
    }
    
    /**
     * @return all next nodes
     */
    // !!! mohlo by se kesovat, ale pozor na invalidovani
    public Set getNextNodes () {
        Set nextNodes = new HashSet ();
        Iterator it = nexts.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge edge = (LogicAutomatonEdge) it.next ();
            nextNodes.add (edge.getTarget ());
        }
        return nextNodes;
    }
    
    /** If it is possible gives set from cache. If you also call getNextNodes ()
     * then call it first (prepares cache).
     */
    public Set getNextEdges (char ch) {
        if (ch == nextEdgesCacheKey) {
            return nextEdgesCache;
        }
        Set nextEdges = new HashSet ();
        Iterator it = nexts.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge edge = (LogicAutomatonEdge) it.next ();
            if (edge.getEdgeValue ().includes (ch)) {
                nextEdges.add (edge.getTarget ());
            }
        }
        System.out.println ("nextEdgesFor: " +  this + " " + nextEdges);
        return nextEdges;
    }
    
    /** Adds next edges and nodes into specified sets
     */
    // !!? impl
    public void getNext (char ch, Set edges, Set nodes) {
        nodes.addAll (getNextNodes (ch));
        edges.addAll (getNextEdges (ch));
    }
    
    /**
     */
    public String toString () {
        return new String ("LogicAutomatonNode #" + id);
    }
    
    public void addPropertyChangeListener (String propName, PropertyChangeListener pcl) {
        propChangeSup.addPropertyChangeListener (propName, pcl);
    }
    
    public void removePropertyChangeListener (String propName, PropertyChangeListener pcl) {
        propChangeSup.removePropertyChangeListener (propName, pcl);
    }
    
    /**
     */
    private void fireEdgeChanged () {
        propChangeSup.firePropertyChange (LogicAutomaton.PROP_EDGE_COUNT, null, null);
    }
    
    ////////////////////////////////////////////////////////////////////////////
    public static class NodeID extends RexElement {
        
        protected int id;
        
        private int idCounter;
        
        public NodeID () {
            //id = idCounter++;
        }
        
        private NodeID (int n) {
            id = n;
        }
        
        private NodeID (NodeID nodeid) {
            this.id = nodeid.id;
        }
        
        public NodeID next () {
            return new NodeID (idCounter++);
        }
        
        public int getID () {
            return id;
        }
        
        public String toString () {
            return new String ("<X" + String.valueOf (id) + ">");
        }
        
        public int hashCode () {
            return new Integer (id).hashCode ();
        }
        
        public int size () {
            return 1;
        }
        
        public Object clone () {
            return new NodeID (this);
        }
        
        public boolean equals (Object o) {
            if (! (o instanceof NodeID))
                return false;
            return id == ((NodeID) o).id;
        }
        
    }
}
