/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.logic.aut2expr;

import org.netbeans.modules.regsup.logic.*;
import java.util.*;

/**
 *
 * @author  Petr Slegr
 * @version 02.12.01
 */
public class ResolveResult {
    
    protected RexElement rex;
    
    protected ArrayList errors = new ArrayList ();
    
    protected ArrayList warnings = new ArrayList ();
    
    /** Creates new ResolveResult */
    public ResolveResult (RexElement rex) {
        this ();
        this.rex = rex;
    }
    
    public ResolveResult () {
    }
    
    public boolean isOk () {
        boolean b1 = rex != null ? true : false;
        boolean b2 = errors.size () == 0;
        return b1 && b2;
    }
    
    // !!! zmenit na getErrorsCount
    public int errorsCount () {
        return errors.size ();
    }
    
    // !!! zmenit na getWarningsCount
    public int warningsCount () {
        return warnings.size ();
    }
    
    /**
     * @return null if result is not ok
     */
    public RexElement getExpression () {
        return rex;
    }
    
    // !!! mohla by byt ne-public
    public void setResult (RexElement result) {
        this.rex = result;
    }
    
    // !!! mohla by byt ne-public
    public void addProblem (ResolveProblem problem) {
        boolean res;
        if (problem.isError ()) {
            res = errors.add (problem);
            System.out.println ("error added: " + problem);
        } else {
            res = warnings.add (problem);
            System.out.println ("problem added: " + problem);
        }
        if (! res) {
            throw new IllegalStateException ("duplication of problem");
        }
    }
    
    private Set getLocalOrGlobal (boolean local, List list) {
        Set res = new HashSet ();
        for (Iterator it = list.iterator (); it.hasNext (); ) {
            ResolveProblem prb = (ResolveProblem) it.next ();
            if (prb.isLocal () == local) {
                res.add (prb);
            }
        }
        return res;
    }
    
    public Set getErrors () {
        return new HashSet (errors);
    }
    
    public Set getLocalErrors () {
        return getLocalOrGlobal (true, errors);
    }
    
    public Set getGlobalErrors () {
        return getLocalOrGlobal (false, errors);
    }
    
    public Set getWarnings () {
        return new HashSet (warnings);
    }
    
    public Set getLocalWarnings () {
        return getLocalOrGlobal (true, warnings);
    }
    
    public Set getGlobalWarnings () {
        return getLocalOrGlobal (false, warnings);
    }

    public Set getProblems () {
        Set res = new HashSet (errors);
        res.addAll (warnings);
        return res;
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
    }
    
}
