/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.util;

import java.util.*;
import java.lang.reflect.Method;
import org.netbeans.modules.regsup.auted.VisualAutomatonElement;
import org.netbeans.modules.regsup.auted.VisualAutomatonNode;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import java.util.Enumeration;
import javax.swing.ButtonModel;
import java.lang.reflect.Field;
import org.openide.util.actions.CallableSystemAction;
import javax.swing.JMenuItem;

/**
 *
 * @author  Petr Slegr
 * @version 19.11.01/2
 */
public class Util {
    
    public static Method findMethod (Class clazz, String name, Class[] args)
    throws SecurityException, NoSuchMethodException {
        Class source = clazz;
        while (null != source) {
            //System.out.println("searching in: " + source);
            Method[] methods = source.getDeclaredMethods (); // bug: was clazz
            for (int i=0; i<methods.length; i++) {
                Method m = methods[i];
                if (m.getName ().equals (name) && Arrays.equals (m.getParameterTypes (), args)) {
                    return m;
                }
            }
            source = source.getSuperclass ();
        }
        StringBuffer buf = new StringBuffer ();
        buf.append (name);
        buf.append (" (");
        for (int i=0; i<args.length; i++) {
            buf.append (args[i].getName ());
            if (i < args.length - 1) {
                buf.append (',');
            }
        }
        buf.append (')');
        throw new NoSuchMethodException (buf.toString ());
    }
    
    public static Field findField (Class clazz, String name)
    throws SecurityException, NoSuchFieldException {
        Class source = clazz;
        while (null != source) {
            //System.out.println("searching in: " + source);
            Field[] fields = source.getDeclaredFields (); // bug: was clazz
            for (int i=0; i<fields.length; i++) {
                Field f = fields[i];
                if (f.getName ().equals (name)) {
                    return f;
                }
            }
            source = source.getSuperclass ();
        }
        throw new NoSuchFieldException (name);
    }
    
    
    public static boolean areAllInstanceOf (VisualAutomatonElement[] elements, Class clazz) {
        for (int i=0; i<elements.length; i++) {
            if (! clazz.isAssignableFrom (elements[i].getClass ())) { // BUG - was reversed
                return false;
            }
        }
        return true;
    }
    
    /** Removes all object from "from" set that are instances of clazz class and
     * copies them to "to" set.
     */
    public static void cutFromToByClass (Class clazz, Set from, Set to) {
        Iterator it = from.iterator ();
        while (it.hasNext ()) {
            Object o = it.next ();
            if (o.getClass ().equals (clazz)) {
                to.add (o);
                it.remove ();
            }
        }
    }
    
    /** For testing in actions. It is expected that action's isEnabled method
     * will be invoked for one-size array but this method is generally designed.
     */
    // !!! asi se uz nepouziva
    public static boolean containsTopLevelContainer (VisualAutomatonElement[] elements) {
        for (int i = 0; i < elements.length; i++) {
            if (elements[i].getContainer () == null) {
                System.out.println("top-level found");
                return true;
            }
        }
        return false;
    }
    
    /**
     * @param enabled check against external context (stronger than action.isEnabled())
     */
    public static JMenuItem preparePopupPresenter (CallableSystemAction action, boolean enabled) {
        if (enabled) {
            return action.getPopupPresenter ();
        } else {
            JMenuItem res = new JMenuItem (action.getName ());
            res.setEnabled (false);
            return res;
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
    }
    
}
