/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import java.io.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import org.openide.*;
import org.openide.util.*;

import org.netbeans.modules.vcscore.*;
import org.netbeans.modules.vcscore.commands.*;
import org.netbeans.modules.vcscore.cmdline.RelativeMountDialog;
import org.netbeans.modules.vcscore.util.*;

import org.netbeans.modules.vcs.cmdline.passwd.*;

/**
 * The CVS filesystem Customizer
 * @author  Martin Entlicher
 */
public class CvsCustomizer extends javax.swing.JPanel {

    private javax.swing.ButtonGroup bg;
    
    private final static String CVS_LOCAL = "local";
    private final static String CVS_SERVER = "server";
    private final static String CVS_PSERVER = "pserver";
    private final static String CVS_EXT = "ext";
    
    private String lastRootDir;
    private String lastRelMount;
    private CvsFileSystem fileSystem;
    private boolean loginCancelled = false;

    private ActionListener closeListener = null;
    

    static final long serialVersionUID =-4163355905662053542L;

    /** Creates new form CvsCustomizer1 */
    public CvsCustomizer() {
        //changeSupport=new PropertyChangeSupport(this);
        initComponents();
        //getRootPane().setDefaultButton(okButton);
        javax.swing.ButtonGroup bg = new javax.swing.ButtonGroup ();
        bg.add (localRadioButton);
        bg.add (serverRadioButton);
        bg.add (pserverRadioButton);
        bg.add (extRadioButton);
        browseButton.setMnemonic(KeyEvent.VK_W);
        repositoryButton.setMnemonic(KeyEvent.VK_R);
        localRadioButton.setMnemonic(KeyEvent.VK_L);
        serverRadioButton.setMnemonic(KeyEvent.VK_S);
        pserverRadioButton.setMnemonic(KeyEvent.VK_P);
        cvsExeButton.setMnemonic(KeyEvent.VK_E);
        cygwinCheckBox.setMnemonic(KeyEvent.VK_U);
        cygwinButton.setMnemonic(KeyEvent.VK_B);
        localRadioButton.requestFocus ();
        removeEnterFromKeymap();
        //D.deb("constructor update ...");
        updateEnabledComponents ();
        //D.deb("constructor localize ...");
        //localizeComponents();
        //D.deb("constructor help ...");
        //HelpCtx.setHelpIDString (this.getRootPane(), CvsCustomizer.class.getName ());
        //rootDirChanged(false);
        //setTitle(org.openide.util.NbBundle.getBundle("org.netbeans.modules.vcs.cmdline.Bundle").getString("CvsCustomizer.title"));
        jTabbedPane1.setTitleAt(0, org.openide.util.NbBundle.getBundle("org.netbeans.modules.vcs.cmdline.Bundle").getString("CvsCustomizer.tab0Title"));
        jTabbedPane1.setTitleAt(1, org.openide.util.NbBundle.getBundle("org.netbeans.modules.vcs.cmdline.Bundle").getString("CvsCustomizer.tab1Title"));
        jTabbedPane1.setTitleAt(2, org.openide.util.NbBundle.getBundle("org.netbeans.modules.vcs.cmdline.Bundle").getString("CvsCustomizer.tab2Title"));
        //pack ();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jTabbedPane1 = new javax.swing.JTabbedPane();
        connectionPanel = new javax.swing.JPanel();
        rootDirLabel = new javax.swing.JLabel();
        rootDirTextField = new javax.swing.JTextField();
        browseButton = new javax.swing.JButton();
        lbRelMount = new javax.swing.JLabel();
        txRelMount = new javax.swing.JTextField();
        btnRelMount = new javax.swing.JButton();
        localRadioButton = new javax.swing.JRadioButton();
        serverRadioButton = new javax.swing.JRadioButton();
        pserverRadioButton = new javax.swing.JRadioButton();
        extRadioButton = new javax.swing.JRadioButton();
        serverLabel = new javax.swing.JLabel();
        serverTextField = new javax.swing.JTextField();
        userNameLabel = new javax.swing.JLabel();
        userNameTextField = new javax.swing.JTextField();
        repositoryLabel = new javax.swing.JLabel();
        repositoryTextField = new javax.swing.JTextField();
        repositoryButton = new javax.swing.JButton();
        settingsPanel = new javax.swing.JPanel();
        cvsExeLabel = new javax.swing.JLabel();
        cvsExeTextField = new javax.swing.JTextField();
        cvsExeButton = new javax.swing.JButton();
        cygwinCheckBox = new javax.swing.JCheckBox();
        cygwinLabel = new javax.swing.JLabel();
        cygwinTextField = new javax.swing.JTextField();
        cygwinButton = new javax.swing.JButton();
        advancedPanel = new javax.swing.JPanel();
        advancedOptCheckBox = new javax.swing.JCheckBox();
        debugCheckBox = new javax.swing.JCheckBox();
        callEditCheckBox = new javax.swing.JCheckBox();
        offLineCheckBox = new javax.swing.JCheckBox();
        
        setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        jTabbedPane1.setBorder(new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder()));
        jTabbedPane1.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
        connectionPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        
        connectionPanel.setBorder(new javax.swing.border.EtchedBorder());
        connectionPanel.setName(" ");
        rootDirLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.rootDirLabel.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.insets = new java.awt.Insets(12, 12, 5, 6);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(rootDirLabel, gridBagConstraints2);
        
        rootDirTextField.setColumns(20);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 5, 5, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints2.weightx = 1.0;
        connectionPanel.add(rootDirTextField, gridBagConstraints2);
        
        browseButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.browseButton.text"));
        browseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 0;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(12, 5, 5, 11);
        connectionPanel.add(browseButton, gridBagConstraints2);
        
        lbRelMount.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.lbRelMount.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.ipadx = 38;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 5, 6);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(lbRelMount, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 5, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(txRelMount, gridBagConstraints2);
        
        btnRelMount.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.btnRelMount.text"));
        btnRelMount.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRelMountActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 1;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 5, 11);
        connectionPanel.add(btnRelMount, gridBagConstraints2);
        
        localRadioButton.setSelected(true);
        localRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.localRadioButton.text"));
        localRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                localRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 2;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.insets = new java.awt.Insets(7, 12, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(localRadioButton, gridBagConstraints2);
        
        serverRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.serverRadioButton.text"));
        serverRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                serverRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 3;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(serverRadioButton, gridBagConstraints2);
        
        pserverRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.pserverRadioButton.text"));
        pserverRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pserverRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 4;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(pserverRadioButton, gridBagConstraints2);
        
        extRadioButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.extRadioButton.text"));
        extRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                extRadioButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 5;
        gridBagConstraints2.gridwidth = 3;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 12, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(extRadioButton, gridBagConstraints2);
        
        serverLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.serverLabel.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 6;
        gridBagConstraints2.insets = new java.awt.Insets(7, 18, 5, 5);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(serverLabel, gridBagConstraints2);
        
        serverTextField.setColumns(20);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 6;
        gridBagConstraints2.gridwidth = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(7, 5, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(serverTextField, gridBagConstraints2);
        
        userNameLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.userNameLabel.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 7;
        gridBagConstraints2.insets = new java.awt.Insets(0, 18, 5, 5);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(userNameLabel, gridBagConstraints2);
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 7;
        gridBagConstraints2.gridwidth = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 5, 11);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(userNameTextField, gridBagConstraints2);
        
        repositoryLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.repositoryLabel.text"));
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 0;
        gridBagConstraints2.gridy = 8;
        gridBagConstraints2.insets = new java.awt.Insets(0, 18, 11, 5);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(repositoryLabel, gridBagConstraints2);
        
        repositoryTextField.setColumns(20);
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 1;
        gridBagConstraints2.gridy = 8;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 11, 0);
        gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
        connectionPanel.add(repositoryTextField, gridBagConstraints2);
        
        repositoryButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.repositoryButton.text"));
        repositoryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                repositoryButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridx = 2;
        gridBagConstraints2.gridy = 8;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints2.insets = new java.awt.Insets(0, 5, 11, 11);
        connectionPanel.add(repositoryButton, gridBagConstraints2);
        
        jTabbedPane1.addTab("CVS Filesystem Settings", connectionPanel);
        
        settingsPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        
        settingsPanel.setBorder(new javax.swing.border.EtchedBorder());
        cvsExeLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.cvsExeLabel.text"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 0;
        gridBagConstraints3.insets = new java.awt.Insets(12, 12, 0, 5);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        settingsPanel.add(cvsExeLabel, gridBagConstraints3);
        
        cvsExeTextField.setColumns(20);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 1;
        gridBagConstraints3.gridy = 0;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(12, 6, 0, 5);
        gridBagConstraints3.weightx = 1.0;
        settingsPanel.add(cvsExeTextField, gridBagConstraints3);
        
        cvsExeButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.cvsExeButton.text"));
        cvsExeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cvsExeButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 2;
        gridBagConstraints3.gridy = 0;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(12, 0, 0, 11);
        settingsPanel.add(cvsExeButton, gridBagConstraints3);
        
        cygwinCheckBox.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.cygwinCheckBox.text"));
        cygwinCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cygwinCheckBoxActionPerformed(evt);
            }
        });
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 1;
        gridBagConstraints3.gridwidth = 3;
        gridBagConstraints3.insets = new java.awt.Insets(5, 12, 0, 11);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        settingsPanel.add(cygwinCheckBox, gridBagConstraints3);
        
        cygwinLabel.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.cygwinLabel.text"));
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridy = 2;
        gridBagConstraints3.insets = new java.awt.Insets(5, 16, 11, 5);
        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
        settingsPanel.add(cygwinLabel, gridBagConstraints3);
        
        cygwinTextField.setColumns(20);
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 1;
        gridBagConstraints3.gridy = 2;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(5, 6, 11, 5);
        gridBagConstraints3.weightx = 1.0;
        settingsPanel.add(cygwinTextField, gridBagConstraints3);
        
        cygwinButton.setText(java.util.ResourceBundle.getBundle("org/netbeans/modules/vcs/cmdline/Bundle").getString("CvsCustomizer.cygwinButton.text"));
        cygwinButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cygwinButtonActionPerformed(evt);
            }
        });
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 2;
        gridBagConstraints3.gridy = 2;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints3.insets = new java.awt.Insets(5, 0, 11, 11);
        settingsPanel.add(cygwinButton, gridBagConstraints3);
        
        jTabbedPane1.addTab("CVS Setup", settingsPanel);
        
        advancedPanel.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        
        advancedOptCheckBox.setToolTipText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.advancedOptCheckBox.toolTipText"));
        advancedOptCheckBox.setText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.advancedOptCheckBox.text"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.insets = new java.awt.Insets(8, 8, 0, 8);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints4.weightx = 1.0;
        advancedPanel.add(advancedOptCheckBox, gridBagConstraints4);
        
        debugCheckBox.setToolTipText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.debugCheckBox.toolTipText"));
        debugCheckBox.setText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.debugCheckBox.text"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 2;
        gridBagConstraints4.insets = new java.awt.Insets(8, 8, 0, 8);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.NORTHWEST;
        advancedPanel.add(debugCheckBox, gridBagConstraints4);
        
        callEditCheckBox.setToolTipText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.callEditCheckBox.toolTipText"));
        callEditCheckBox.setText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.callEditCheckBox.text"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 3;
        gridBagConstraints4.insets = new java.awt.Insets(8, 8, 0, 8);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.NORTHWEST;
        advancedPanel.add(callEditCheckBox, gridBagConstraints4);
        
        offLineCheckBox.setToolTipText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.offLineCheckBox.toolTipText"));
        offLineCheckBox.setText(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.offLineCheckBox.text"));
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridy = 6;
        gridBagConstraints4.insets = new java.awt.Insets(8, 8, 8, 8);
        gridBagConstraints4.anchor = java.awt.GridBagConstraints.NORTHWEST;
        advancedPanel.add(offLineCheckBox, gridBagConstraints4);
        
        jTabbedPane1.addTab("Advanced Settings", advancedPanel);
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints1.ipadx = 1;
        gridBagConstraints1.insets = new java.awt.Insets(12, 12, 5, 11);
        add(jTabbedPane1, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void cygwinButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cygwinButtonActionPerformed
        // Add your handling code here:
        JFileChooser fc = new JFileChooser ();
        fc.setFileSelectionMode (JFileChooser.FILES_ONLY);
        if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            cygwinTextField.setText(fc.getSelectedFile().getAbsolutePath());
        }
    }//GEN-LAST:event_cygwinButtonActionPerformed

    private void cvsExeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cvsExeButtonActionPerformed
        // Add your handling code here:
        JFileChooser fc = new JFileChooser ();
        fc.setFileSelectionMode (JFileChooser.FILES_ONLY);
        if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            cvsExeTextField.setText(fc.getSelectedFile().getAbsolutePath());
        }
    }//GEN-LAST:event_cvsExeButtonActionPerformed

    private void cygwinCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cygwinCheckBoxActionPerformed
        // Add your handling code here:
        boolean isShell = cygwinCheckBox.isSelected();
        cygwinLabel.setEnabled(isShell);
        cygwinTextField.setEnabled(isShell);
        cygwinButton.setEnabled(isShell);
    }//GEN-LAST:event_cygwinCheckBoxActionPerformed

    private void repositoryButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_repositoryButtonActionPerformed
        // Add your handling code here:
        ChooseDirDialog chooseDir = new ChooseDirDialog(new JFrame(), new File(repositoryTextField.getText ()));
        VcsUtilities.centerWindow (chooseDir);
        HelpCtx.setHelpIDString (chooseDir.getRootPane (), CvsCustomizer.class.getName ());
        chooseDir.show();
        String selected=chooseDir.getSelectedDir();
        if (selected == null) {
            //D.deb("no directory selected"); // NOI18N
            return ;
        }
        File dir = new File(selected);
        if (!dir.isDirectory()) {
            //E.err("not directory "+dir); // NOI18N
            return ;
        }
        repositoryTextField.setText(selected);
    }//GEN-LAST:event_repositoryButtonActionPerformed

    private void extRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_extRadioButtonActionPerformed
        // Add your handling code here:
        updateEnabledComponents ();
    }//GEN-LAST:event_extRadioButtonActionPerformed

    private void pserverRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pserverRadioButtonActionPerformed
        // Add your handling code here:
        updateEnabledComponents ();
    }//GEN-LAST:event_pserverRadioButtonActionPerformed

    private void serverRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_serverRadioButtonActionPerformed
        // Add your handling code here:
        updateEnabledComponents ();
    }//GEN-LAST:event_serverRadioButtonActionPerformed

    private void localRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_localRadioButtonActionPerformed
        // Add your handling code here:
        updateEnabledComponents ();
    }//GEN-LAST:event_localRadioButtonActionPerformed

    private void btnRelMountActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRelMountActionPerformed
        // Add your handling code here:
        String work = rootDirTextField.getText();

        String dir = work + File.separator + txRelMount.getText();
//        String dir = work;
        
        RelativeMountDialog mountDlg = new RelativeMountDialog();
        mountDlg.setDir(work, txRelMount.getText());
        java.awt.Dialog dlg = TopManager.getDefault().createDialog(mountDlg);
        //VcsUtilities.centerWindow (mountDlg);
        //HelpCtx.setHelpIDString (dlg.getRootPane (), CvsCustomizer.class.getName ());
        dlg.setVisible(true);
        String selected = mountDlg.getRelMount();
        if (selected != null) {
            txRelMount.setText(selected);
        }
    }//GEN-LAST:event_btnRelMountActionPerformed

    private void browseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseButtonActionPerformed
        // Add your handling code here:
        ChooseDirDialog chooseDir = new ChooseDirDialog(new JFrame(), new File(rootDirTextField.getText ()));
        VcsUtilities.centerWindow (chooseDir);
        HelpCtx.setHelpIDString (chooseDir.getRootPane (), CvsCustomizer.class.getName ());
        chooseDir.show();
        String selected=chooseDir.getSelectedDir();
        if (selected == null) {
            //D.deb("no directory selected"); // NOI18N
            return ;
        }
        rootDirTextField.setText(selected);
        fillTextFields();
    }//GEN-LAST:event_browseButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JPanel connectionPanel;
    private javax.swing.JLabel rootDirLabel;
    private javax.swing.JTextField rootDirTextField;
    private javax.swing.JButton browseButton;
    private javax.swing.JLabel lbRelMount;
    private javax.swing.JTextField txRelMount;
    private javax.swing.JButton btnRelMount;
    private javax.swing.JRadioButton localRadioButton;
    private javax.swing.JRadioButton serverRadioButton;
    private javax.swing.JRadioButton pserverRadioButton;
    private javax.swing.JRadioButton extRadioButton;
    private javax.swing.JLabel serverLabel;
    private javax.swing.JTextField serverTextField;
    private javax.swing.JLabel userNameLabel;
    private javax.swing.JTextField userNameTextField;
    private javax.swing.JLabel repositoryLabel;
    private javax.swing.JTextField repositoryTextField;
    private javax.swing.JButton repositoryButton;
    private javax.swing.JPanel settingsPanel;
    private javax.swing.JLabel cvsExeLabel;
    private javax.swing.JTextField cvsExeTextField;
    private javax.swing.JButton cvsExeButton;
    private javax.swing.JCheckBox cygwinCheckBox;
    private javax.swing.JLabel cygwinLabel;
    private javax.swing.JTextField cygwinTextField;
    private javax.swing.JButton cygwinButton;
    private javax.swing.JPanel advancedPanel;
    private javax.swing.JCheckBox advancedOptCheckBox;
    private javax.swing.JCheckBox debugCheckBox;
    private javax.swing.JCheckBox callEditCheckBox;
    private javax.swing.JCheckBox offLineCheckBox;
    // End of variables declaration//GEN-END:variables

    private String lastEnteredRootDir;
    
    private ActionListener getActionListener() {
        return new ActionListener() {
            public void actionPerformed(final ActionEvent ev) {
                if (ev.getID() == ActionEvent.ACTION_PERFORMED) {
                    if (NotifyDescriptor.OK_OPTION.equals(ev.getSource())) {
                        new Thread(new Runnable() { // Do not use RequestProcessor, a deadlock will occure
                            public void run() {
                                if (testValidInput()) {
                                    if (closeListener != null) {
                                        javax.swing.SwingUtilities.invokeLater(new Runnable() {
                                            public void run() {
                                                closeListener.actionPerformed(ev);
                                                closeListener = null;
                                            }
                                        });
                                    }
                                }
                            }
                        }).start();
                    }
                }
            }
        };
    }
    
    private void setCloseListener(ActionListener closeListener) {
        this.closeListener = closeListener;
    }
    
    private boolean testValidInput() {
        applyChanges();
        if (!isLoggedIn()) {
            if (loginCancelled) {
                //setVisible(true);
                return false;
            } else {
                fileSystem.setOffLine(true); // set offline mode
            }
        }
        fileSystem.getCacheProvider().refreshDirFromDiskCache(fileSystem.getFile(""));
        //org.openide.TopManager.getDefault ().getRepository ().addFileSystem (fileSystem);
        return true;
    }
    
    /** for pservers checks if the user is logged in.
     */
    private boolean isLoggedIn() {
        if (pserverRadioButton.isSelected()) {
            CVSPasswd passFile = new CVSPasswd(fileSystem.getCygwinPath());  // this way or has to be rewritten to suit all OS.
            passFile.loadPassFile();
            String connectStr = ":pserver:" + userNameTextField.getText().trim()
                                + "@" + serverTextField.getText().trim() 
                                + ":" + repositoryTextField.getText().trim();
            final LoginDialog login = LoginDialog.createDialog(fileSystem);
            login.setPserverName(serverTextField.getText().trim());
            login.setConnectString(connectStr);
            final PasswdEntry entry = passFile.find(connectStr);
            try {
                SwingUtilities.invokeAndWait(new Runnable () {
                    public void run() {
                        if (entry == null) {
                            login.show();
                        } else {
                            login.showLogging();
                        }
                    }
                });
            } catch (InterruptedException intExc) {
                TopManager.getDefault().notifyException(intExc);
            } catch (java.lang.reflect.InvocationTargetException itExc) {
                TopManager.getDefault().notifyException(itExc);
            }
            loginCancelled = !login.isOffline();
            return login.isLoggedIn();
        }
        return true;  // is not pserver, -> always ok.
    }    
    
    private void removeEnterFromKeymap() {
        VcsUtilities.removeEnterFromKeymap(userNameTextField);
        VcsUtilities.removeEnterFromKeymap(serverTextField);
        VcsUtilities.removeEnterFromKeymap(repositoryTextField);
        VcsUtilities.removeEnterFromKeymap(rootDirTextField);
        VcsUtilities.removeEnterFromKeymap(cvsExeTextField);
        VcsUtilities.removeEnterFromKeymap(cygwinTextField);
    }

    private void updateEnabledComponents () {
        if(localRadioButton.isSelected ()) {
            userNameTextField.setEnabled (false);
            serverTextField.setEnabled (false);
            userNameLabel.setEnabled (false);
            repositoryButton.setEnabled (true);
            serverLabel.setEnabled (false);
        } else {
            userNameTextField.setEnabled (true);
            serverTextField.setEnabled (true);
            userNameLabel.setEnabled (true);
            repositoryButton.setEnabled (false);
            serverLabel.setEnabled (true);
        }
        //String osName = System.getProperty("os.name");
        if (org.openide.util.Utilities.isWindows()) { // NOI18N
            cygwinCheckBox.setEnabled (true);
            if (cygwinCheckBox.isSelected ()) {
                cygwinLabel.setEnabled (true);
                cygwinTextField.setEnabled (true);
                cygwinButton.setEnabled (true);
            } else {
                cygwinLabel.setEnabled (false);
                cygwinTextField.setEnabled (false);
                cygwinButton.setEnabled (false);
            }
        } else {
            cygwinCheckBox.setVisible (false);
            cygwinLabel.setVisible (false);
            cygwinTextField.setVisible (false);
            cygwinButton.setVisible (false);
        }
    }

    private void fillTextFields() {
        BufferedReader buff = null;
        String dirName = rootDirTextField.getText();
        if (dirName.equals(this.lastEnteredRootDir)) return; // user left textfield but didn't do any changes
        lastEnteredRootDir = dirName;
        dirName = dirName + File.separator + "CVS" + File.separator + "Root";
        File dirFile = new File(dirName);
        if (dirFile.exists()) {
            try {
                buff = new BufferedReader(new InputStreamReader(new FileInputStream(dirFile.getAbsolutePath())));
                String line = buff.readLine();
                if (line.startsWith(":")) {
                    line = line.substring(1);
                    StringTokenizer token = new StringTokenizer(line, ":",false);
                    if (token.hasMoreTokens()) {
                        String type = token.nextToken();
                        if (type.equals(CvsCustomizer.CVS_LOCAL))  {
                            localRadioButton.setSelected(true);
                            updateEnabledComponents ();
                            repositoryTextField.setText(line.substring(type.length() + 1));
                        } else {   //some kind of server..
                            if (type.equals(CvsCustomizer.CVS_SERVER)) serverRadioButton.setSelected(true);
                            if (type.equals(CvsCustomizer.CVS_PSERVER)) pserverRadioButton.setSelected(true);
                            if (type.equals(CvsCustomizer.CVS_EXT)) extRadioButton.setSelected(true);
                            updateEnabledComponents ();
                            if (token.hasMoreTokens()) {
                                String userServer =  token.nextToken();
                                this.userNameTextField.setText(userServer.substring(0,userServer.indexOf('@')));
                                this.serverTextField.setText(userServer.substring(userServer.indexOf('@') + 1));
                            }
                            if (token.hasMoreTokens())  repositoryTextField.setText(token.nextToken());
                        }
                    }
                }
            } catch (IOException exc) { //doesn't matter - nothing will be filled in
            } finally {
                if (buff != null) {
                    try {
                        buff.close();
                    } catch (IOException eIO) {}
                }
            }
        }
    }

    private void setObject(CvsFileSystem fileSystem) {
        this.fileSystem = fileSystem;
        String st = fileSystem.getCvsServerType ();
        //lastServerType = st;
        if(st.equals (fileSystem.CVS_SERVER_LOCAL)) localRadioButton.setSelected (true);
        else if (st.equals (fileSystem.CVS_SERVER_SERVER)) serverRadioButton.setSelected (true);
        else if (st.equals (fileSystem.CVS_SERVER_PSERVER)) pserverRadioButton.setSelected (true);
        else if (st.equals (fileSystem.CVS_SERVER_EXT)) extRadioButton.setSelected(true);
        
        //lastUserName = fileSystem.getCvsUserName ();
        userNameTextField.setText(fileSystem.getCvsUserName ());
        
        //lastServerName = fileSystem.getCvsServer ();
        serverTextField.setText(fileSystem.getCvsServer ());
        
        //lastRepository = fileSystem.getCvsRoot();
        repositoryTextField.setText(fileSystem.getCvsRoot());
        
        String relMount = fileSystem.getRelativeMountPoint();
        txRelMount.setText(relMount);
        lastRelMount = relMount;
        
        String rootDir = VcsFileSystem.substractRootDir(fileSystem.getRootDirectory().toString(), relMount);
        rootDirTextField.setText(rootDir);
        lastRootDir = rootDir;
        
        //lastCvsExe = fileSystem.getCvsExePath ();
        cvsExeTextField.setText(fileSystem.getCvsExePath ());
        
        boolean useUnixShell = fileSystem.isUseUnixShell();
        cygwinCheckBox.setSelected(useUnixShell);
        String cygwinPath = fileSystem.getCygwinPath ();
        cygwinTextField.setText(cygwinPath);
        
        //        lastRefresh = ""+getRefreshInSecs (); // NOI18N
        //        refreshTextField.setText (lastRefresh);
        
        updateEnabledComponents();
        //fileSystem.setCvsRoot(lastRepository);
        
        advancedOptCheckBox.setSelected(fileSystem.isExpertMode());
        callEditCheckBox.setSelected(fileSystem.isCallEditFilesOn());
        //checkMsgTemplCheckBox.setSelected(fileSystem.isExecutePreCommit());
        debugCheckBox.setSelected(fileSystem.getDebug());
        //lockCheckBox.setSelected(fileSystem.isLockFilesOn());
        //lockPromptCheckBox.setSelected(fileSystem.isPromptForLockOn());
        //lockPromptCheckBox.setEnabled(fileSystem.isLockFilesOn());
        offLineCheckBox.setSelected(fileSystem.isOffLine());
    }
    
    private void applyChanges() {
        if (fileSystem != null) {
            if (localRadioButton.isSelected()) {
                fileSystem.setCvsServerType (fileSystem.CVS_SERVER_LOCAL);
            } else if (serverRadioButton.isSelected()) {
                fileSystem.setCvsServerType (fileSystem.CVS_SERVER_SERVER);
            } else if (pserverRadioButton.isSelected()) {
                fileSystem.setCvsServerType (fileSystem.CVS_SERVER_PSERVER);
            } else if (extRadioButton.isSelected()) {
                fileSystem.setCvsServerType (fileSystem.CVS_SERVER_EXT);
            }
            fileSystem.setCvsUserName (userNameTextField.getText());
            fileSystem.setCvsServer(serverTextField.getText());
            fileSystem.setCvsRoot(repositoryTextField.getText());
            String value = rootDirTextField.getText();
            if (!value.equals(lastRootDir)) {
                //fileSystem.setRootDirectory(new File(value));
                rootDirChanged(value, true);
                try {
                    fileSystem.setRelativeMountPoint(""); // we need to do that because the working dir changed
                    fileSystem.setCvsModule(""); // we need to do that because the working dir changed
                } catch (java.beans.PropertyVetoException pvexc) {
                    TopManager.getDefault().notifyException(pvexc);
                } catch (java.io.IOException ioexc) {
                    TopManager.getDefault().notifyException(ioexc);
                }
                lastRootDir = value;
            }
            
            value = txRelMount.getText();
            if (!value.equals(lastRelMount)) {
                lastRelMount = value;
                try {
                    fileSystem.setRelativeMountPoint(value);
                    fileSystem.setCvsModule (value);
                } catch (java.beans.PropertyVetoException pvexc) {
                    TopManager.getDefault().notifyException(pvexc);
                } catch (java.io.IOException ioexc) {
                    TopManager.getDefault().notifyException(ioexc);
                }
                //rootDirChanged(customizerPanel.rootDirTextField.getText(), false);
            }
            
            value = cvsExeTextField.getText();
            fileSystem.setCvsExePath (value);
            fileSystem.setUseUnixShell(cygwinCheckBox.isSelected());
            value = cygwinTextField.getText();
            fileSystem.setCygwinPath(value);
            //fileSystem.setPromptForLockOn(lockPromptCheckBox.isSelected());
            //fileSystem.setLockFilesOn(lockCheckBox.isSelected());
            fileSystem.setDebug(debugCheckBox.isSelected());
            //fileSystem.setExecutePreCommit(checkMsgTemplCheckBox.isSelected());
            fileSystem.setCallEditFilesOn(callEditCheckBox.isSelected());
            fileSystem.setExpertMode(advancedOptCheckBox.isSelected());
            fileSystem.setOffLine(offLineCheckBox.isSelected());
        }
    }
    
    /**
     * Change the root directory of the filesystem. Get the directory from the text field
     * and perform the change in the filesystem. If the root directory does not exists and
     * <code>info</code> is <code>true</code>, nothing is changed.
     * @param info whether to inform the user of nonexistent root directory.
     */
    private void rootDirChanged (String selected, boolean info) {
        // root dir set by hand
        //String selected = rootDirTextField.getText ();
            /*
            String module = moduleTextField.getText ();
            String moduleDir = module.equals ("") ? selected : selected + java.io.File.separator + module; // NOI18N
            File dir=new File(moduleDir);
             */
        File root = new File(selected);
        if( info && !root.isDirectory() ){
            //E.err("not directory "+root); // NOI18N
            final String badDir = root.toString();
            TopManager.getDefault ().notify (new NotifyDescriptor.Message(MessageFormat.format (org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.notDirectory"), new Object[] { badDir } )));
            return ;
        }
        try {
            fileSystem.setRootDirectory(root);
            //rootDirTextField.setText(selected);
        }
        catch (PropertyVetoException veto){
            fileSystem.debug(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.canNotChangeWD"));
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    //if (isRootNotSetDlg) {
                    //    isRootNotSetDlg = false;
                        TopManager.getDefault().notify(new NotifyDescriptor.Message(org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.canNotChangeWD")));
                    //    isRootNotSetDlg = true;
                    //}
                }
            });
            //E.err(veto,"setRootDirectory() failed"); // NOI18N
        }
        catch (IOException e){
            //E.err(e,"setRootDirectory() failed"); // NOI18N
            final String badDir = root.toString();
            TopManager.getDefault ().notify (new NotifyDescriptor.Message(MessageFormat.format (org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.cannotSetDirectory"), new Object[] { badDir } )));
        }
    }

    public static class Dialog extends DialogDescriptor implements Customizer {
        
        private CvsCustomizer customizerPanel = null;
        private CvsFileSystem fileSystem = null;

        public Dialog() {
            this(new CvsCustomizer(),
                 org.openide.util.NbBundle.getBundle(CvsCustomizer.class).getString("CvsCustomizer.title"));
        }
        
        public Dialog(CvsCustomizer innerPane, String title) {
            super(innerPane, title, true, innerPane.getActionListener());
            customizerPanel = innerPane;
            setClosingOptions(new Object[] { NotifyDescriptor.OK_OPTION, NotifyDescriptor.CANCEL_OPTION });
        }
        
        public void setCloseListener(ActionListener closeListener) {
            this.customizerPanel.setCloseListener(closeListener);
        }
    
        public void setObject(final Object obj) {
            this.fileSystem = (CvsFileSystem) obj;

            customizerPanel.setObject(fileSystem);
            /*
            setButtonListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent ev) {
                    if (ev.getID() == ActionEvent.ACTION_PERFORMED) {
                        if (NotifyDescriptor.OK_OPTION.equals(ev.getSource())) {
                            applyChanges();
                        }
                    }
                }
            });
             */
        }
        
    }
}
