/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline.compatinstall;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;

import org.openide.TopManager;
import org.openide.filesystems.FileObject;
import org.openide.modules.ModuleInstall;
import org.openide.modules.ModuleInfo;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;

/**
 *
 * @author  Martin Entlicher
 */
public class CvsModuleInstall extends ModuleInstall {
    
    private static final String VCS_GENERIC_CODE_NAME_BASE = "org.netbeans.modules.vcs.advanced"; // NOI18N
    private static final String VCS_CVS_COMPAT_MODULE_FILE = "org-netbeans-modules-vcs-cmdline-compat.xml"; // NOI18N
    private static final String VCS_CVS_COMPAT_CODE_NAME_BASE = "org.netbeans.modules.vcs.cmdline.compat"; // NOI18N
    private static final String PARAM_ENABLED = "<param name=\"enabled\">"; // NOI18N

    static final long serialVersionUID =-3513549434587273290L;
    /** Creates new CvsModuleInstall */
    public CvsModuleInstall() {
    }

    public void validate() throws IllegalStateException {
        if (isVcsCvsCompatInstalled()) {
            setEnabledCompat(false);
            waitForVcsCvsCompatToUninstall();
        }
    }
    
    /*
    public void installed() {
        setEnabledCompat(false);
        restored();
    }
     */
    
    public void uninstalled() {
        if (isVcsGenericInstalled()) {
            org.openide.util.RequestProcessor.postRequest(new Runnable() {
                public void run() {
                    setEnabledCompat(true);
                }
            });
        }
    }
    
    private boolean isVcsGenericInstalled() {
        Lookup.Result modules = Lookup.getDefault().lookup(new Lookup.Template(ModuleInfo.class));
        Collection moduleInfos = modules.allInstances();
        for (Iterator it = moduleInfos.iterator(); it.hasNext(); ) {
            ModuleInfo info = (ModuleInfo) it.next();
            if (VCS_GENERIC_CODE_NAME_BASE.equals(info.getCodeNameBase())) {
                return info.isEnabled();
            }
        }
        return false;
    }
    
    private boolean isVcsCvsCompatInstalled() {
        Lookup.Result modules = Lookup.getDefault().lookup(new Lookup.Template(ModuleInfo.class));
        Collection moduleInfos = modules.allInstances();
        for (Iterator it = moduleInfos.iterator(); it.hasNext(); ) {
            ModuleInfo info = (ModuleInfo) it.next();
            if (VCS_CVS_COMPAT_CODE_NAME_BASE.equals(info.getCodeNameBase())) {
                return info.isEnabled();
            }
        }
        return false;
    }
    
    private synchronized void reCheckVcsCvsCompatInstall() {
        notify();
    }
    
    private synchronized void waitForVcsCvsCompatToUninstall() {
        Lookup.Result modules = Lookup.getDefault().lookup(new Lookup.Template(ModuleInfo.class));
        modules.addLookupListener(new LookupListener() {
            public void resultChanged(LookupEvent evt) {
                reCheckVcsCvsCompatInstall();
            }
        });
        while (!isVcsCvsCompatInstalled()) {
            try {
                wait();
            } catch (InterruptedException intrExc) {}
        }
    }
    
    private void setEnabledCompat(boolean enabled) {
        FileObject modules = TopManager.getDefault().getRepository().getDefaultFileSystem().findResource("Modules"); // NOI18N
        FileObject compatModule = modules.getFileObject(VCS_CVS_COMPAT_MODULE_FILE);
        if (compatModule == null) return ;
        BufferedReader reader = null;
        boolean success = true;
        StringBuffer moduleXmlContent = new StringBuffer();
        try {
            reader = new BufferedReader(new InputStreamReader(compatModule.getInputStream()));
            String line;
            while ((line = reader.readLine()) != null) {
                moduleXmlContent.append(line+"\n");
            }
        } catch (IOException ioExc) {
            success = false;
        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException exc) {}
        }
        if (!success) return ;
        String moduleXmlContentStr = moduleXmlContent.toString();
        int index = moduleXmlContentStr.indexOf(PARAM_ENABLED);
        if (index < 0) return ;
        index += PARAM_ENABLED.length();
        String moduleXmlContentStr1 = moduleXmlContentStr.substring(0, index);
        String moduleXmlContentStr2 = moduleXmlContentStr.substring(index);
        if (moduleXmlContentStr2.startsWith("true")) moduleXmlContentStr2 = moduleXmlContentStr2.substring(4); // NOI18N
        else if (moduleXmlContentStr2.startsWith("false")) moduleXmlContentStr2 = moduleXmlContentStr2.substring(5); // NOI18N
        else return ; // unrecognized item
        moduleXmlContentStr = moduleXmlContentStr1 + (enabled ? "true" : "false") + moduleXmlContentStr2;
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new OutputStreamWriter(compatModule.getOutputStream(compatModule.lock())));
            writer.write(moduleXmlContentStr);
        } catch (IOException ioExc) {
        } finally {
            try {
                if (writer != null) {
                    writer.close();
                }
            } catch (IOException exc) {}
        }
    }
}
