/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline.passwd;

import java.util.*;
/** This class stores info about one cvs repository. It's read by {@link org.netbeans.modules.vcs.cmdline.passwd.CVSPasswd }
 *
 * @author Milos Kleint
 * @version 1.0
 */
    public class PasswdEntry {
      String entry = null;
      String passwd = null;
      String type = null;
      String user = null;
      String server = null;
      String root = null;
      String main = null;

      public PasswdEntry() {
      }

      public boolean setEntry(String ent) {
        entry = ent;
        int spacePos = entry.indexOf(' ');
        if (spacePos < 0) {return false;} // corrupted line.
        main = entry.substring(0,spacePos);
        if (main.startsWith(":")) { main = main.substring(1);}
        passwd = entry.substring(spacePos + 1);
        StringTokenizer token = new StringTokenizer(main, ":",false);
        if (token.hasMoreTokens()) {
          type = token.nextToken();
        } else return false;
        if (token.hasMoreTokens()) {
          String userServer = token.nextToken();  
          int ind3 = userServer.indexOf('@');
          if (ind3 < 0) return false; // corrupted line
          user = userServer.substring(0, ind3);  //user name
          server = userServer.substring(ind3 + 1); // server name
        } else return false;
        if (token.hasMoreTokens()) {            
            final StringBuffer temp = new StringBuffer("");
            while (token.hasMoreTokens()) {
                temp.append(token.nextToken());
                if (token.hasMoreTokens())
                    temp.append(':');
            }                
            root = temp.toString();            
        } else return false;
        return true;
     }

      public String getEntry(boolean withPasswd) {
        String entry = ":" + type + ":" + user + "@" + server + ":" + root;
        if (withPasswd) {entry = entry + " " + passwd;}
        return entry;
      }
      public boolean matchToCurrent(String curr){
       String s = getEntry(false);
//       D("matchToCurrent:" + curr + " + " + s);
       if (s.equalsIgnoreCase(curr)) { return true; }
       return false;  
      }

      public String getType() {
        return type;
      }
/*      public void setType(String tp) {
        type = tp;
      }
*/      
      public String getRoot() {
        return root;
      } 

      public String getUser() {
        return user;
      } 

      public String getPasswd() { // already scrambled
        return passwd;
      } 
/*      public void setRoot(String rootDir) {
        root = rootDir;
      } 

      public void setUser(String userName) {
        user= userName;
      } 

      public void setPasswd(String pass) {
        passwd = pass;
      } 
*/      
      
      public String toString() {
        return getEntry(true);
      }

      public String getServer() {
        return server;
      } 
/*
      public void setServer(String serv) {
        server = serv;
      } 
*/

/** Method used to compose the authentification string that is send to the server in the beginning of the communication.
 * For details see CVS Client/Server protocol specification.
 * @return the authentification string
 */
      public String getAuthString() {
        return (getRoot() + "\n" + getUser() + "\n" + getPasswd() + "\n");
      }

      private void D(String debug) {
   //     System.out.println("PasswdEntry(): "+debug);
      }

  }